##############################################################################
# For copyright and license notices, see __manifest__.py file in module root
# directory
##############################################################################
from odoo import models, fields, api, _
from odoo.exceptions import ValidationError

class AccountMove(models.Model):
    _inherit = "account.move"

    computed_currency_rate = fields.Float(
        compute='_compute_currency_rate',
        string='Currency Rate (preview)',
        digits=(16, 6),
    )
    l10n_ar_currency_rate = fields.Float(
        compute='_compute_l10n_ar_currency_rate', store=True)

    vat_total = fields.Float(
        compute='_compute_vat_total',
        string='VAT Total',
        store=True
    )

    other_taxes_total = fields.Float(
        compute='_compute_other_taxes_total',
        string='Other Taxes Total',
        store=True
    )

    @api.depends('invoice_line_ids', 'invoice_line_ids.tax_ids')
    def _compute_vat_total(self):
        for rec in self:
            rec.vat_total = 0
            if rec.tax_totals:
                if rec.tax_totals.get('subtotals_order'):
                    taxes = rec.tax_totals.get('groups_by_subtotal').get(rec.tax_totals.get('subtotals_order')[0])
                    if taxes:
                        for tax in taxes:
                            tax_vat_code = rec.env['account.tax.group'].browse(tax.get('tax_group_id')).l10n_ar_vat_afip_code
                            if tax_vat_code:
                                rec.vat_total += tax.get('tax_group_amount')

    @api.depends('invoice_line_ids', 'invoice_line_ids.tax_ids')
    def _compute_other_taxes_total(self):
        for rec in self:
            rec.other_taxes_total = 0
            if rec.tax_totals:
                if rec.tax_totals.get('subtotals_order'):
                    taxes = rec.tax_totals.get('groups_by_subtotal').get(rec.tax_totals.get('subtotals_order')[0])
                    if taxes:
                        for tax in taxes:
                            tax_tribute_code = rec.env['account.tax.group'].browse(tax.get('tax_group_id')).l10n_ar_tribute_afip_code
                            if tax_tribute_code:
                                rec.other_taxes_total += tax.get('tax_group_amount')

    @api.onchange('invoice_line_ids', 'invoice_line_ids.tax_ids', 'invoice_line_ids.price_unit', 'invoice_line_ids.quantity')
    def _onchange_vat_total(self):
        for rec in self:
            rec.vat_total = 0
            if rec.tax_totals:
                if rec.tax_totals.get('subtotals_order'):
                    taxes = rec.tax_totals.get('groups_by_subtotal').get(rec.tax_totals.get('subtotals_order')[0])
                    if taxes:
                        for tax in taxes:
                            tax_vat_code = rec.env['account.tax.group'].browse(tax.get('tax_group_id')).l10n_ar_vat_afip_code
                            if tax_vat_code:
                                rec.vat_total += tax.get('tax_group_amount')

    @api.onchange('invoice_line_ids', 'invoice_line_ids.tax_ids', 'invoice_line_ids.price_unit', 'invoice_line_ids.quantity')
    def _onchange_other_taxes_total(self):
        for rec in self:
            rec.other_taxes_total = 0
            if rec.tax_totals:
                if rec.tax_totals.get('subtotals_order'):
                    taxes = rec.tax_totals.get('groups_by_subtotal').get(rec.tax_totals.get('subtotals_order')[0])
                    if taxes:
                        for tax in taxes:
                            tax_tribute_code = rec.env['account.tax.group'].browse(tax.get('tax_group_id')).l10n_ar_tribute_afip_code
                            if tax_tribute_code:
                                rec.other_taxes_total += tax.get('tax_group_amount')


    @api.depends('reversed_entry_id')
    def _compute_l10n_ar_currency_rate(self):
        """ If it's a credit note on foreign currency and foreing currency is the same as original credit note, then
        we use original invoice rate """
        ar_reversed_other_currency = self.filtered(
            lambda x: x.is_invoice() and x.reversed_entry_id and
            x.company_id.country_id == self.env.ref('base.ar') and
            x.currency_id != x.company_id.currency_id and
            x.reversed_entry_id.currency_id == x.currency_id)
        self.filtered(lambda x: x.move_type == 'entry').l10n_ar_currency_rate = False
        for rec in ar_reversed_other_currency:
            rec.l10n_ar_currency_rate = rec.reversed_entry_id.l10n_ar_currency_rate

    @api.depends('currency_id', 'company_id', 'date', 'invoice_date')
    def _compute_currency_rate(self):
        need_currency_rate = self.filtered(lambda x: x.currency_id and x.company_id and (x.currency_id != x.company_id.currency_id))
        remaining = self - need_currency_rate
        for rec in need_currency_rate:
            if rec.l10n_ar_currency_rate:
                rec.computed_currency_rate = rec.l10n_ar_currency_rate
            else:
                rec.computed_currency_rate = rec.currency_id._convert(
                    1.0, rec.company_id.currency_id, rec.company_id,
                    # para previsualizar lo que sería la tasa usamos la fecha contable, sino usamos la fecha al día de hoy
                    # la fecha contable en facturas de venta realmente está seteada cuando el invoice_date está activo o
                    # posterior a la validación de la factura es por eso que comparamos invoice_date
                    date=rec.date if rec.invoice_date else fields.Date.context_today(rec),
                    round=False)
        remaining.computed_currency_rate = 1.0

    @api.constrains('ref', 'move_type', 'partner_id', 'journal_id', 'invoice_date')
    def _check_duplicate_supplier_reference(self):
        """ We make reference only unique if you are not using documents.
        Documents already guarantee to not encode twice same vendor bill """
        '''
        for move in self.filtered(lambda x: not x.l10n_latam_use_documents):
            if self.search_count([('id', '!=', move.id)]) > 0:
                raise ValidationError(_('Duplicate supplier reference found'))
        '''
        return True


    def _get_name_invoice_report(self):
        """Use always argentinian like report (regardless use documents)"""
        self.ensure_one()
        if self.company_id.country_id.code == 'AR':
            return 'l10n_ar.report_invoice_document'
        return super()._get_name_invoice_report()

    def _get_l10n_latam_documents_domain(self):
        self.ensure_one()
        # TODO: add prefix "_l10n_ar" to method use_specific_document_types
        if self.company_id.country_id == self.env.ref('base.ar') and self.journal_id.use_specific_document_types():
            return [
                ('id', 'in', self.journal_id.l10n_ar_document_type_ids.ids),
                '|', ('code', 'in', self._get_l10n_ar_codes_used_for_inv_and_ref()),
                ('internal_type', 'in', ['credit_note'] if self.move_type in ['out_refund', 'in_refund'] else ['invoice', 'debit_note']),
            ]
        return super()._get_l10n_latam_documents_domain()

    def _l10n_ar_include_vat(self):
        self.ensure_one()
        if not self.l10n_latam_use_documents:
            discriminate_taxes = self.journal_id.discriminate_taxes
            if discriminate_taxes == 'yes':
                return False
            elif discriminate_taxes == 'no':
                return True
            else:
                return not (
                    self.company_id.l10n_ar_company_requires_vat and
                    self.partner_id.l10n_ar_afip_responsibility_type_id.code in ['1'] or False)
        return self.l10n_latam_document_type_id.l10n_ar_letter in ['B', 'C', 'X', 'R']

    def _is_manual_document_number(self):
        res = super()._is_manual_document_number()
        # when issuer is supplier de numbering works opposite (supplier numerate invoices, customer encode bill)
        if self.country_code == 'AR' and self.journal_id._l10n_ar_journal_issuer_is_supplier():
            return not res
        return res
