"""This module contains the model for the check search functionality"""
from odoo import _, api, fields, models


class AccountPayment(models.Model):
    """Inherit the account.payment model to add the state_check field"""

    _inherit = "account.payment"

    state_check = fields.Selection(
        [("in_hand", "in_hand"), ("delivered", "delivered")],
        string="State Check",
        default="in_hand",
        help="The state of the check: 'in_hand' or 'delivered'",
    )

    def _prepare_move_line_default_vals(self, write_off_line_vals=None):
        """Add check name and operation on liquidity line"""
        write_off_line_vals = write_off_line_vals or {}

        # Determine the value of 'state_check' based on 'payment_type'
        state_check_value = (
            "delivered" if self.payment_type == "outbound" else "in_hand"
        )

        if self.payment_type == "outbound" and self.state_check == "in_hand":
            self.l10n_latam_check_id.write({"state_check": "delivered"})
        else:
            self.write({"state_check": "in_hand"})
        # Find all payments with the same check number and update their state_check to state_check_value
        # self.env['account.payment'].search([('check_number', '=', self.check_number)]).write({'state_check': state_check_value})

        # Continue with the logic in the original method
        return super(AccountPayment, self)._prepare_move_line_default_vals(
            write_off_line_vals
        )

    @api.constrains("check_number", "journal_id")
    def _constrains_check_number(self):
        """Rewrite the original method to rewrite query to allow check numbers with letters"""
        third_party_checks = self.filtered(
            lambda x: x.payment_method_line_id.code == "new_third_party_checks"
        )
        self = self - third_party_checks
        payment_checks = self.filtered("check_number")
        if not payment_checks:
            return
        self.env.flush_all()
        self.env.cr.execute(
            """
            SELECT payment.check_number, move.journal_id
              FROM account_payment payment
              JOIN account_move move ON move.id = payment.move_id
              JOIN account_journal journal ON journal.id = move.journal_id,
                   account_payment other_payment
              JOIN account_move other_move ON other_move.id = other_payment.move_id
             WHERE payment.check_number = other_payment.check_number
               AND move.journal_id = other_move.journal_id
               AND payment.id != other_payment.id
               AND payment.id IN %(ids)s
               AND move.state = 'posted'
               AND other_move.state = 'posted'
               AND payment.check_number IS NOT NULL
               AND other_payment.check_number IS NOT NULL
        """,
            {
                "ids": tuple(payment_checks.ids),
            },
        )

        res = self.env.cr.dictfetchall()
        if res:
            raise ValidationError(
                _(
                    "The following numbers are already used:\n%s",
                    "\n".join(
                        _(
                            "%(number)s in journal %(journal)s",
                            number=r["check_number"],
                            journal=self.env["account.journal"]
                            .browse(r["journal_id"])
                            .display_name,
                        )
                        for r in res
                    ),
                )
            )
