"""Wizard closing exercise"""
from odoo import _, fields, models
from odoo.exceptions import UserError
from odoo.tools import float_is_zero


class WzdClosingExcercise(models.TransientModel):
    """Wizard closing exercise"""

    _name = "nybble.closing_exercise"
    _description = "Wizard Closing Exercise"

    name_action = fields.Selection(
        [
            ("Pasaje a resultados", "Pasaje a resultados"),
            ("Cierre de ejercicio contable", "Cierre de ejercicio contable"),
        ],
        string="Action",
        required=True,
        default=False,
    )
    date_result = fields.Date("Date")

    def close_exercise(self):
        """Action closing exercise"""
        if self.name_action == "Pasaje a resultados":
            self.action_passage_to_results()
        else:
            self.action_accounting_closing()

    def action_passage_to_results(self):
        """Action passage to results"""
        company_id = self.env.user.company_id
        if company_id.fiscalyear_lock_date and self.date_result:
            if self.date_result > company_id.fiscalyear_lock_date:
                raise UserError(
                    _(
                        "The registration date must be on or before the fiscal year lock date"
                    )
                )
            # Income accounts
            income_accounts = self.env["account.account"].search(
                [
                    ("account_type", "=", "income"),
                    ("company_id", "=", company_id.id),
                    ("deprecated", "=", False),
                ]
            )
            # Expense accounts
            expense_accounts = self.env["account.account"].search(
                [
                    ("account_type", "=", "expense"),
                    ("company_id", "=", company_id.id),
                    ("deprecated", "=", False),
                ]
            )
            # Find the journal
            journal_id = self.env["account.journal"].search(
                [
                    ("type", "=", "general"),
                    ("journal_result", "=", True),
                    ("company_id", "=", company_id.id),
                ],
                limit=1,
            )
            if not journal_id:
                raise UserError(_("There is no journal configured as a result journal"))
            # Create accounting move
            move = self.env["account.move"].create(
                {
                    "date": self.date_result,
                    "journal_id": journal_id.id,
                    "ref": "Closing exercise entry - Passage to results",
                    "date_result": self.date_result,
                }
            )
            # Create accounting move lines for income accounts
            move_lines_income = []
            move_lines_expense = []
            move_lines_adjust = []
            move_lines = []
            balance = 0.0
            sum_income = 0.0
            debit = 0.0
            credit = 0.0
            for account in income_accounts:
                balance = 0.0
                debit = 0.0
                credit = 0.0
                move_lines_query = self.env["account.move.line"].search(
                    [
                        ("account_id", "=", account.id),
                        ("date", "<=", self.date_result),
                        ("parent_state", "=", "posted"),
                        ("company_id", "=", company_id.id),
                    ]
                )
                if move_lines_query:
                    for line in move_lines_query:
                        balance += line.balance
                    sum_income += balance
                    if balance > 0:
                        debit = 0.0
                        credit = balance
                    elif balance < 0:
                        debit = abs(balance)
                        credit = 0.0
                    if not float_is_zero(
                        balance,
                        precision_rounding=self.env.company.currency_id.rounding,
                    ):
                        move_line_income = {
                            "move_id": move.id,
                            "account_id": account.id,
                            "name": account.name,
                            "debit": debit,
                            "credit": credit,
                        }
                        move_lines_income.append(move_line_income)
            # Create accounting move lines for expense accounts
            balance = 0.0
            sum_expense = 0.0
            debit = 0.0
            credit = 0.0
            for account in expense_accounts:
                balance = 0.0
                debit = 0.0
                credit = 0.0
                move_lines_query = self.env["account.move.line"].search(
                    [
                        ("account_id", "=", account.id),
                        ("date", "<=", self.date_result),
                        ("parent_state", "=", "posted"),
                        ("company_id", "=", company_id.id),
                    ]
                )
                if move_lines_query:
                    for line in move_lines_query:
                        balance += line.balance
                    sum_expense += balance
                    if balance > 0:
                        debit = 0.0
                        credit = balance
                    elif balance < 0:
                        debit = abs(balance)
                        credit = 0.0
                    if not float_is_zero(
                        balance,
                        precision_rounding=self.env.company.currency_id.rounding,
                    ):
                        move_line_expense = {
                            "move_id": move.id,
                            "account_id": account.id,
                            "name": account.name,
                            "debit": debit,
                            "credit": credit,
                        }
                        move_lines_expense.append(move_line_expense)
            # Calculate debit and credit totals for income and expense accounts
            # Balance the accounting entry if needed
            if sum_income != sum_expense:
                # Calculate the difference
                diff_amount = abs(sum_income) - abs(sum_expense)
                # Find account result
                account_id = self.env["account.account"].search(
                    [("account_result", "=", True), ("company_id", "=", company_id.id)],
                    limit=1,
                )
                if not account_id:
                    raise UserError(
                        _("There is no account configured as a result account")
                    )
                # Determine on which side the adjustment should be made
                if not float_is_zero(
                    diff_amount,
                    precision_rounding=self.env.company.currency_id.rounding,
                ):
                    diff_amounts = diff_amount
                    if diff_amounts > 0:
                        # Debits total is greater than credits total, so an additional credit should be added
                        move_line_vals = {
                            "move_id": move.id,
                            "account_id": account_id.id,
                            "name": "Adjustment to balance the entry",
                            "debit": 0.0,
                            "credit": abs(diff_amounts),
                        }
                    else:
                        # Credits total is greater than debits total, so an additional debit should be added
                        move_line_vals = {
                            "move_id": move.id,
                            "account_id": account_id.id,
                            "name": "Adjustment to balance the entry",
                            "debit": abs(diff_amounts),
                            "credit": 0.0,
                        }
                    # Create accounting move lines for the adjustment
                    move_lines_adjust.append(move_line_vals)
            move_lines = move_lines_income + move_lines_expense + move_lines_adjust
            # Create accounting move lines for the adjustment
            self.env["account.move.line"].create(move_lines)
            move.action_post()
            move.sudo().update({"date": self.date_result})
        else:
            raise UserError(_("The company has no lock date configured"))

    def action_accounting_closing(self):
        """Action accounting closing"""
        company_id = self.env.user.company_id
        if self.date_result > company_id.fiscalyear_lock_date:
            raise UserError(
                _(
                    "The registration date must be on or before the fiscal year lock date"
                )
            )
        # Find the journal
        journal_id = self.env["account.journal"].search(
            [
                ("type", "=", "general"),
                ("journal_result_end", "=", True),
                ("company_id", "=", company_id.id),
            ],
            limit=1,
        )
        if not journal_id:
            raise UserError(_("There is no journal configured as a result journal end"))
        # Create accounting move
        move = self.env["account.move"].create(
            {
                "date": self.date_result,
                "journal_id": journal_id.id,
                "date_result": self.date_result,
                "ref": "Closing exercise entry - Accounting closing",
            }
        )
        # Find the accounts active
        active_accounts = self.env["account.account"].search(
            [
                (
                    "account_type",
                    "in",
                    ["asset_cash", "asset_fixed", "asset_current", "asset_receivable"],
                ),
                ("company_id", "=", company_id.id),
            ]
        )
        # Find the accounts pasive
        pasive_accounts = self.env["account.account"].search(
            [
                (
                    "account_type",
                    "in",
                    ["liability_payable", "liability_non_current", "liability_current"],
                ),
                ("company_id", "=", company_id.id),
            ]
        )
        # Find the accounts result PN
        pn_accounts = self.env["account.account"].search(
            [
                ("account_type", "in", ["equity", "equity_unaffected"]),
                ("company_id", "=", company_id.id),
            ]
        )
        move_lines_active = []
        move_lines_pasive = []
        move_lines_pn = []
        move_lines = []
        move_lines_adjust = []
        # Create accounting move lines for active accounts
        sum_active = 0.0
        for account in active_accounts:
            balance = 0.0
            debit = 0.0
            credit = 0.0
            move_lines_query = self.env["account.move.line"].search(
                [
                    ("account_id", "=", account.id),
                    ("date", "<=", self.date_result),
                    ("parent_state", "=", "posted"),
                    ("company_id", "=", company_id.id),
                ]
            )
            if move_lines_query:
                for line in move_lines_query:
                    balance += line.balance
                sum_active += balance
                if balance > 0:
                    credit = balance
                elif balance < 0:
                    debit = abs(balance)
                if not float_is_zero(
                    balance, precision_rounding=self.env.company.currency_id.rounding
                ):
                    move_line_active = {
                        "move_id": move.id,
                        "account_id": account.id,
                        "name": account.name,
                        "debit": debit,
                        "credit": credit,
                    }
                    move_lines_active.append(move_line_active)
        # Create accounting move lines for pasive accounts
        sum_pasive = 0.0
        for account in pasive_accounts:
            balance = 0.0
            debit = 0.0
            credit = 0.0
            move_lines_query = self.env["account.move.line"].search(
                [
                    ("account_id", "=", account.id),
                    ("date", "<=", self.date_result),
                    ("parent_state", "=", "posted"),
                    ("company_id", "=", company_id.id),
                ]
            )
            if move_lines_query:
                for line in move_lines_query:
                    balance += line.balance
                sum_pasive += balance
                if balance > 0:
                    credit = balance
                elif balance < 0:
                    debit = abs(balance)
                if not float_is_zero(
                    balance, precision_rounding=self.env.company.currency_id.rounding
                ):
                    move_line_pasive = {
                        "move_id": move.id,
                        "account_id": account.id,
                        "name": account.name,
                        "debit": debit,
                        "credit": credit,
                    }
                    move_lines_pasive.append(move_line_pasive)
        # Create accounting move lines for result PN accounts
        sum_pn = 0.0
        for account in pn_accounts:
            balance = 0.0
            debit = 0.0
            credit = 0.0
            move_lines_query = self.env["account.move.line"].search(
                [
                    ("account_id", "=", account.id),
                    ("date", "<=", self.date_result),
                    ("parent_state", "=", "posted"),
                    ("company_id", "=", company_id.id),
                ]
            )
            if move_lines_query:
                for line in move_lines_query:
                    balance += line.balance
                sum_pn += balance
                if balance > 0:
                    credit = balance
                elif balance < 0:
                    debit = abs(balance)
                if not float_is_zero(
                    balance, precision_rounding=self.env.company.currency_id.rounding
                ):
                    move_line_pn = {
                        "move_id": move.id,
                        "account_id": account.id,
                        "name": account.name,
                        "debit": debit,
                        "credit": credit,
                    }
                    move_lines_pn.append(move_line_pn)
        # Calculate debit and credit totals for active, pasive and result PN accounts
        # Balance the accounting entry if needed
        move_lines_adjust = []
        if sum_active != sum_pasive + sum_pn:
            # Calculate the difference
            diff_amount = abs(sum_active) - abs(sum_pasive + sum_pn)
            # Find account result
            account_id = self.env["account.account"].search(
                [("account_end_result", "=", True), ("company_id", "=", company_id.id)],
                limit=1,
            )
            if not account_id:
                raise UserError(
                    _("There is no account configured as a result account end")
                )
            # Determine on which side the adjustment should be made
            if not float_is_zero(
                diff_amount, precision_rounding=self.env.company.currency_id.rounding
            ):
                diff_amounts = diff_amount
                if diff_amounts > 0:
                    # Debits total is greater than credits total, so an additional credit should be added
                    move_line_vals = {
                        "move_id": move.id,
                        "account_id": account_id.id,
                        "name": "Adjustment to balance the entry",
                        "debit": 0.0,
                        "credit": abs(diff_amounts),
                    }
                else:
                    # Credits total is greater than debits total, so an additional debit should be added
                    move_line_vals = {
                        "move_id": move.id,
                        "account_id": account_id.id,
                        "name": "Adjustment to balance the entry",
                        "debit": abs(diff_amounts),
                        "credit": 0.0,
                    }
                # Create accounting move lines for the adjustment
                move_lines_adjust.append(move_line_vals)
            move_lines = (
                move_lines_active
                + move_lines_pasive
                + move_lines_pn
                + move_lines_adjust
            )
            # Create accounting move lines for the adjustment
        self.env["account.move.line"].create(move_lines)
        return True
