"""Customization of the account.move model."""
from odoo import api, fields, models


class AccountMoveLine(models.Model):
    """Customization of the account.move.line model."""

    _inherit = "account.move.line"

    price_unit2 = fields.Float(string="Cash Price", store=True, invisible=True)
    price_subtotal2 = fields.Monetary(
        string="Cash Subtotal",
        store=True,
        invisible=True,
        compute="_compute_cash_amounts",
    )
    price_total2 = fields.Monetary(
        string="Cash Total", store=True, invisible=True, compute="_compute_cash_amounts"
    )

    @api.depends("quantity", "price_unit2", "discount", "tax_ids")
    def _compute_cash_amounts(self):
        """Compute the cash amounts for the invoice line."""
        for line in self:
            price = line.price_unit2 * (1 - (line.discount or 0.0) / 100.0)
            taxes = line.tax_ids.compute_all(
                price,
                line.currency_id,
                line.quantity,
                product=line.product_id,
                partner=line.partner_id,
            )
            line.price_subtotal2 = taxes["total_excluded"]
            line.price_total2 = taxes["total_included"]


class AccountMove(models.Model):
    """Customization of the account.move model."""

    _inherit = "account.move"

    amount_untaxed2 = fields.Monetary(
        string="Cash Taxable Amount",
        invisible=True,
        store=True,
        compute="_compute_cash_amounts",
    )
    amount_total2 = fields.Monetary(
        string="Cash Total", invisible=True, store=True, compute="_compute_cash_amounts"
    )

    @api.depends("invoice_line_ids.price_subtotal2", "invoice_line_ids.price_total2")
    def _compute_cash_amounts(self):
        """Compute the cash amounts for the invoice."""
        for move in self:
            move.amount_untaxed2 = sum(move.invoice_line_ids.mapped("price_subtotal2"))
            move.amount_total2 = sum(move.invoice_line_ids.mapped("price_total2"))

    def action_post(self):
        """Post the invoice and calculate the cash prices."""
        result = super(AccountMove, self).action_post()
        self._calculate_cash_prices()
        return result

    def _calculate_cash_prices(self):
        """Calculate the cash prices for the invoice lines."""
        cash_pricelist = self.env["product.pricelist"].search(
            [("is_cash_list", "=", True)], limit=1
        )
        if not cash_pricelist:
            return
        for move in self.filtered(
            lambda m: m.move_type in ("out_invoice", "out_refund")
        ):
            for line in move.invoice_line_ids:
                product = line.product_id
                quantity = line.quantity
                date = move.date or fields.Date.today()
                uom = line.product_uom_id
                cash_price = cash_pricelist._get_product_price(
                    product, quantity, date=date, uom=uom
                )
                line.price_unit2 = cash_price
            move._compute_cash_amounts()
