"""Extend the account.payment model to update the billing.paid.ontime model when a payment group is posted."""
from odoo import models


class AccountPaymentGroup(models.Model):
    """Extend the account.payment.group model to update the billing.paid.ontime model when a payment group is posted."""

    _inherit = "account.payment.group"

    def post(self):
        """Post the payment group and update the billing.paid.ontime model."""
        res = super(AccountPaymentGroup, self).post()
        self._update_billing_paid_ontime()
        return res

    def _update_billing_paid_ontime(self):
        """Update the billing.paid.ontime model based on the payments in the payment group."""
        for payment_group in self:
            for invoice in payment_group.to_pay_move_line_ids.move_id:
                if (
                    invoice.move_type in ("out_invoice", "out_refund")
                    and invoice.state == "posted"
                ):
                    amount_paid = sum(payment_group.payment_ids.mapped("amount"))
                    for payment in payment_group.payment_ids:
                        self.env["billing.paid.ontime"].sudo().create(
                            {
                                "invoice_name": invoice.name,
                                "partner_id": invoice.partner_id.id,
                                "invoice_date": invoice.invoice_date,
                                "invoice_date_due": invoice.invoice_date_due,
                                "invoice_payment_term_id": invoice.invoice_payment_term_id.id,
                                "amount_total": invoice.amount_total,
                                "amount_untaxed": invoice.amount_untaxed,
                                "amount_total2": invoice.amount_total2
                                or invoice.amount_total,
                                "amount_untaxed2": invoice.amount_untaxed2
                                or invoice.amount_untaxed,
                                "team_id": invoice.team_id.id,
                                "invoice_user_id": invoice.invoice_user_id.id,
                                "currency_id": invoice.currency_id.id,
                                "company_id": invoice.company_id.id,
                                "name": payment_group.name,
                                "payment_date": payment_group.payment_date,
                                "payment_method_id": payment.payment_method_line_id.id,
                                "journal_id": payment.journal_id.id,
                                "payment_amount": amount_paid,
                            }
                        )

    def action_draft(self):
        """Set unlink the billing.paid.ontime model when the payment group is set to draft."""
        res = super(AccountPaymentGroup, self).action_draft()
        self._unlink_billing_paid_ontime()
        return res

    def _unlink_billing_paid_ontime(self):
        """Unlink the billing.paid.ontime model based on the payments in the payment group."""
        for payment_group in self:
            for invoice in payment_group.to_pay_move_line_ids.move_id:
                if (
                    invoice.move_type in ("out_invoice", "out_refund")
                    and invoice.state == "posted"
                ):
                    self.env["billing.paid.ontime"].search(
                        [
                            ("name", "=", payment_group.name),
                            ("company_id", "=", payment_group.company_id.id),
                        ]
                    ).sudo().unlink()


class AccountPayment(models.Model):
    """Extend the account.payment.group model to update the billing.paid.ontime model when a payment group is posted."""

    _inherit = "account.payment"

    def action_draft(self):
        """Set unlink the billing.paid.ontime model when the payment is set to draft."""
        res = super(AccountPayment, self).action_draft()
        self._unlink_billing_paid_ontime()
        return res

    def _unlink_billing_paid_ontime(self):
        """Unlink the billing.paid.ontime model based on the payment."""
        for payment in self:
            for invoice in payment.move_id:
                if (
                    invoice.move_type in ("out_invoice", "out_refund")
                    and invoice.state == "posted"
                ):
                    self.env["billing.paid.ontime"].search(
                        [
                            ("name", "=", payment.name),
                            ("company_id", "=", payment.company_id.id),
                        ]
                    ).sudo().unlink()
