"""Manage the model of the billing paid on time"""
import logging

from odoo import api, fields, models

_logger = logging.getLogger(__name__)


class BillingPaidOntime(models.Model):
    """Manage the model of the billing paid on time"""

    _name = "billing.paid.ontime"
    _description = "Billing Paid On Time"
    _rec_name = "invoice_name"

    invoice_name = fields.Char(
        string="Invoice Number",
        help="Reference number of the sales transaction",
        readonly=True,
    )
    partner_id = fields.Many2one("res.partner", string="Customer", readonly=True)
    invoice_date = fields.Date(
        string="Invoice Date", help="Issue date of the sales transaction", readonly=True
    )
    invoice_date_due = fields.Date(
        string="Due Date", help="Due date of the sales transaction", readonly=True
    )
    invoice_payment_term_id = fields.Many2one(
        "account.payment.term",
        string="Payment Term",
        help="Payment term of the sales transaction",
        readonly=True,
    )
    amount_total = fields.Monetary(
        string="Total Invoice",
        help="Total sales transaction amount including taxes",
        readonly=True,
    )
    amount_untaxed = fields.Monetary(
        string="Untaxed Amount",
        help="Total taxable amount of the sales transaction",
        readonly=True,
    )
    amount_total2 = fields.Monetary(
        string="Total Cash Invoice",
        help="Total cash sales transaction amount including taxes",
        readonly=True,
    )
    amount_untaxed2 = fields.Monetary(
        string="Untaxed Cash Amount",
        help="Total taxable amount of the cash sales transaction without taxes",
        readonly=True,
    )
    financial_cost = fields.Monetary(
        string="Financial Cost",
        help="Financial cost related to the sales transaction",
        compute="_compute_financial_cost",
        store=True,
        readonly=True,
    )
    team_id = fields.Many2one(
        "crm.team",
        string="Sales Team",
        help="Sales team related to the sales transaction",
        readonly=True,
    )
    invoice_user_id = fields.Many2one(
        "res.users",
        string="Salesperson",
        help="Salesperson related to the sales transaction",
        readonly=True,
    )
    currency_id = fields.Many2one("res.currency", string="Currency", readonly=True)
    company_id = fields.Many2one(
        "res.company",
        string="Company",
        default=lambda self: self.env.company,
        readonly=True,
    )
    name = fields.Char(
        string="Receipt", help="Customer payment receipt number", readonly=True
    )
    payment_date = fields.Date(string="Payment Date", readonly=True)
    payment_method_id = fields.Many2one(
        "account.payment.method.line", string="Payment Method", readonly=True
    )
    journal_id = fields.Many2one("account.journal", string="Journal", readonly=True)
    payment_amount = fields.Monetary(
        string="Payment Amount",
        help="Payment receipt amount including taxes",
        readonly=True,
    )
    amount_untaxed_paidontime = fields.Monetary(
        string="Untaxed Paid On Time",
        help="Untaxed amount of the payment receipt paid on time",
        compute="_compute_amounts",
        store=True,
    )
    amount_untaxed_paidlate = fields.Monetary(
        string="Untaxed Paid Late",
        help="Untaxed amount of the payment receipt paid late",
        compute="_compute_amounts",
        store=True,
    )
    financial_cost_portion = fields.Monetary(
        string="Financial Cost Portion",
        help="Portion of financial cost related to the payment receipt amount",
        compute="_compute_financial_cost_portion",
        store=True,
    )
    amount_untaxed_commission = fields.Monetary(
        string="Untaxed Commission Amount",
        help="Untaxed amount of the payment receipt paid on time minus the financial cost",
        compute="_compute_commission_amounts",
        store=True,
    )
    amount_total_commission = fields.Monetary(
        string="Total Commission Amount",
        help="Total amount of the payment receipt paid on time minus the financial cost",
        compute="_compute_commission_amounts",
        store=True,
    )
    state_commission = fields.Selection(
        [
            ("pending", "Pending"),
            ("in_process", "In Process"),
            ("liquidated", "Liquidated"),
        ],
        string="Commission State",
        default="pending",
        help="State of the payment in the commission settlement",
    )

    @api.depends("amount_total", "amount_total2")
    def _compute_financial_cost(self):
        """Compute the financial cost related to the sales transaction"""
        for record in self:
            record.financial_cost = record.amount_total - record.amount_total2

    @api.depends(
        "payment_amount",
        "payment_date",
        "invoice_date_due",
        "amount_untaxed",
        "amount_total",
    )
    def _compute_amounts(self):
        """Compute the amounts of the payment receipt"""
        for record in self:
            record.amount_untaxed_paidontime = 0
            record.amount_untaxed_paidlate = 0

            if record.payment_date and record.invoice_date_due:
                if record.payment_date <= record.invoice_date_due:
                    record.amount_untaxed_paidontime = (
                        record.payment_amount
                        * (record.amount_untaxed / record.amount_total)
                        if record.amount_total
                        else 0
                    )
                else:
                    record.amount_untaxed_paidlate = (
                        record.payment_amount
                        * (record.amount_untaxed / record.amount_total)
                        if record.amount_total
                        else 0
                    )
            else:
                # Log a warning or handle the case where dates are not set
                _logger.warning(
                    f"Invalid date values for record {record.id}: payment_date = {record.payment_date}, invoice_date_due = {record.invoice_date_due}"
                )

    @api.depends("payment_amount", "amount_total", "financial_cost")
    def _compute_financial_cost_portion(self):
        """Compute the portion of financial cost related to the payment receipt amount"""
        for record in self:
            payment_percentage = (record.payment_amount * 100) / record.amount_total
            record.financial_cost_portion = (
                abs(record.financial_cost) * payment_percentage
            ) / 100

    @api.depends(
        "amount_untaxed_paidontime", "financial_cost_portion", "payment_amount"
    )
    def _compute_commission_amounts(self):
        """Compute the commission amounts of the payment receipt"""
        for record in self:
            untaxed_commission = 0
            total_commission = 0
            if record.amount_untaxed_paidontime > 0:
                untaxed_commission = (
                    record.amount_untaxed_paidontime - record.financial_cost_portion
                )
                if untaxed_commission < 0:
                    record.amount_untaxed_commission = 0
                else:
                    record.amount_untaxed_commission = untaxed_commission
                total_commission = record.payment_amount - record.financial_cost_portion
                if total_commission < 0:
                    record.amount_total_commission = 0
                else:
                    record.amount_total_commission = total_commission
            else:
                record.amount_untaxed_commission = 0
                record.amount_total_commission = 0

    @api.model
    def create(self, vals):
        """Set the state of the commission to pending when creating a new record"""
        vals["state_commission"] = "pending"
        return super(BillingPaidOntime, self).create(vals)
