"""Model to classify stock based on turnover"""
from odoo import _, api, fields, models
from odoo.exceptions import ValidationError


class UsersCommissionsGroup(models.Model):
    """Model to classify stock based on turnover"""

    _name = "users.commissions.group"
    _description = "Commission Agents"

    name = fields.Char(string="Name", required=True)
    user_ids = fields.Many2many("res.users", string="Commercial", required=True)
    commission_scheme_ids = fields.One2many(
        "commission.scheme.line", "commission_group_id", string="Commission Scheme"
    )
    company_id = fields.Many2one(
        "res.company",
        string="Company",
        required=True,
        default=lambda self: self.env.company,
    )

    @api.constrains("user_ids")
    def _check_unique_user(self):
        """Check that a user is not part of more than one commission group"""
        for record in self:
            for user in record.user_ids:
                if self.search_count([("user_ids", "in", user.id)]) > 1:
                    raise ValidationError(
                        _(
                            "The same salesperson cannot be part of more than one group of commission agents."
                        )
                    )


class CommissionSchemeLine(models.Model):
    """Model to classify stock based on turnover"""

    _name = "commission.scheme.line"
    _description = "Commission Scheme Line"

    commission_group_id = fields.Many2one(
        "users.commissions.group", string="Group of Commissioners"
    )
    amount_charged_from = fields.Float(string="Amount charged from", required=True)
    amount_charged_to = fields.Float(string="Amount collected up to", required=True)
    amount_percentage = fields.Float(string="Percentage %", required=True)

    @api.constrains("amount_charged_from", "amount_charged_to", "amount_percentage")
    def _check_amounts(self):
        """Check that the amounts and percentages are valid"""
        for record in self:
            if (
                record.amount_charged_from < 0
                or record.amount_charged_to < 0
                or record.amount_percentage < 0
            ):
                raise ValidationError(_("Amounts and percentages cannot be negative"))
            if (
                record.amount_charged_from == 0
                or record.amount_charged_to == 0
                or record.amount_percentage == 0
            ):
                raise ValidationError(
                    _("Amounts and percentages must be greater than zero")
                )
            if record.amount_charged_from >= record.amount_charged_to:
                raise ValidationError(
                    _("The 'from' amount must be less than the 'to' amount")
                )

    @api.constrains("commission_group_id", "amount_charged_from", "amount_charged_to")
    def _check_overlapping_ranges(self):
        """Check that the ranges of amounts do not overlap"""
        for record in self:
            overlapping = self.search(
                [
                    ("commission_group_id", "=", record.commission_group_id.id),
                    ("id", "!=", record.id),
                    "|",
                    "&",
                    ("amount_charged_from", "<=", record.amount_charged_from),
                    ("amount_charged_to", ">", record.amount_charged_from),
                    "&",
                    ("amount_charged_from", "<", record.amount_charged_to),
                    ("amount_charged_to", ">=", record.amount_charged_to),
                ]
            )
            if overlapping:
                raise ValidationError(_("The amount ranges cannot overlap"))
