"""Module for Commission Settlement model."""
from odoo import _, api, fields, models
from odoo.exceptions import ValidationError


class CommissionSettlement(models.Model):
    """Class for Commission Settlement model."""

    _name = "commissions.billing.paid.ontime"
    _description = "Commission Settlement"
    _inherit = ["mail.thread", "mail.activity.mixin"]

    name = fields.Char(
        string="Sequence",
        readonly=True,
        required=True,
        copy=False,
        default=lambda self: _("New"),
    )
    team_id = fields.Many2one(
        "crm.team", string="Sales Team", required=True, tracking=True
    )
    user_id = fields.Many2one(
        "res.users", string="Salesperson", required=True, tracking=True
    )
    create_uid = fields.Many2one("res.users", string="Created by", readonly=True)
    create_date = fields.Datetime(string="Created on", readonly=True)
    date_start = fields.Date(string="Start Date", required=True, tracking=True)
    date_end = fields.Date(string="End Date", required=True, tracking=True)
    amount_trm = fields.Float(string="TRM", required=True, tracking=True)
    company_id = fields.Many2one(
        "res.company",
        string="Company",
        required=True,
        default=lambda self: self.env.company,
    )
    state = fields.Selection(
        [
            ("draft", "BORRADOR"),
            ("in_progress", "EN PROCESO"),
            ("done", "HECHO"),
            ("liquidated", "LIQUIDADO"),
        ],
        string="Status",
        default="draft",
        tracking=True,
        readonly=True,
    )
    line_ids = fields.One2many(
        "commissions_liquidation.lines",
        "settlement_id",
        string="Líneas de Comisión",
        readonly=True,
    )
    currency_id = fields.Many2one(
        "res.currency",
        string="Currency",
        default=lambda self: self.env.company.currency_id,
        required=True,
    )

    observations = fields.Html(string="Observaciones")
    total_amount_paidontime = fields.Monetary(
        string="Monto cobrado a término",
        compute="_compute_totals",
        store=True,
        currency_field="currency_id",
    )
    commission_percentage = fields.Float(
        string="Porcentaje de comisión (%)",
        compute="_compute_commission_percentage",
        store=True,
    )
    total_commission_liquidation = fields.Monetary(
        string="Total Comisión",
        compute="_compute_totals",
        store=True,
        currency_field="currency_id",
    )

    @api.depends("line_ids.amount_total_commission", "amount_trm")
    def _compute_totals(self):
        for record in self:
            record.total_amount_paidontime = sum(
                record.line_ids.mapped("amount_total_commission")
            )
            record.total_commission_liquidation = record.total_amount_paidontime * (
                record.commission_percentage
            )

    @api.depends("total_amount_paidontime", "amount_trm", "user_id")
    def _compute_commission_percentage(self):
        for record in self:
            amount_usd = (
                record.total_amount_paidontime / record.amount_trm
                if record.amount_trm
                else 0
            )
            commission_group = self.env["users.commissions.group"].search(
                [("user_ids", "in", record.user_id.id)], limit=1
            )

            if commission_group:
                for scheme in commission_group.commission_scheme_ids:
                    if (
                        scheme.amount_charged_from
                        <= amount_usd
                        <= scheme.amount_charged_to
                    ):
                        record.commission_percentage = scheme.amount_percentage / 100
                        break
                else:
                    record.commission_percentage = 0
            else:
                record.commission_percentage = 0

    @api.model
    def create(self, vals):
        """Override the create method to set the sequence."""
        if vals.get("name", _("New")) == _("New"):
            vals["name"] = self.env["ir.sequence"].next_by_code(
                "commissions.billing.paid.ontime"
            )
        return super(CommissionSettlement, self).create(vals)

    @api.constrains("amount_trm")
    def _check_amount_trm(self):
        """Check that the TRM is greater than zero."""
        for record in self:
            if record.amount_trm <= 0:
                raise ValidationError(_("TRM must be greater than zero."))

    def action_calculate_commissions(self):
        """Calculate the commissions based on the billing records."""
        self.ensure_one()
        if not (self.team_id and self.user_id and self.date_start and self.date_end):
            raise ValidationError(
                _(
                    "Debe seleccionar Equipo de ventas, Comercial y Fechas de periodo de liquidación"
                )
            )

        self.line_ids.unlink()

        domain = [
            ("team_id", "=", self.team_id.id),
            ("invoice_user_id", "=", self.user_id.id),
            ("payment_date", ">=", self.date_start),
            ("payment_date", "<=", self.date_end),
            ("state_commission", "=", "pending"),
        ]
        billing_records = self.env["billing.paid.ontime"].search(domain)

        if billing_records:
            for record in billing_records:
                self.env["commissions_liquidation.lines"].create(
                    {
                        "settlement_id": self.id,
                        "receipt_name": record.name,
                        "payment_date": record.payment_date,
                        "payment_method_id": record.payment_method_id.id,
                        "journal_id": record.journal_id.id,
                        "invoice_name": record.invoice_name,
                        "partner_id": record.partner_id.id,
                        "team_id": record.team_id.id,
                        "user_id": record.invoice_user_id.id,
                        "payment_amount": record.payment_amount,
                        "amount_untaxed_paidontime": record.amount_untaxed_paidontime,
                        "amount_untaxed_paidlate": record.amount_untaxed_paidlate,
                        "financial_cost_portion": record.financial_cost_portion,
                        "amount_untaxed_commission": record.amount_untaxed_commission,
                        "amount_total_commission": record.amount_total_commission,
                    }
                )
                record.sudo().state_commission = "in_process"

            self.state = "in_progress"
        else:
            raise ValidationError(
                _("No billing records were found to settle commissions.")
            )

    def action_reset_to_draft(self):
        """Reset commission settlement to draft state."""
        self.ensure_one()
        if self.state not in ["in_progress", "done"]:
            raise ValidationError(
                _("You can only reset to draft from 'In Progress' or 'Done' states.")
            )

        return {
            "type": "ir.actions.act_window",
            "name": _("Confirma volver a estado borrador"),
            "res_model": "confirmation.wizard",
            "view_mode": "form",
            "target": "new",
            "context": {
                "default_message": _(
                    "Restablecer a borrador eliminará todas las lineas en la sección de Líneas de Comisión. Deberá calcular nuevamente el proceso de Liquidación de comisiones"
                ),
                "default_model": self._name,
                "default_method": "action_reset_to_draft_confirm",
                "default_res_id": self.id,
            },
        }

    def action_reset_to_draft_confirm(self):
        """Confirm reset to draft after user confirmation."""
        self.write({"state": "draft"})

        # Reset related billing records
        for line in self.line_ids:
            billing_records = self.env["billing.paid.ontime"].search(
                [
                    ("team_id", "=", self.team_id.id),
                    ("invoice_user_id", "=", self.user_id.id),
                    ("payment_date", ">=", self.date_start),
                    ("payment_date", "<=", self.date_end),
                    ("name", "=", line.receipt_name),
                    ("state_commission", "=", "in_process"),
                ]
            )
            billing_records.sudo().write({"state_commission": "pending"})

        # Delete commission lines
        self.line_ids.sudo().unlink()

        return {
            "type": "ir.actions.client",
            "tag": "display_notification",
            "params": {
                "title": _("Success"),
                "message": _("Se restablecio a estado borrador correctamente."),
                "type": "success",
                "sticky": False,
                "next": {
                    "type": "ir.actions.act_window",
                    "res_model": "commissions.billing.paid.ontime",
                    "res_id": self.id,
                    "view_mode": "form",
                    "views": [(False, "form")],
                    "target": "current",
                },
            },
        }

    def action_confirm(self):
        """Confirm commission settlement."""
        self.ensure_one()
        if self.state != "in_progress":
            raise ValidationError(
                _("You can only confirm a settlement that is 'In Progress'.")
            )

        return {
            "type": "ir.actions.act_window",
            "name": _("Confirma"),
            "res_model": "confirmation.wizard",
            "view_mode": "form",
            "target": "new",
            "context": {
                "default_message": _("¿Desea confirmar esta transacción?"),
                "default_model": self._name,
                "default_method": "action_confirm_confirm",
                "default_res_id": self.id,
            },
        }

    def action_confirm_confirm(self):
        """Execute confirmation after user approval."""
        self.write({"state": "done"})
        return {
            "type": "ir.actions.client",
            "tag": "display_notification",
            "params": {
                "title": _("Success"),
                "message": _("Se restablecio a estado confirmado correctamente."),
                "type": "success",
                "sticky": False,
                "next": {
                    "type": "ir.actions.act_window",
                    "res_model": "commissions.billing.paid.ontime",
                    "res_id": self.id,
                    "view_mode": "form",
                    "views": [(False, "form")],
                    "target": "current",
                },
            },
        }

    def action_liquidate(self):
        """Liquidate commission settlement."""
        self.ensure_one()
        if self.state != "done":
            raise ValidationError(
                _("You can only liquidate a settlement that is 'Done'.")
            )

        return {
            "type": "ir.actions.act_window",
            "name": _("Confirma Liquidación"),
            "res_model": "confirmation.wizard",
            "view_mode": "form",
            "target": "new",
            "context": {
                "default_message": _(
                    '¿Desea cerrar la Liquidación de Comisiones? Este proceso actualizará todos los recibos de pago a estado "Liquidado" y no podrá realizar ninguna modificación adicional. Esta acción NO puede revertirse'
                ),
                "default_model": self._name,
                "default_method": "action_liquidate_confirm",
                "default_res_id": self.id,
            },
        }

    def action_liquidate_confirm(self):
        """Execute liquidation after user confirmation."""
        self.write({"state": "liquidated"})

        # Update related billing records
        billing_records = self.env["billing.paid.ontime"].search(
            [
                ("team_id", "=", self.team_id.id),
                ("invoice_user_id", "=", self.user_id.id),
                ("payment_date", ">=", self.date_start),
                ("payment_date", "<=", self.date_end),
                ("state_commission", "=", "in_process"),
            ]
        )
        billing_records.sudo().write({"state_commission": "liquidated"})

    def action_confirm_confirm(self):
        """Execute confirmation after user approval."""
        self.write({"state": "done"})
        return {
            "type": "ir.actions.client",
            "tag": "display_notification",
            "params": {
                "title": _("Success"),
                "message": _("Se restablecio a estado liquidado correctamente."),
                "type": "success",
                "sticky": False,
                "next": {
                    "type": "ir.actions.act_window",
                    "res_model": "commissions.billing.paid.ontime",
                    "res_id": self.id,
                    "view_mode": "form",
                    "views": [(False, "form")],
                    "target": "current",
                },
            },
        }


class CommissionSettlementLine(models.Model):
    """Class for Commission Settlement Line model."""

    _name = "commissions_liquidation.lines"
    _description = "Commission Settlement Line"
    _order = "payment_date asc"

    settlement_id = fields.Many2one(
        "commissions.billing.paid.ontime",
        string="Settlement",
        required=True,
        ondelete="cascade",
    )
    receipt_name = fields.Char(string="Receipt", required=True)
    payment_date = fields.Date(string="Payment Date", required=True)
    payment_method_id = fields.Many2one(
        "account.payment.method.line", string="Payment Method"
    )
    journal_id = fields.Many2one("account.journal", string="Journal")
    invoice_name = fields.Char(string="Invoice Number")
    partner_id = fields.Many2one("res.partner", string="Customer", required=True)
    team_id = fields.Many2one("crm.team", string="Sales Team")
    user_id = fields.Many2one("res.users", string="Salesperson")
    payment_amount = fields.Monetary(string="Payment Amount", required=True)
    amount_untaxed_paidontime = fields.Monetary(
        string="Untaxed Amount Paid On Time", required=True
    )
    amount_untaxed_paidlate = fields.Monetary(string="Untaxed Amount Paid Late")
    financial_cost_portion = fields.Monetary(string="Financial Cost Portion")
    amount_untaxed_commission = fields.Monetary(string="Untaxed Amount for Commission")
    amount_total_commission = fields.Monetary(string="Total Amount for Commission")
    currency_id = fields.Many2one(
        "res.currency",
        string="Currency",
        related="settlement_id.company_id.currency_id",
    )
    company_id = fields.Many2one(
        "res.company", string="Company", related="settlement_id.company_id"
    )
