"""Test cases for Account Move and Account Move Line customizations."""
from datetime import date

from odoo.tests.common import TransactionCase


class TestAccountMoveCustomization(TransactionCase):
    """Test cases for Account Move customization."""

    def setUp(self):
        """Set up test environment."""
        super().setUp()

        # Create test company
        self.company = self.env["res.company"].create(
            {
                "name": "Test Company",
                "currency_id": self.env.ref("base.USD").id,
            }
        )

        # Create test partner
        self.partner = self.env["res.partner"].create(
            {
                "name": "Test Partner",
                "company_id": self.company.id,
            }
        )

        # Create test product category
        self.category = self.env["product.category"].create(
            {
                "name": "Test Category",
            }
        )

        # Create test product
        self.product = self.env["product.product"].create(
            {
                "name": "Test Product",
                "type": "product",
                "categ_id": self.category.id,
                "standard_price": 800.00,
                "list_price": 1000.00,
            }
        )

        # Create cash pricelist
        self.cash_pricelist = self.env["product.pricelist"].create(
            {
                "name": "Cash Pricelist",
                "is_cash_list": True,
                "item_ids": [
                    (
                        0,
                        0,
                        {
                            "applied_on": "3_global",
                            "compute_price": "percentage",
                            "percent_price": 10.0,  # 10% discount for cash
                        },
                    )
                ],
            }
        )

        # Create invoice
        self.invoice = self.env["account.move"].create(
            {
                "move_type": "out_invoice",
                "partner_id": self.partner.id,
                "invoice_date": date.today(),
                "invoice_line_ids": [
                    (
                        0,
                        0,
                        {
                            "product_id": self.product.id,
                            "quantity": 2,
                            "price_unit": 1000.00,  # Regular price
                            "price_unit2": 900.00,  # Cash price (10% less)
                            "tax_ids": [
                                (
                                    6,
                                    0,
                                    self.env["account.tax"]
                                    .search([("type_tax_use", "=", "sale")], limit=1)
                                    .ids,
                                )
                            ],
                        },
                    )
                ],
            }
        )

    def test_compute_cash_amounts_line(self):
        """Test computation of cash amounts for invoice line."""
        line = self.invoice.invoice_line_ids[0]

        # Test initial values
        self.assertEqual(line.price_unit2, 900.00)

        # Compute cash amounts
        line._compute_cash_amounts()

        # Verify subtotal2 and total2 (exact values will depend on tax configuration)
        expected_subtotal2 = 1800.00  # 2 units * 900
        self.assertAlmostEqual(line.price_subtotal2, expected_subtotal2, places=2)
        self.assertTrue(
            line.price_total2 >= line.price_subtotal2
        )  # Total should include taxes

    def test_compute_cash_amounts_move(self):
        """Test computation of cash amounts for entire invoice."""
        # Compute cash amounts
        self.invoice._compute_cash_amounts()

        # Verify amounts
        expected_untaxed2 = 1800.00  # 2 units * 900
        self.assertAlmostEqual(
            self.invoice.amount_untaxed2, expected_untaxed2, places=2
        )
        self.assertTrue(
            self.invoice.amount_total2 >= self.invoice.amount_untaxed2
        )  # Total should include taxes

    def test_calculate_cash_prices(self):
        """Test calculation of cash prices from pricelist."""
        # Create new invoice without price_unit2
        invoice = self.env["account.move"].create(
            {
                "move_type": "out_invoice",
                "partner_id": self.partner.id,
                "invoice_date": date.today(),
                "invoice_line_ids": [
                    (
                        0,
                        0,
                        {
                            "product_id": self.product.id,
                            "quantity": 2,
                            "price_unit": 1000.00,
                            "tax_ids": [
                                (
                                    6,
                                    0,
                                    self.env["account.tax"]
                                    .search([("type_tax_use", "=", "sale")], limit=1)
                                    .ids,
                                )
                            ],
                        },
                    )
                ],
            }
        )

        # Calculate cash prices
        invoice._calculate_cash_prices()

        # Verify cash price was calculated correctly (10% less than regular price)
        line = invoice.invoice_line_ids[0]
        self.assertEqual(line.price_unit2, 900.00)

    def test_post_invoice(self):
        """Test posting invoice triggers cash price calculation."""
        # Create new invoice
        invoice = self.env["account.move"].create(
            {
                "move_type": "out_invoice",
                "partner_id": self.partner.id,
                "invoice_date": date.today(),
                "invoice_line_ids": [
                    (
                        0,
                        0,
                        {
                            "product_id": self.product.id,
                            "quantity": 2,
                            "price_unit": 1000.00,
                            "tax_ids": [
                                (
                                    6,
                                    0,
                                    self.env["account.tax"]
                                    .search([("type_tax_use", "=", "sale")], limit=1)
                                    .ids,
                                )
                            ],
                        },
                    )
                ],
            }
        )

        # Post invoice
        invoice.action_post()

        # Verify cash prices were calculated
        line = invoice.invoice_line_ids[0]
        self.assertEqual(line.price_unit2, 900.00)
        self.assertTrue(line.price_subtotal2 > 0)
        self.assertTrue(line.price_total2 > 0)

    def test_no_cash_pricelist(self):
        """Test behavior when no cash pricelist exists."""
        # Archive the cash pricelist
        self.cash_pricelist.active = False

        # Create new invoice
        invoice = self.env["account.move"].create(
            {
                "move_type": "out_invoice",
                "partner_id": self.partner.id,
                "invoice_date": date.today(),
                "invoice_line_ids": [
                    (
                        0,
                        0,
                        {
                            "product_id": self.product.id,
                            "quantity": 2,
                            "price_unit": 1000.00,
                            "tax_ids": [
                                (
                                    6,
                                    0,
                                    self.env["account.tax"]
                                    .search([("type_tax_use", "=", "sale")], limit=1)
                                    .ids,
                                )
                            ],
                        },
                    )
                ],
            }
        )

        # Calculate cash prices (should not raise error)
        invoice._calculate_cash_prices()

        # Verify no changes were made
        line = invoice.invoice_line_ids[0]
        self.assertEqual(line.price_unit2, 0.0)

    def test_discount_application(self):
        """Test cash amount computation with discounts."""
        # Update invoice line with discount
        self.invoice.invoice_line_ids[0].write(
            {
                "discount": 20,  # 20% discount
            }
        )

        # Compute cash amounts
        self.invoice._compute_cash_amounts()

        # Verify amounts (900 - 20% = 720 per unit)
        expected_subtotal2 = 1440.00  # 2 units * 720
        self.assertAlmostEqual(
            self.invoice.amount_untaxed2, expected_subtotal2, places=2
        )
