"""Test cases for Account Payment and Payment Group models."""
from datetime import date, timedelta

from odoo.tests.common import TransactionCase


class TestAccountPaymentGroup(TransactionCase):
    """Test cases for Account Payment Group model."""

    def setUp(self):
        """Set up test environment."""
        super().setUp()

        # Create test company
        self.company = self.env["res.company"].create(
            {
                "name": "Test Company",
                "currency_id": self.env.ref("base.USD").id,
            }
        )

        # Create test partner
        self.partner = self.env["res.partner"].create(
            {
                "name": "Test Partner",
                "company_id": self.company.id,
            }
        )

        # Create test sales team
        self.sales_team = self.env["crm.team"].create(
            {
                "name": "Test Sales Team",
                "company_id": self.company.id,
            }
        )

        # Create test payment term
        self.payment_term = self.env["account.payment.term"].create(
            {
                "name": "Test Payment Term",
                "line_ids": [
                    (
                        0,
                        0,
                        {
                            "value": "balance",
                            "days": 30,
                        },
                    )
                ],
            }
        )

        # Create test invoice
        self.invoice = self.env["account.move"].create(
            {
                "move_type": "out_invoice",
                "partner_id": self.partner.id,
                "invoice_date": date.today(),
                "invoice_date_due": date.today() + timedelta(days=30),
                "invoice_payment_term_id": self.payment_term.id,
                "team_id": self.sales_team.id,
                "invoice_user_id": self.env.user.id,
                "invoice_line_ids": [
                    (
                        0,
                        0,
                        {
                            "name": "Test Product",
                            "quantity": 1,
                            "price_unit": 100.00,
                        },
                    )
                ],
            }
        )
        self.invoice.action_post()

        # Create payment method
        self.payment_method_line = self.env["account.payment.method.line"].create(
            {
                "name": "Test Payment Method",
                "payment_method_id": self.env.ref(
                    "account.account_payment_method_manual_in"
                ).id,
                "journal_id": self.env["account.journal"].search([], limit=1).id,
            }
        )

        # Create payment group
        self.payment_group = self.env["account.payment.group"].create(
            {
                "partner_id": self.partner.id,
                "company_id": self.company.id,
                "payment_date": date.today(),
                "communication": "Test Payment",
            }
        )

        # Add invoice to payment group
        self.payment_group.to_pay_move_line_ids = self.invoice.line_ids.filtered(
            lambda x: x.account_type == "asset_receivable"
        )

        # Create payment
        self.payment = self.env["account.payment"].create(
            {
                "payment_group_id": self.payment_group.id,
                "amount": 100.00,
                "payment_type": "inbound",
                "partner_type": "customer",
                "partner_id": self.partner.id,
                "payment_method_line_id": self.payment_method_line.id,
            }
        )

    def test_payment_group_post(self):
        """Test posting payment group creates billing.paid.ontime record."""
        self.payment_group.post()

        # Check if billing.paid.ontime record was created
        billing_record = self.env["billing.paid.ontime"].search(
            [
                ("name", "=", self.payment_group.name),
                ("company_id", "=", self.company.id),
            ]
        )

        self.assertTrue(billing_record, "Billing record should be created")
        self.assertEqual(billing_record.invoice_name, self.invoice.name)
        self.assertEqual(billing_record.partner_id, self.partner)
        self.assertEqual(billing_record.payment_amount, 100.00)
        self.assertEqual(billing_record.team_id, self.sales_team)

    def test_payment_group_draft(self):
        """Test setting payment group to draft deletes billing.paid.ontime record."""
        # First post the payment group
        self.payment_group.post()

        # Verify billing record exists
        billing_record = self.env["billing.paid.ontime"].search(
            [
                ("name", "=", self.payment_group.name),
                ("company_id", "=", self.company.id),
            ]
        )
        self.assertTrue(billing_record, "Billing record should exist after posting")

        # Set to draft
        self.payment_group.action_draft()

        # Verify billing record was deleted
        billing_record = self.env["billing.paid.ontime"].search(
            [
                ("name", "=", self.payment_group.name),
                ("company_id", "=", self.company.id),
            ]
        )
        self.assertFalse(billing_record, "Billing record should be deleted after draft")


class TestAccountPayment(TransactionCase):
    """Test cases for Account Payment model."""

    def setUp(self):
        """Set up test environment."""
        super().setUp()
        # Reuse setup from TestAccountPaymentGroup
        test_payment_group = TestAccountPaymentGroup()
        test_payment_group.setUp()
        self.payment = test_payment_group.payment
        self.company = test_payment_group.company

    def test_payment_draft(self):
        """Test setting payment to draft deletes billing.paid.ontime record."""
        # First post the payment
        self.payment.action_post()

        # Create billing record
        self.env["billing.paid.ontime"].create(
            {
                "name": self.payment.name,
                "company_id": self.company.id,
                "partner_id": self.payment.partner_id.id,
                "payment_amount": self.payment.amount,
            }
        )

        # Verify billing record exists
        billing_record = self.env["billing.paid.ontime"].search(
            [
                ("name", "=", self.payment.name),
                ("company_id", "=", self.company.id),
            ]
        )
        self.assertTrue(billing_record, "Billing record should exist")

        # Set to draft
        self.payment.action_draft()

        # Verify billing record was deleted
        billing_record = self.env["billing.paid.ontime"].search(
            [
                ("name", "=", self.payment.name),
                ("company_id", "=", self.company.id),
            ]
        )
        self.assertFalse(billing_record, "Billing record should be deleted after draft")
