"""Test cases for Billing Paid On Time model."""
from datetime import date, timedelta

from odoo.tests.common import TransactionCase


class TestBillingPaidOntime(TransactionCase):
    """Test cases for Billing Paid On Time model."""

    def setUp(self):
        """Set up test environment."""
        super().setUp()

        # Create test company
        self.company = self.env["res.company"].create(
            {
                "name": "Test Company",
                "currency_id": self.env.ref("base.USD").id,
            }
        )

        # Create test partner
        self.partner = self.env["res.partner"].create(
            {
                "name": "Test Partner",
                "company_id": self.company.id,
            }
        )

        # Create test sales team
        self.sales_team = self.env["crm.team"].create(
            {
                "name": "Test Sales Team",
                "company_id": self.company.id,
            }
        )

        # Create test payment term
        self.payment_term = self.env["account.payment.term"].create(
            {
                "name": "Test Payment Term",
                "line_ids": [
                    (
                        0,
                        0,
                        {
                            "value": "balance",
                            "days": 30,
                        },
                    )
                ],
            }
        )

        # Create test payment method
        self.payment_method = self.env["account.payment.method.line"].create(
            {
                "name": "Test Payment Method",
                "payment_method_id": self.env.ref(
                    "account.account_payment_method_manual_in"
                ).id,
                "journal_id": self.env["account.journal"].search([], limit=1).id,
            }
        )

        # Create test billing record
        self.billing_record = self.env["billing.paid.ontime"].create(
            {
                "invoice_name": "INV/2024/0001",
                "partner_id": self.partner.id,
                "invoice_date": date.today(),
                "invoice_date_due": date.today() + timedelta(days=30),
                "invoice_payment_term_id": self.payment_term.id,
                "amount_total": 1000.00,
                "amount_untaxed": 900.00,
                "amount_total2": 950.00,
                "amount_untaxed2": 855.00,
                "team_id": self.sales_team.id,
                "invoice_user_id": self.env.user.id,
                "currency_id": self.company.currency_id.id,
                "company_id": self.company.id,
                "name": "RCPT/2024/0001",
                "payment_date": date.today(),
                "payment_method_id": self.payment_method.id,
                "journal_id": self.payment_method.journal_id.id,
                "payment_amount": 500.00,
            }
        )

    def test_create_billing_record(self):
        """Test creation of billing record."""
        self.assertEqual(self.billing_record.state_commission, "pending")
        self.assertEqual(self.billing_record.invoice_name, "INV/2024/0001")
        self.assertEqual(self.billing_record.partner_id, self.partner)
        self.assertEqual(self.billing_record.payment_amount, 500.00)

    def test_compute_financial_cost(self):
        """Test computation of financial cost."""
        self.assertEqual(self.billing_record.financial_cost, 50.00)  # 1000 - 950

    def test_compute_amounts_paid_on_time(self):
        """Test computation of amounts when paid on time."""
        # Payment date is today, due date is in 30 days
        self.assertEqual(
            self.billing_record.amount_untaxed_paidontime, 450.00
        )  # (500 * 900/1000)
        self.assertEqual(self.billing_record.amount_untaxed_paidlate, 0.00)

    def test_compute_amounts_paid_late(self):
        """Test computation of amounts when paid late."""
        # Set payment date to after due date
        self.billing_record.payment_date = date.today() + timedelta(days=31)

        self.assertEqual(self.billing_record.amount_untaxed_paidontime, 0.00)
        self.assertEqual(
            self.billing_record.amount_untaxed_paidlate, 450.00
        )  # (500 * 900/1000)

    def test_compute_financial_cost_portion(self):
        """Test computation of financial cost portion."""
        # Financial cost is 50.00, payment is 50% of total
        expected_cost_portion = 25.00  # (500/1000 * 50)
        self.assertEqual(
            self.billing_record.financial_cost_portion, expected_cost_portion
        )

    def test_compute_commission_amounts_paid_on_time(self):
        """Test computation of commission amounts when paid on time."""
        # amount_untaxed_paidontime = 450.00
        # financial_cost_portion = 25.00
        expected_untaxed_commission = 425.00  # 450 - 25
        expected_total_commission = 475.00  # 500 - 25

        self.assertEqual(
            self.billing_record.amount_untaxed_commission, expected_untaxed_commission
        )
        self.assertEqual(
            self.billing_record.amount_total_commission, expected_total_commission
        )

    def test_compute_commission_amounts_paid_late(self):
        """Test computation of commission amounts when paid late."""
        # Set payment date to after due date
        self.billing_record.payment_date = date.today() + timedelta(days=31)

        self.assertEqual(self.billing_record.amount_untaxed_commission, 0.00)
        self.assertEqual(self.billing_record.amount_total_commission, 0.00)

    def test_missing_dates(self):
        """Test behavior when dates are missing."""
        billing_record = self.env["billing.paid.ontime"].create(
            {
                "invoice_name": "INV/2024/0002",
                "partner_id": self.partner.id,
                "amount_total": 1000.00,
                "amount_untaxed": 900.00,
                "payment_amount": 500.00,
                "currency_id": self.company.currency_id.id,
                "company_id": self.company.id,
            }
        )

        self.assertEqual(billing_record.amount_untaxed_paidontime, 0.00)
        self.assertEqual(billing_record.amount_untaxed_paidlate, 0.00)

    def test_zero_amount_total(self):
        """Test behavior when amount_total is zero."""
        billing_record = self.env["billing.paid.ontime"].create(
            {
                "invoice_name": "INV/2024/0003",
                "partner_id": self.partner.id,
                "invoice_date": date.today(),
                "invoice_date_due": date.today() + timedelta(days=30),
                "amount_total": 0.00,
                "amount_untaxed": 0.00,
                "payment_amount": 0.00,
                "payment_date": date.today(),
                "currency_id": self.company.currency_id.id,
                "company_id": self.company.id,
            }
        )

        self.assertEqual(billing_record.amount_untaxed_paidontime, 0.00)
        self.assertEqual(billing_record.amount_untaxed_paidlate, 0.00)
