"""Test cases for Commission Settlement model."""
from datetime import date

from odoo.exceptions import ValidationError
from odoo.tests.common import TransactionCase


class TestCommissionSettlement(TransactionCase):
    """Test cases for Commission Settlement model."""

    def setUp(self):
        """Set up the test environment."""
        super().setUp()
        # Create test data
        self.test_team = self.env["crm.team"].create({"name": "Test Team"})
        self.test_user = self.env["res.users"].create(
            {
                "name": "Test User",
                "login": "test_user",
                "email": "test_user@example.com",
            }
        )

        # Create test billing records
        self.billing_record = self.env["billing.paid.ontime"].create(
            {
                "name": "TEST/001",
                "team_id": self.test_team.id,
                "invoice_user_id": self.test_user.id,
                "payment_date": date(2023, 6, 1),
                "state_commission": "pending",
                "payment_amount": 1000.0,
                "amount_untaxed_paidontime": 900.0,
                "amount_total_commission": 900.0,
                "partner_id": self.env["res.partner"]
                .create({"name": "Test Partner"})
                .id,
            }
        )

        # Create commission settlement
        self.commission_settlement = self.env["commissions.billing.paid.ontime"].create(
            {
                "team_id": self.test_team.id,
                "user_id": self.test_user.id,
                "date_start": date(2023, 1, 1),
                "date_end": date(2023, 12, 31),
                "amount_trm": 1.0,
            }
        )

    def test_create_commission_settlement(self):
        """Test the creation of a commission settlement."""
        self.assertEqual(self.commission_settlement.state, "draft")
        # Remove the name check since it's generated by sequence
        self.assertTrue(self.commission_settlement.name.startswith("LIQ/"))

    def test_action_calculate_commissions(self):
        """Test the action_calculate_commissions method."""
        self.commission_settlement.action_calculate_commissions()
        self.assertEqual(self.commission_settlement.state, "in_progress")
        self.assertTrue(len(self.commission_settlement.line_ids) > 0)

    def test_action_reset_to_draft(self):
        """Test the action_reset_to_draft method."""
        # First calculate commissions to move to in_progress
        self.commission_settlement.action_calculate_commissions()
        self.commission_settlement.action_reset_to_draft_confirm()
        self.assertEqual(self.commission_settlement.state, "draft")
        self.assertEqual(len(self.commission_settlement.line_ids), 0)

    def test_action_confirm(self):
        """Test the action_confirm method."""
        self.commission_settlement.action_calculate_commissions()
        self.commission_settlement.action_confirm_confirm()
        self.assertEqual(self.commission_settlement.state, "done")

    def test_action_liquidate(self):
        """Test the action_liquidate method."""
        self.commission_settlement.action_calculate_commissions()
        self.commission_settlement.action_confirm_confirm()
        self.commission_settlement.action_liquidate_confirm()
        self.assertEqual(self.commission_settlement.state, "liquidated")

    def test_check_amount_trm(self):
        """Test the _check_amount_trm constraint."""
        with self.assertRaises(ValidationError):
            self.commission_settlement.write({"amount_trm": 0.0})
