"""Class to manage minimum margin for sale order lines."""
from odoo import api, fields, models


class ProductTemplate(models.Model):
    """Class to manage product templates."""

    _inherit = "product.template"

    tmpl_brand_id = fields.Many2one(
        "product.brand2", string="Marca", index=True, ondelete="restrict", store=True
    )
    family_id = fields.Many2one(
        "product.family",
        string="Rubro",
        index=True,
        ondelete="restrict",
        store=True,
        help="Rubro al que está relacionado el producto o servicio",
    )
    subfamily_id = fields.Many2one(
        "product.subfamily",
        string="SubRubro",
        help="SubRubro al que está relacionado el producto o servicio",
        domain="[('family_id', '=', family_id)]",
        states={"draft": [("readonly", False)]},
    )
    spare_parts = fields.Boolean(string="Posee repuestos", default=False)
    spare_parts_ids = fields.One2many(
        "product.spare.parts", "product_template_id", string="Repuestos"
    )
    parts_visibility = fields.Boolean(string="Repuestos Visible", default=False)
    machinery_ids = fields.Many2many(
        "product.template",
        "colum1",
        "column2",
        string="Maquinaria",
        compute="_compute_spare_parts_machinery",
    )

    def _compute_spare_parts_machinery(self):
        """Compute machinery products."""
        list_spare_parts = self.env["product.spare.parts"].search(
            [("product_id.product_tmpl_id", "in", self.ids)]
        )
        product_template_ids = list_spare_parts.mapped("product_template_id")
        self.update({"machinery_ids": [(6, 0, product_template_ids.ids)]})

    @api.onchange("spare_parts")
    def _onchange_spare_parts(self):
        """Set parts visibility based on spare parts."""
        if self.spare_parts:
            self.parts_visibility = True
        else:
            self.parts_visibility = False

    @api.onchange("family_id")
    def _onchange_family_id(self):
        """Set domain for subfamily based on family."""
        if self.family_id:
            return {"domain": {"subfamily_id": [("family_id", "=", self.family_id.id)]}}
        else:
            return {"domain": {"subfamily_id": []}}


class ProductProduct(models.Model):
    """Class to manage product products."""

    _inherit = "product.product"

    brand_id = fields.Many2one(
        related="product_tmpl_id.tmpl_brand_id",
        string="Marca",
        index=True,
        ondelete="restrict",
        store=True,
    )
    family_id = fields.Many2one(
        related="product_tmpl_id.family_id",
        string="Rubro",
        index=True,
        ondelete="restrict",
        store=True,
        help="Rubro al que está relacionado el producto o servicio",
    )
    subfamily_id = fields.Many2one(
        related="product_tmpl_id.subfamily_id",
        readonly=True,
        store=True,
    )
    product_tag_ids = fields.Many2many(
        related="product_tmpl_id.product_tag_ids",
        string="Etiquetas de Producto",
        readonly=True,
    )
    sequence = fields.Integer(string="Secuencia")

    def write(self, values):
        """Update branding in template when writing to product."""
        result = super(ProductProduct, self).write(values)
        if "brand_id" in values and self.product_tmpl_id:
            self.product_tmpl_id.write({"brand_id": values["brand_id"]})
        if "family_id" in values and self.product_tmpl_id:
            self.product_tmpl_id.write({"family_id": values["family_id"]})
        if "subfamily_id" in values and self.product_tmpl_id:
            self.product_tmpl_id.write({"subfamily_id": values["subfamily_id"]})
        return result
