"""This module is used to extend the payment group and payment model, adding fields and validations when paying with cards."""
import logging

from odoo import _, api, fields, models
from odoo.exceptions import ValidationError

_logger = logging.getLogger(__name__)


class AccountPaymentGroup(models.Model):
    """It is extended and added to the payment method payment. So you can group by payment method in the tree view."""

    _inherit = "account.payment.group"

    payment_methods = fields.Char(
        string="Payment Methods",
        compute="_compute_payment_methods2",
        search="_search_payment_methods2",
        store=True,
        change_default=True,
        index=True,
    )
    journal_id = fields.Many2one("account.journal", string="Journal")

    def _search_payment_methods2(self, operator, value):
        """Search for payment methods."""
        recs = self.search([("payment_ids.journal_id.name", operator, value)])
        return [("id", "in", recs.ids)]

    @api.depends("payment_ids")
    def _compute_payment_methods2(self):
        """The name of the newspaper is added as a payment method."""
        for rec in self:
            rec.payment_methods = ", ".join(
                rec.payment_ids.sudo().mapped("journal_id.name")
            )


class AccountPayment(models.Model):
    """The payment line is extended, thus adding fields and validations when paying with cards."""

    _inherit = "account.payment"

    account_card_ids = fields.Many2many(
        "res.tarjeta", string="Cards", related="journal_id.account_card_ids"
    )
    card_id = fields.Many2one(
        "res.tarjeta", string="Card", domain="[('id', 'in', account_card_ids)]"
    )
    card_type = fields.Selection(
        [("credit", "credit"), ("debit", "debit")], related="card_id.type"
    )
    magnet_bar = fields.Char("magnet bar")
    card_number = fields.Char("Number of card", size=4)
    tiket_number = fields.Integer("Number of Coupon")
    lot_number = fields.Integer("Number of Lot")
    ope_number = fields.Char("Operator")
    quota_number = fields.Char("Dues")
    date_end = fields.Char("Expiration date")
    fee = fields.Float(
        string="Fee",
        default=0,
    )
    total_amount = fields.Float(
        string="total amount",
        default=0,
    )
    sale_order_id = fields.Many2one(
        "sale.order",
        string="Order",
    )
    is_show = fields.Boolean("Show", default=False)
    is_show_data = fields.Boolean("Show", default=False)
    reconciliation_ids = fields.One2many(
        "credit.card.reconciliation",
        "payment_id",
        string="Reconciliation",
        domain=[("journal_id.account_card_ids", "!=", False)],
    )
    state_reconcile = fields.Selection(
        [("Pending", "Pending"), ("Settled", "Settled")],
        string="Coupon Status",
        default="Pending",
    )
    statement_id = fields.Many2one("account.bank.statement", string="Extracto Bancario")

    @api.onchange("card_id")
    def _compute_card_data(self):
        """Depending on the value in journal_id, it shows or not necessary fields for Card."""
        if self.card_id and self.payment_type == "inbound":
            self.is_show_data = True
        else:
            self.is_show_data = False

    @api.constrains("card_id", "card_number", "tiket_number", "lot_number")
    def _validate_tar_data(self):
        """Validation to not enter duplicate cards."""
        for rec in self:
            if rec.card_id.id and rec.payment_type == "inbound":
                res = rec.search(
                    [
                        ("card_id", "=", rec.card_id.id),
                        ("card_number", "=", rec.card_number),
                        ("tiket_number", "=", rec.tiket_number),
                        ("lot_number", "=", rec.lot_number),
                    ]
                )
                if len(res) > 1:
                    raise ValidationError(_("The card has already been entered"))

    @api.onchange("journal_id")
    def _compute_card(self):
        """Depending on the value in journal_id, it shows or not necessary fields for Card."""
        if self.journal_id.account_card_ids and self.payment_type == "inbound":
            self.is_show = True
        else:
            self.is_show = False
            self.is_show_data = False

    @api.onchange("lot_number")
    def _onchange_lot_number(self):
        """Validate that Lot Number is numerical."""
        if self.lot_number:
            if int(self.lot_number):
                self.lot_number = self.lot_number
            else:
                raise ValidationError(_("The Lot Number field must be numerical."))

    @api.onchange("tiket_number")
    def _onchange_tiket_number(self):
        """Validate that Coupon Number is numeric."""
        if self.tiket_number:
            if int(self.tiket_number):
                self.tiket_number = self.tiket_number
            else:
                raise ValidationError(_("The Coupon Number field must be numerical"))

    @api.onchange("card_number")
    def _onchange_card_number(self):
        """Validates that Card Number is 4 digits."""
        if self.card_number:
            if len(str(self.card_number)) == 4:
                self.card_number = self.card_number
            else:
                raise ValidationError(_("The card number must be 4 digits"))

    @api.model
    def create(self, vals):
        """Create a payment record and a reconciliation record if the payment is made with a card."""
        payment = super(AccountPayment, self).create(vals)
        if payment.journal_id.account_card_ids and payment.payment_type == "inbound":
            self.env["credit.card.reconciliation"].create_from_payment(payment)
        return payment


class CreditCardReconciliation(models.Model):
    """This model is used to reconcile the payment with the card."""

    _name = "credit.card.reconciliation"
    _description = "Credit Card Reconciliation"

    payment_id = fields.Many2one(
        "account.payment", string="Payment", ondelete="cascade", required=False
    )
    date = fields.Date(string="Date", store=True, compute="_compute_date")
    journal_id = fields.Many2one(
        "account.journal", string="Journal", store=True, compute="_compute_journal_id"
    )
    amount = fields.Monetary(string="Amount", store=True, compute="_compute_amount")
    currency_id = fields.Many2one(
        related="payment_id.currency_id", string="Currency", store=True
    )
    lot_number = fields.Integer(
        string="Number of Lot", store=True, compute="_compute_lot_number"
    )
    tiket_number = fields.Integer(
        string="Number of Coupon", store=True, compute="_compute_tiket_number"
    )
    card_number = fields.Char(
        string="Number of card", store=True, compute="_compute_card_number"
    )
    ope_number = fields.Char(
        string="Operator", store=True, compute="_compute_ope_number"
    )
    card_id = fields.Many2one(
        "res.tarjeta", string="Card", store=True, compute="_compute_card_id"
    )

    state_reconcile = fields.Selection(
        [("Pending", "Pending"), ("Settled", "Settled")],
        string="Coupon Status",
        default="Pending",
    )

    @api.depends("payment_id")
    def _compute_date(self):
        for rec in self:
            if rec.payment_id:
                rec.date = rec.payment_id.date

    @api.depends("payment_id")
    def _compute_journal_id(self):
        for rec in self:
            if rec.payment_id:
                rec.journal_id = rec.payment_id.journal_id

    @api.depends("payment_id")
    def _compute_amount(self):
        for rec in self:
            if rec.payment_id:
                rec.amount = rec.payment_id.amount

    @api.depends("payment_id")
    def _compute_lot_number(self):
        for rec in self:
            if rec.payment_id:
                rec.lot_number = rec.payment_id.lot_number

    @api.depends("payment_id")
    def _compute_tiket_number(self):
        for rec in self:
            if rec.payment_id:
                rec.tiket_number = rec.payment_id.tiket_number

    @api.depends("payment_id")
    def _compute_card_number(self):
        for rec in self:
            if rec.payment_id:
                rec.card_number = rec.payment_id.card_number

    @api.depends("payment_id")
    def _compute_ope_number(self):
        for rec in self:
            if rec.payment_id:
                rec.ope_number = rec.payment_id.ope_number

    @api.depends("payment_id")
    def _compute_card_id(self):
        for rec in self:
            if rec.payment_id:
                rec.card_id = rec.payment_id.card_id

    @api.model
    def fields_get(self, allfields=None, attributes=None):
        """Returns model fields attributes"""
        res = super(CreditCardReconciliation, self).fields_get(
            allfields=allfields, attributes=attributes
        )
        if res.get("date", False):
            res["date"]["readonly"] = False
        if res.get("journal_id", False):
            res["journal_id"]["readonly"] = False
        if res.get("amount", False):
            res["amount"]["readonly"] = False
        if res.get("lot_number", False):
            res["lot_number"]["readonly"] = False
        if res.get("tiket_number", False):
            res["tiket_number"]["readonly"] = False
        if res.get("card_number", False):
            res["card_number"]["readonly"] = False
        if res.get("ope_number", False):
            res["ope_number"]["readonly"] = False
        if res.get("card_id", False):
            res["card_id"]["readonly"] = False
        return res

    @api.model
    def create_from_payment(self, payment):
        """Create a reconciliation record from a payment."""
        if payment.journal_id.account_card_ids and payment.payment_type == "inbound":
            self.create(
                {
                    "payment_id": payment.id,
                }
            )
