"""Models for the fast.payment module"""
from odoo import api, fields, models


class FastPayment(models.Model):
    """Modesl for the fast.payment module"""

    _name = "fast.payment"
    _description = "Fast Payment"

    name = fields.Char(string="Número", readonly=True, copy=False, default="New")
    payment_type = fields.Selection(
        [("send", "Enviar"), ("receive", "Recibir")],
        string="Tipo de pago",
        required=True,
        default="receive",
    )

    partner_id = fields.Many2one(
        "res.partner",
        string="Cliente",
    )
    amount = fields.Monetary(string="Importe", required=True)
    currency_id = fields.Many2one(
        "res.currency",
        string="Moneda",
        required=True,
        default=lambda self: self.env.company.currency_id,
    )
    date = fields.Date(string="Fecha", required=True, default=fields.Date.context_today)
    memo = fields.Char(string="Memo")
    journal_id = fields.Many2one(
        "account.journal",
        string="Diario",
        required=True,
        domain=[("type", "in", ["bank", "cash"])],
    )
    company_id = fields.Many2one(
        "res.company", string="Compañía", default=lambda self: self.env.company
    )
    state = fields.Selection(
        [("draft", "Borrador"), ("posted", "Publicado"), ("cancelled", "Cancelado")],
        string="Estado",
        default="draft",
        required=True,
    )

    @api.model_create_multi
    def create(self, vals_list):
        """Create method for fast.payment"""
        for vals in vals_list:
            if vals.get("name", "New") == "New":
                vals["name"] = (
                    self.env["ir.sequence"].next_by_code("fast.payment") or "New"
                )
        return super().create(vals_list)

    def action_post(self):
        """action_post method for fast.payment"""
        self.ensure_one()
        if self.state != "draft":
            return

        # Crear asiento contable
        move_vals = self._prepare_move_values()
        move = self.env["account.move"].create(move_vals)
        move.action_post()

        self.write(
            {
                "state": "posted",
            }
        )

        return True

    def _prepare_move_values(self):
        """prepare_move_values method for fast.payment"""
        self.ensure_one()

        if self.payment_type == "receive":
            debit_account = self.journal_id.default_account_id
            credit_account = self.partner_id.property_account_receivable_id
        else:
            debit_account = self.partner_id.property_account_payable_id
            credit_account = self.journal_id.default_account_id

        move_lines = []

        move_lines.append(
            (
                0,
                0,
                {
                    "account_id": debit_account.id,
                    "partner_id": self.partner_id.id,
                    "debit": self.amount,
                    "credit": 0.0,
                    "currency_id": self.currency_id.id,
                    "date_maturity": self.date,
                },
            )
        )

        move_lines.append(
            (
                0,
                0,
                {
                    "account_id": credit_account.id,
                    "partner_id": self.partner_id.id,
                    "debit": 0.0,
                    "credit": self.amount,
                    "currency_id": self.currency_id.id,
                    "date_maturity": self.date,
                },
            )
        )

        return {
            "journal_id": self.journal_id.id,
            "date": self.date,
            "ref": self.memo or "",
            "line_ids": move_lines,
            "company_id": self.company_id.id,
            "move_type": "entry",
        }
