"""Module for the Sale Order model."""
from odoo import _, models
from odoo.exceptions import UserError


class SaleOrder(models.Model):
    """Class to manage Sale Orders."""

    _inherit = "sale.order"

    def _prepare_invoice(self):
        """Override to add sales team journal"""
        invoice_vals = super()._prepare_invoice()
        if not self.provisional:
            if not self.team_id.journal_id:
                raise UserError(
                    _(
                        "You must set the Sales Journal in the Sales Team form to be able to issue an Invoice"
                    )
                )

            invoice_vals["journal_id"] = self.team_id.journal_id.id
        return invoice_vals

    def action_fast_payment(self):
        """
        Action triggered by the Fast Payment button.
        Add your custom fast payment logic here.
        """
        self.ensure_one()

        # Validate Tipo de responsabilidad AFIP
        if not self.partner_id.l10n_ar_afip_responsibility_type_id:
            raise UserError(
                _(
                    "The AFIP Responsibility Type field is required. Please set this parameter in the customer form before processing the fast payment."
                )
            )

        # Validate Tipo y número de Identificación
        if (
            not self.partner_id.l10n_latam_identification_type_id
            or not self.partner_id.vat
        ):
            raise UserError(
                _(
                    "The Identification Type and Number fields are required. Please set these parameters in the customer form before processing the fast payment."
                )
            )

        # Validate Sales Team Journal
        if not self.team_id.journal_id:
            raise UserError(
                _(
                    "You must set the Sales Journal in the Sales Team form to be able to issue an Invoice"
                )
            )

        # Check stock availability
        stock_picking = self.picking_ids.filtered(lambda p: p.state != "done")
        if stock_picking:
            stock_picking.action_assign()
            if any(
                move.forecast_availability != move.product_uom_qty
                for move in stock_picking.move_ids_without_package
            ):
                raise UserError(
                    _(
                        "One or more products related to the sales order do not have available stock in their warehouse. You must verify inventory as you cannot issue an Invoice without available stock."
                    )
                )

        # Display Wizard for Payment Lines
        view = self.env.ref("nybble_fast_payment.view_fast_payment_wizard")
        context = {
            "default_sale_order_id": self.id,
            "default_amount_total": self.amount_total,
        }
        return {
            "name": _("Fast Payment"),
            "type": "ir.actions.act_window",
            "view_mode": "form",
            "res_model": "fast.payment.wizard",
            "views": [(view.id, "form")],
            "view_id": view.id,
            "target": "new",
            "context": context,
        }
