"""Tests for the sale.order model"""
from odoo.exceptions import AccessError, UserError
from odoo.tests.common import TransactionCase, tagged


@tagged("post_install", "-at_install")
class TestFastPayment(TransactionCase):
    """Tests for the fast payment functionality in sale orders"""

    def setUp(self):
        """Set up test data"""
        super().setUp()

        # Create base data
        self.partner = self.env["res.partner"].create(
            {
                "name": "Test Partner",
                "email": "test@example.com",
            }
        )

        self.product = self.env["product.product"].create(
            {
                "name": "Test Product",
                "type": "product",
                "list_price": 100.0,
            }
        )

        # Create users with different permissions
        self.user_no_access = self.env["res.users"].create(
            {
                "name": "User No Access",
                "login": "user_no_access",
                "email": "no_access@example.com",
                "groups_id": [(6, 0, [self.env.ref("base.group_user").id])],
            }
        )

        self.user_sale_only = self.env["res.users"].create(
            {
                "name": "User Sales Only",
                "login": "user_sale_only",
                "email": "sale_only@example.com",
                "groups_id": [
                    (
                        6,
                        0,
                        [
                            self.env.ref("base.group_user").id,
                            self.env.ref("sales_team.group_sale_salesman").id,
                        ],
                    )
                ],
            }
        )

        self.user_fast_payment = self.env["res.users"].create(
            {
                "name": "User Fast Payment",
                "login": "user_fast_payment",
                "email": "fast_payment@example.com",
                "groups_id": [
                    (
                        6,
                        0,
                        [
                            self.env.ref("base.group_user").id,
                            self.env.ref("sales_team.group_sale_salesman").id,
                            self.env.ref("nybble_fast_payment.group_fast_payment").id,
                        ],
                    )
                ],
            }
        )

        # Create a test sale order
        self.sale_order = self.env["sale.order"].create(
            {
                "partner_id": self.partner.id,
                "order_line": [
                    (
                        0,
                        0,
                        {
                            "product_id": self.product.id,
                            "product_uom_qty": 1,
                            "price_unit": 100.0,
                        },
                    )
                ],
            }
        )

    def test_01_button_visibility(self):
        """Test the visibility conditions of the fast payment button"""
        # Confirm the order to set state to 'sale'
        self.sale_order.action_confirm()

        # Test with user without fast payment access
        with self.assertRaises(AccessError):
            self.sale_order.with_user(self.user_no_access).action_fast_payment()

        # Test with sales user without fast payment access
        with self.assertRaises(AccessError):
            self.sale_order.with_user(self.user_sale_only).action_fast_payment()

        # Test with fast payment user - should work
        try:
            self.sale_order.with_user(self.user_fast_payment).action_fast_payment()
        except AccessError:
            self.fail("Fast payment user should have access to the action")

    def test_02_order_states(self):
        """Test button behavior in different order states"""
        # Test in draft state
        with self.assertRaises(UserError):
            self.sale_order.with_user(self.user_fast_payment).action_fast_payment()

        # Test in sale state
        self.sale_order.action_confirm()
        try:
            self.sale_order.with_user(self.user_fast_payment).action_fast_payment()
        except UserError:
            self.fail("Should allow fast payment in 'sale' state")

        # Test in done state
        self.sale_order.action_done()
        try:
            self.sale_order.with_user(self.user_fast_payment).action_fast_payment()
        except UserError:
            self.fail("Should allow fast payment in 'done' state")

    def test_03_invoice_status(self):
        """Test button behavior with different invoice statuses"""
        self.sale_order.action_confirm()

        # Test with 'to invoice' status
        self.assertEqual(self.sale_order.invoice_status, "to invoice")
        try:
            self.sale_order.with_user(self.user_fast_payment).action_fast_payment()
        except UserError:
            self.fail("Should allow fast payment when invoice_status is 'to invoice'")

        # Create and post invoice
        invoice = self.sale_order._create_invoices()
        invoice.action_post()

        # Test with 'invoiced' status
        self.assertEqual(self.sale_order.invoice_status, "invoiced")
        with self.assertRaises(UserError):
            self.sale_order.with_user(self.user_fast_payment).action_fast_payment()

    def test_04_multiple_orders(self):
        """Test fast payment with multiple orders"""
        # Create second order
        second_order = self.sale_order.copy()

        # Confirm both orders
        self.sale_order.action_confirm()
        second_order.action_confirm()

        # Test fast payment on both orders
        orders = self.sale_order + second_order
        with self.assertRaises(ValueError):
            # Should raise error when trying to process multiple orders
            orders.with_user(self.user_fast_payment).action_fast_payment()

    def test_05_no_order_lines(self):
        """Test fast payment with empty order"""
        empty_order = self.env["sale.order"].create(
            {
                "partner_id": self.partner.id,
            }
        )
        empty_order.action_confirm()

        # Should still work with empty order
        try:
            empty_order.with_user(self.user_fast_payment).action_fast_payment()
        except UserError:
            self.fail("Should allow fast payment on empty orders")
