"""Class definition for the Fast Payment Wizard."""
from odoo import _, api, fields, models
from odoo.exceptions import UserError
from odoo.tools.float_utils import float_compare


class FastPaymentWizard(models.TransientModel):
    """Fast Payment Wizard."""

    _name = "fast.payment.wizard"
    _description = "Fast Payment Wizard"

    sale_order_id = fields.Many2one(
        "sale.order", string="Sales Order", required=True, readonly=True
    )
    currency_id = fields.Many2one("res.currency", string="Currency", readonly=True)
    amount_total = fields.Monetary(
        string="Total Amount",
        required=True,
        readonly=True,
        currency_field="currency_id",
    )
    date = fields.Date(string="Fecha", default=fields.Date.context_today)
    payment_method = fields.Selection(
        [("manual", "Manual"), ("other", "Otro")],
        string="Método de pago",
        default="manual",
    )
    card_payment_ids = fields.One2many(
        "fast.payment.card.line", "wizard_id", string="Líneas de Pago con Tarjeta"
    )

    card_payment_check_ids = fields.One2many(
        "fast.payment.check.line", "wizard_id", string="Líneas de Pago con Cheque"
    )
    journal_id = fields.Many2one(
        "account.journal", string="Diario", domain="[('type', 'in', ['bank', 'cash'])]"
    )
    payment_line_ids = fields.One2many(
        "fast.payment.line", "wizard_id", string="Payment Lines"
    )
    account_card_ids = fields.Many2many(
        "res.tarjeta", string="Cards", related="journal_id.account_card_ids"
    )

    card_id = fields.Many2one(
        "res.tarjeta", string="Card", domain="[('id', 'in', account_card_ids)]"
    )
    card_type = fields.Selection(
        [("credit", "credit"), ("debit", "debit")], related="card_id.type"
    )
    magnet_bar = fields.Char("magnet bar")
    card_number = fields.Char("Number of card", size=4)
    tiket_number = fields.Integer("Number of Coupon")
    lot_number = fields.Integer("Number of Lot")
    ope_number = fields.Char("Operator")
    quota_number = fields.Char("Dues")
    date_end = fields.Char("Expiration date")
    has_card_payments = fields.Boolean(
        string="Has Card Payments", compute="_compute_has_card_payments", store=True
    )
    show_cheque_info = fields.Boolean(compute="_compute_show_cheque_info")
    has_check_payments = fields.Boolean(
        string="Has Check Payments", compute="_compute_has_check_payments", store=True
    )
    total_payments = fields.Monetary(
        string="Total de Pagos",
        compute="_compute_total_payments",
        currency_field="currency_id",
    )
    computed_difference = fields.Monetary(
        string="Diferencia de Pago",
        compute="_compute_total_payments",
        currency_field="currency_id",
    )
    show_paid = fields.Boolean(
        string="Mostrar Vuelto", compute="_compute_total_payments"
    )

    @api.depends("payment_line_ids.amount")
    def _compute_total_payments(self):
        """Compute total payments and difference."""
        for wizard in self:
            total = sum(wizard.payment_line_ids.mapped("amount"))
            wizard.total_payments = total
            wizard.computed_difference = wizard.amount_total - total
            wizard.show_paid = total > wizard.amount_total

    @api.onchange("journal_id")
    def _compute_show_cheque_info(self):
        """Compute show cheque info."""
        for record in self:
            record.show_cheque_info = any(
                line.name == "Nuevo Cheque de Tercero"
                for line in record.journal_id.inbound_payment_method_line_ids
            )

    @api.depends("payment_line_ids.journal_id.account_card_ids")
    def _compute_account_card_ids(self):
        """Compute account card ids."""
        for record in self:
            record.account_card_ids = record.payment_line_ids.mapped(
                "journal_id.account_card_ids"
            )

    @api.depends("payment_line_ids.has_cards")
    def _compute_has_card_payments(self):
        """Compute has card payments."""
        for record in self:
            record.has_card_payments = any(
                line.has_cards for line in record.payment_line_ids
            )

    @api.depends("payment_line_ids.has_checks")
    def _compute_has_check_payments(self):
        """Compute has check payments."""
        for record in self:
            record.has_check_payments = any(
                line.has_checks for line in record.payment_line_ids
            )

    def action_confirm_fast_payment(self):
        """Confirm fast payment."""
        self.ensure_one()

        total_payments = sum(self.payment_line_ids.mapped("amount"))
        if float_compare(total_payments, self.amount_total, precision_digits=2) != 0:
            raise UserError(
                _("El total de los pagos debe ser igual al total de la orden")
            )

        sales_team = self.sale_order_id.team_id
        if not sales_team or not sales_team.journal_id:
            raise UserError(
                _("El equipo de ventas debe tener configurado un diario de facturación")
            )
        with self.env.cr.savepoint():
            invoice = False
            try:
                existing_invoice = self.sale_order_id.invoice_ids.filtered(
                    lambda x: x.state in ["draft", "posted"]
                )

                if existing_invoice:
                    invoice = existing_invoice[0]
                    if invoice.state == "draft":
                        invoice.write({"journal_id": sales_team.journal_id.id})
                        invoice.sudo().action_post()
                else:
                    ctx = {
                        "default_journal_id": sales_team.journal_id.id,
                        "force_journal_id": sales_team.journal_id.id,
                        "default_move_type": "out_invoice",
                        "default_sale_order_id": self.sale_order_id.id,
                        "company_id": self.env.company.id,
                    }
                    invoice = (
                        self.sale_order_id.with_context(**ctx).sudo()._create_invoices()
                    )
                    if invoice:
                        invoice = invoice[0]
                        invoice.sudo().action_post()

                if not invoice or not invoice.exists() or invoice.state != "posted":
                    raise UserError(
                        _(
                            "Error: La factura debe estar confirmada antes de proceder con el pago"
                        )
                    )
                payment_group = self._create_payment_group_and_payments(invoice)
                picking = self._create_picking()
            except Exception as e:
                raise UserError(_("Error en el proceso de facturación: %s") % str(e))
        return {
            "type": "ir.actions.act_window",
            "res_model": "sale.order",
            "view_mode": "form",
            "res_id": self.sale_order_id.id,
            "target": "current",
        }

    def _create_picking(self):
        """Create picking."""
        try:
            stock_picking = self.sale_order_id.picking_ids.filtered(
                lambda p: p.state != "done"
            )
            if stock_picking:
                stock_picking.sudo().button_validate()
        except Exception as e:
            raise UserError(_("Error al validar el albarán: %s") % str(e))

    def _create_payment_group_and_payments(self, invoice):
        """Create payment group and payments."""
        try:
            invoice = self.env["account.move"].sudo().browse(invoice.id)
            if not invoice.exists():
                raise UserError(_("La factura no existe o ha sido eliminada"))

            payment_group_vals = {
                "name": self.env["ir.sequence"].next_by_code("account.payment.group")
                or _("New"),
                "partner_id": self.sale_order_id.partner_id.id,
                "partner_type": "customer",
                "receiptbook_id": 13,
                "payment_date": self.date,
                "currency_id": self.sale_order_id.currency_id.id
                or self.env.company.currency_id.id,
                "payment_methods": "multi",
                "notes": f"Pago rápido para orden {self.sale_order_id.name}",
                "company_id": self.env.company.id,
                "selected_debt": invoice.amount_total,
            }

            try:
                payment_group = (
                    self.env["account.payment.group"].sudo().create(payment_group_vals)
                )
            except Exception as e:
                raise UserError(_("Error al crear el grupo de pago: %s") % str(e))
            receivable_lines = invoice.line_ids.filtered(
                lambda x: x.account_id.account_type == "asset_receivable"
            )
            if not receivable_lines:
                raise UserError(_("No se encontró una cuenta por cobrar en la factura"))

            payments_vals_list = []
            for line in self.payment_line_ids:
                journal = line.journal_id
                if not journal.exists():
                    raise UserError(
                        _("El diario de pago no existe o ha sido eliminado")
                    )

                payment_method_line = (
                    journal.inbound_payment_method_line_ids[0]
                    if journal.inbound_payment_method_line_ids
                    else False
                )
                if not payment_method_line:
                    raise UserError(
                        _("El diario %s no tiene métodos de pago configurados")
                        % journal.name
                    )

                payment_vals = {
                    "payment_group_id": payment_group.id,
                    "payment_type": "inbound",
                    "partner_type": "customer",
                    "partner_id": self.sale_order_id.partner_id.id,
                    "amount": line.amount,
                    "currency_id": self.sale_order_id.currency_id.id
                    or self.env.company.currency_id.id,
                    "date": line.date,
                    "journal_id": journal.id,
                    "payment_method_line_id": payment_method_line.id,
                    "ref": f"Pago factura {invoice.name}",
                    "destination_account_id": receivable_lines[0].account_id.id,
                    "reconciled_invoice_ids": [(4, invoice.id)],
                }

                if line.has_cards:
                    for card_line in self.card_payment_ids:
                        payment_vals.update(
                            {
                                "card_id": card_line.card_id.id,
                                "card_number": card_line.card_number,
                                "tiket_number": card_line.tiket_number,
                                "lot_number": card_line.lot_number,
                                "ope_number": card_line.ope_number,
                                "date_end": card_line.date_end,
                            }
                        )
                check_line = self.card_payment_check_ids.filtered(
                    lambda x: x.selected_journal_id.id == line.journal_id.id
                )
                if check_line and check_line.selected_journal_id.exists():
                    journal = check_line.selected_journal_id
                    payment_method_line = (
                        journal.inbound_payment_method_line_ids[0]
                        if journal.inbound_payment_method_line_ids
                        else False
                    )
                    if not payment_method_line:
                        raise UserError(
                            _(
                                "El diario del cheque no tiene métodos de pago configurados"
                            )
                        )
                if check_line:
                    for check_line in self.card_payment_check_ids:
                        payment_vals.update(
                            {
                                "l10n_latam_check_number": check_line.cheque_number,
                                "l10n_latam_check_payment_date": check_line.cheque_payment_date,
                                "l10n_latam_check_bank_id": check_line.cheque_bank_id.id
                                if check_line.cheque_bank_id.exists()
                                else False,
                                "l10n_latam_check_issuer_vat": check_line.cheque_sender_cuit,
                                "l10n_ar_partner_vat": check_line.cheque_receiver_cuit,
                            }
                        )

                payments_vals_list.append((payment_vals, line))
            created_payments = []
            try:
                for payment_vals, line in payments_vals_list:
                    payment = (
                        self.env["account.payment"]
                        .sudo()
                        .with_context(
                            default_payment_type="inbound",
                            skip_card_reconciliation=True,
                        )
                        .create(payment_vals)
                    )
                    created_payments.append(payment)
                    line.payment_id = payment.id
            except Exception as e:
                raise UserError(_("Error al crear los pagos: %s") % str(e))

            try:
                payment_group.sudo().post()
            except Exception as e:
                raise UserError(str(e))
            return payment_group

        except Exception as e:
            raise UserError(_("Error al procesar los pagos: %s") % str(e))


class FastPaymentLine(models.TransientModel):
    """Fast Payment Line."""

    _name = "fast.payment.line"
    _description = "Fast Payment Line"

    wizard_id = fields.Many2one("fast.payment.wizard", string="Wizard")
    date = fields.Date(string="Fecha", default=fields.Date.context_today)
    payment_method = fields.Selection(
        [("manual", "Manual"), ("other", "Otro")],
        string="Método de pago",
        default="manual",
    )
    journal_id = fields.Many2one(
        "account.journal", string="Diario", domain="[('type', 'in', ['bank', 'cash'])]"
    )
    memo = fields.Char(string="Memo")
    amount = fields.Monetary(string="Importe", currency_field="currency_id")
    currency_id = fields.Many2one(
        related="wizard_id.currency_id", store=True, string="Moneda", readonly=True
    )
    payment_id = fields.Many2one("account.payment", string="Pago")
    partner_id = fields.Many2one(
        "res.partner",
        related="wizard_id.sale_order_id.partner_id",
        string="Cliente",
        readonly=True,
    )
    has_cards = fields.Boolean(
        string="Tiene Tarjetas", compute="_compute_has_cards", store=True
    )
    has_checks = fields.Boolean(
        string="Tiene cheque", compute="_compute_has_checks", store=True
    )
    account_card_ids = fields.Many2many(
        "res.tarjeta",
        related="journal_id.account_card_ids",
        string="Tarjetas disponibles",
        readonly=True,
    )
    card_id = fields.Many2one("res.tarjeta", string="Tarjeta")
    magnet_bar = fields.Char("magnet bar")
    card_number = fields.Char("Number of card", size=4)
    tiket_number = fields.Integer("Number of Coupon")
    lot_number = fields.Integer("Number of Lot")
    ope_number = fields.Char("Operator")
    quota_number = fields.Char("Dues")
    date_end = fields.Char("Expiration date")
    card_type = fields.Selection(
        [("credit", "credit"), ("debit", "debit")], related="card_id.type"
    )

    @api.depends("journal_id")
    def _compute_has_cards(self):
        """Compute has cards."""
        for line in self:
            line.has_cards = bool(line.journal_id.account_card_ids)

    @api.depends("journal_id")
    def _compute_has_checks(self):
        """Compute has checks."""
        for line in self:
            line.has_checks = any(
                line.name == "Nuevo Cheque de Tercero"
                for line in line.journal_id.inbound_payment_method_line_ids
            )


class FastPaymentCardLine(models.TransientModel):
    """Fast Payment Card Line."""

    _name = "fast.payment.card.line"

    wizard_id = fields.Many2one("fast.payment.wizard")

    journal_id = fields.Many2one(
        "account.journal", string="Diario", domain="[('type', 'in', ['bank', 'cash'])]"
    )
    account_card_ids = fields.Many2many(
        "res.tarjeta", string="Cards", compute="_compute_account_card_ids"
    )
    card_id = fields.Many2one(
        "res.tarjeta", string="Card", domain="[('id', 'in', account_card_ids)]"
    )
    card_number = fields.Char(string="Número Tarjeta")
    tiket_number = fields.Char(string="Cupón")
    lot_number = fields.Char(string="Número de lote")
    ope_number = fields.Char(string="Operador")
    date_end = fields.Date(string="Fecha expiración")
    journal_ids = fields.Many2many(
        "account.journal",
        string="Diarios",
        domain="[('type', 'in', ['bank', 'cash'])]",
        compute="_compute_journal_ids",
        store=True,
    )
    selected_journal_id = fields.Many2one(
        "account.journal",
        string="Diario Seleccionado",
        domain="[('id', 'in', journal_ids)]",
    )

    @api.depends("wizard_id.payment_line_ids.journal_id")
    def _compute_journal_ids(self):
        """Compute journal ids."""
        for record in self:
            if record.wizard_id.payment_line_ids:
                record.journal_ids = record.wizard_id.payment_line_ids.mapped(
                    "journal_id"
                )

    @api.depends("selected_journal_id")
    def _compute_account_card_ids(self):
        """Compute account card ids."""
        for record in self:
            if record.selected_journal_id:
                record.account_card_ids = record.selected_journal_id.account_card_ids
            else:
                record.account_card_ids = self.env["res.tarjeta"].browse([])


class FastPaymentCheckLine(models.TransientModel):
    """Fast Payment Check Line."""

    _name = "fast.payment.check.line"

    wizard_id = fields.Many2one("fast.payment.wizard")
    cheque_number = fields.Char(string="Número de Cheque")
    cheque_payment_date = fields.Date(string="Fecha de Pago del Cheque")
    cheque_bank_id = fields.Many2one("res.bank", string="Banco del Cheque")
    cheque_sender_cuit = fields.Char(string="CUIT Emisor del Cheque")
    cheque_receiver_cuit = fields.Char(
        string="CUIT Destinatario", default=lambda self: self.env.company.vat
    )
    journal_ids = fields.Many2many(
        "account.journal",
        string="Diarios",
        domain="[('type', 'in', ['bank', 'cash'])]",
        compute="_compute_journal_ids",
        store=True,
    )
    selected_journal_id = fields.Many2one(
        "account.journal",
        string="Diario Seleccionado",
        domain="[('id', 'in', journal_ids)]",
    )

    @api.depends("wizard_id.payment_line_ids.journal_id")
    def _compute_journal_ids(self):
        """Compute journal ids."""
        for record in self:
            if record.wizard_id.payment_line_ids:
                record.journal_ids = record.wizard_id.payment_line_ids.mapped(
                    "journal_id"
                )
