"""This module contains the custom implementation of the `account.move` model."""
import xml.etree.cElementTree as ET

from odoo import _, api, fields, models
from odoo.exceptions import ValidationError


class AccountMove(models.Model):
    """
    This class represents an accounting move in the system.
    It inherits from the base `account.move` model and adds additional functionality.
    """

    _order = "id desc"
    _inherit = ["account.move"]

    provisional = fields.Boolean(
        "On behalf of",
    )

    @api.model
    def get_view(self, view_id=None, view_type="form", **options):
        """Oculta el campo provisional para el grupo group_user."""
        res = super().get_view(view_id, view_type, **options)
        group_user = self.env.user.has_group(
            "nybble_internal_voucher.group_voucher_user"
        )
        group_manager = self.env.user.has_group(
            "nybble_internal_voucher.group_voucher_admin"
        )
        if not group_manager and (group_user):
            if view_type in ["form"]:
                root = ET.fromstring(res["arch"])
                for el in root.iter("field"):
                    if el.attrib.get("name") in ["provisional"]:
                        el.attrib.update(
                            {
                                "modifiers": '{"invisible": [["id", "!=", 0]]}',
                            }
                        )
                res.update({"arch": ET.tostring(root, encoding="utf8", method="xml")})
        return res

    def _search_default_journal(self):
        if self.payment_id and self.payment_id.journal_id:
            return self.payment_id.journal_id
        if self.statement_line_id and self.statement_line_id.journal_id:
            return self.statement_line_id.journal_id
        if self.statement_line_ids.statement_id.journal_id:
            return self.statement_line_ids.statement_id.journal_id[:1]

        journal_types = self._get_valid_journal_types()
        company_id = (self.company_id or self.env.company).id
        group_user = self.env.user.has_group(
            "nybble_internal_voucher.group_voucher_user"
        )
        group_manager = self.env.user.has_group(
            "nybble_internal_voucher.group_voucher_admin"
        )

        if not group_manager and group_user:
            domain = [
                ("provisional", "=", False),
                ("company_id", "=", company_id),
                ("type", "in", journal_types),
            ]
        else:
            domain = [
                ("company_id", "=", company_id),
                ("provisional", "=", False),
                ("type", "in", journal_types),
            ]

        journal = None
        # the currency is not a hard dependence, it triggers via manual add_to_compute
        # avoid computing the currency before all it's dependences are set (like the journal...)
        if self.env.cache.contains(self, self._fields["currency_id"]):
            currency_id = self.currency_id.id or self._context.get(
                "default_currency_id"
            )
            if currency_id and currency_id != self.company_id.currency_id.id:
                currency_domain = domain + [("currency_id", "=", currency_id)]
                journal = self.env["account.journal"].search(currency_domain, limit=1)

        if not journal:
            journal = self.env["account.journal"].search(domain, limit=1)

        if not journal:
            company = self.env["res.company"].browse(company_id)

            error_msg = _(
                "No journal could be found in company %(company_name)s for any of those types: %(journal_types)s",
                company_name=company.display_name,
                journal_types=", ".join(journal_types),
            )
            raise UserError(error_msg)

        return journal

    def action_post(self):
        """
        Validates that the Journal selected in Invoice is
        provisional if the Originating Order is provisional.
        Validates that the Journal selected in Invoice is Fiscal
        if the originating Order is Fiscal
        """
        if self.env.context.get("default_type"):
            context = dict(self.env.context)
            del context["default_type"]
            self = self.with_context(context)
        if self.invoice_origin:
            if self.journal_id.type == "sale":
                order = self.env["sale.order"].search(
                    [("name", "=", self.invoice_origin)]
                )
            elif self.journal_id.type == "purchase":
                order = self.env["purchase.order"].search(
                    [("name", "=", self.invoice_origin)]
                )
            if order.provisional and self.journal_id.provisional == False:
                raise ValidationError(
                    _(
                        "The original sales order %s from which invoice %s is derived "
                        "is marked provisional but journal %s is not."
                        " An invoice that derives from a provisional purchase order "
                        "must be related to a provisional journal as well. "
                        "Correct this error and try confirming the invoice again.",
                        order.name,
                        self,
                        self.journal_id.name,
                    )
                )
            elif self.journal_id.provisional and order.provisional == False:
                raise ValidationError(
                    _(
                        "The journal %s asociated to the invoice %s is provisional"
                        " but the original order is not. Correct this error and "
                        "try confirming the invoice again.",
                        self.journal_id.name,
                        self,
                    )
                )
        return super().action_post()


class AccountMoveLine(models.Model):
    """Adds necessary model to apply to user rules."""

    _order = "id desc"
    _inherit = ["account.move.line"]


class ResPartner(models.Model):
    """Adds necessary fields to apply to user rules."""

    _order = "id desc"
    _inherit = ["res.partner"]

    def _get_unreconciled_aml_domain(self):
        res = super()._get_unreconciled_aml_domain()
        group_user = self.env.user.has_group(
            "nybble_internal_voucher.group_voucher_user"
        )
        group_manager = self.env.user.has_group(
            "nybble_internal_voucher.group_voucher_admin"
        )
        if not group_manager and (group_user):
            res.append(("move_id.journal_id.provisional", "=", False))
        return res
