"""Models for the nybble_internal_voucher module"""
from odoo import _, api, fields, models
from odoo.exceptions import ValidationError


class AccountPaymentGroup(models.Model):
    """Adds necessary fields to apply to user rules"""

    _inherit = "account.payment.group"

    provisional = fields.Boolean(
        "On behalf of",
        default=False,
        # groups="!nybble_internal_voucher.group_voucher_user",
    )

    def post(self):
        """Post payment group."""
        active_ids = self.env.context.get("active_ids")
        move_ids = self.env["account.move"].browse(active_ids)
        created_automatically = self._context.get("created_automatically")
        posted_payment_groups = self.filtered(lambda x: x.state == "posted")
        if posted_payment_groups:
            raise ValidationError(
                _(
                    "You can't post and already posted payment group. Payment group ids: %s"
                )
                % posted_payment_groups.ids
            )
        for rec in self:
            if not rec.document_number:
                if rec.receiptbook_id and not rec.receiptbook_id.sequence_id:
                    raise ValidationError(
                        _(
                            "Error!. Please define sequence on the receiptbook"
                            " related documents to this payment or set the "
                            "document number."
                        )
                    )
                if rec.receiptbook_id.sequence_id:
                    rec.document_number = rec.receiptbook_id.with_context(
                        ir_sequence_date=rec.payment_date
                    ).sequence_id.next_by_id()
            rec.payment_ids.l10n_latam_document_type_id = rec.document_type_id.id

            if not rec.payment_ids:
                raise ValidationError(
                    _("You can not confirm a payment group without payment lines!")
                )
            if (
                rec.payment_subtype == "double_validation"
                and rec.payment_difference
                and not created_automatically
            ):
                raise ValidationError(
                    _("To Pay Amount and Payment Amount must be equal!")
                )
            rec.payment_ids.filtered(
                lambda p: p.partner_id != rec.partner_id.commercial_partner_id
            ).write({"partner_id": rec.partner_id.commercial_partner_id.id})

            if not created_automatically:
                for pay in rec.payment_ids.filtered(lambda x: x.state == "draft"):
                    pay.action_post()
            rec.payment_ids.name = rec.name

            if not rec.receiptbook_id and not rec.name:
                rec.name = (
                    any(rec.payment_ids.mapped("name"))
                    and ", ".join(rec.payment_ids.mapped("name"))
                    or False
                )

            posted_payments = rec.payment_ids.filtered(lambda x: x.state == "posted")
            if not created_automatically and posted_payments:
                counterpart_aml = posted_payments.mapped("line_ids").filtered(
                    lambda r: not r.reconciled
                    and r.account_id.account_type
                    in ("liability_payable", "asset_receivable")
                )
                if counterpart_aml and rec.to_pay_move_line_ids:
                    (counterpart_aml + (rec.to_pay_move_line_ids)).reconcile()

            rec.state = "posted"

            if rec.receiptbook_id.mail_template_id:
                rec.message_post_with_template(rec.receiptbook_id.mail_template_id.id)
            if move_ids and move_ids.exists() and move_ids.journal_id.provisional:
                rec.provisional = True
        return True

    def write(self, vals):
        """Write payment group."""
        active_ids = self.env.context.get("active_ids")
        move_ids = self.env["account.move"].browse(active_ids)
        mov_id = self.env["account.move"].search([("id", "=", move_ids.id)])
        provisional = False
        if mov_id:
            if move_ids.journal_id.provisional:
                provisional = True
            val = {
                "provisional": provisional,
            }
            vals.update(val)
        res = super(AccountPaymentGroup, self).write(vals)
        return res

    @api.model
    def create(self, vals):
        """Create payment group."""
        res = super(AccountPaymentGroup, self).create(vals)
        res.update({"name": res.receiptbook_id.sequence_id._next_do()})
        return res


class AccountPayment(models.Model):
    """Adds necessary fields to apply to user rules"""

    _inherit = "account.payment"

    @api.depends("payment_type", "payment_group_id")
    def _compute_available_journal_ids(self):
        """Compute available journal ids."""
        active_ids = self.env.context.get("move_id")
        move_id = self.env["account.move"].browse(active_ids)
        self.ensure_one()
        if move_id.journal_id.provisional:
            journals = self.env["account.journal"].search(
                [
                    ("company_id", "in", self.env.companies.ids),
                    ("provisional", "=", True),
                    ("type", "in", ("bank", "cash")),
                ]
            )
        else:
            journals = self.env["account.journal"].search(
                [
                    ("company_id", "in", self.env.companies.ids),
                    ("provisional", "=", False),
                    ("type", "in", ("bank", "cash")),
                ]
            )
        for pay in self:
            filtered_domain = (
                [("inbound_payment_method_line_ids", "!=", False)]
                if pay.payment_type == "inbound"
                else [("outbound_payment_method_line_ids", "!=", False)]
            )
            if pay.payment_group_id:
                filtered_domain.append(
                    ("company_id", "=", pay.payment_group_id.company_id.id)
                )
            pay.available_journal_ids = journals.filtered_domain(filtered_domain)

    def get_journals_domain(self):
        """Get journals domain."""
        active_ids = self.env.context.get("move_id")
        move_id = self.env["account.move"].browse(active_ids)
        self.ensure_one()
        domain = [("type", "in", ("bank", "cash"))]
        if self.payment_type == "inbound":
            domain.append(("at_least_one_inbound", "=", True))
            if move_id.journal_id.provisional:
                domain.append(("provisional", "=", True))
            else:
                domain.append(("provisional", "=", False))
        elif self.payment_type == "outbound":
            domain.append(("at_least_one_outbound", "=", True))
            if move_id.journal_id.provisional:
                domain.append(("provisional", "=", True))
            else:
                domain.append(("provisional", "=", False))
        return domain
