import xml.etree.cElementTree as ET

from odoo import _, api, fields, models
from odoo.exceptions import ValidationError


class PurchaseOrder(models.Model):
    """Add provisional field in Purchase Budget / Purchase Order."""

    _inherit = "purchase.order"

    provisional = fields.Boolean(
        "On behalf of",
    )

    @api.model
    def get_view(self, view_id=None, view_type="form", **options):
        """Oculta el campo provisional para el grupo group_user."""
        res = super().get_view(view_id, view_type, **options)
        group_user = self.env.user.has_group(
            "nybble_internal_voucher.group_voucher_user"
        )
        group_manager = self.env.user.has_group(
            "nybble_internal_voucher.group_voucher_admin"
        )
        if not group_manager and (group_user):
            if view_type in ["form"]:
                root = ET.fromstring(res["arch"])
                for el in root.iter("field"):
                    if el.attrib.get("name") in ["provisional"]:
                        el.attrib.update(
                            {
                                "modifiers": '{"invisible": [["id", "!=", 0]]}',
                            }
                        )
                res.update({"arch": ET.tostring(root, encoding="utf8", method="xml")})
        return res

    # Override _prepare_picking() method to set provisional field in stock.picking
    def _prepare_picking(self):
        """Add provisional field in Purchase Budget / Purchase Order."""
        res = super()._prepare_picking()
        res.update({"provisional": self.provisional})
        return res

    def button_confirm(self):
        """Add provisional field in Purchase Budget / Purchase Order."""
        res = super().button_confirm()
        if self.provisional:
            for line in self.order_line:
                company = self.company_id
                config_settings = (
                    self.env["res.config.settings"]
                    .sudo()
                    .search([("company_id", "=", company.id)], limit=1)
                )
                # Check if the configuration was found
                if config_settings:
                    # Get the internal sales tax from the configuration
                    tax_id = config_settings.internal_voucher_purchase_tax
                else:
                    tax = self.env["account.tax"].search(
                        [
                            ("type_tax_use", "=", "purchase"),
                            ("provisional", "=", True),
                            ("company_id", "=", company.id),
                        ],
                        limit=1,
                    )
                    tax_id = tax[0]
                line.write({"taxes_id": [(6, 0, [tax_id.id])]})
        return res


class PurchaseOrderLine(models.Model):
    _inherit = "purchase.order.line"

    def _prepare_account_move_line(self, move=False):
        """
        PO line functionality is added to determine which journal
        and tax apply in case of internal receipt.
        Extends function and adds in the case of a marked Order
        As provisional, the VAT tax line not taxed.
        It also adds in the Order marked as provisional, the Journal
        of sale marked as provisional.
        """
        res = super()._prepare_account_move_line(move)
        if self.order_id.provisional:
            journal1 = self.env["account.journal"].search(
                [("provisional", "=", True), ("type", "=", "purchase")]
            )
            journal = journal1[0]
            # I get the tax from the module configuration
            # I validate that the parameter is set
            company = self.company_id
            config_settings = (
                self.env["res.config.settings"]
                .sudo()
                .search([("company_id", "=", company.id)], limit=1)
            )
            # Check if the configuration was found
            if config_settings:
                # Get the internal sales tax from the configuration
                tax_id = config_settings.internal_voucher_purchase_tax
            else:
                tax = self.env["account.tax"].search(
                    [
                        ("type_tax_use", "=", "purchase"),
                        ("provisional", "=", True),
                        ("company_id", "=", company.id),
                    ],
                    limit=1,
                )
                tax_id = tax[0]
            if not tax_id:
                raise ValidationError(
                    _("Internal voucher purchase tax not found. Please set one.")
                )
            res.update({"tax_ids": [(6, 0, [tax_id.id])], "journal_id": journal.id})
        else:
            journal2 = self.env["account.journal"].search(
                [("provisional", "=", False), ("type", "=", "purchase")]
            )
            journal = journal2[0]
        if not journal:
            raise ValidationError(
                _("Provissional journal for sales not found." " Please create one.")
            )
        res.update({"journal_id": journal.id})
        return res
