"""This module contains the model representing a sales order."""
import xml.etree.cElementTree as ET

from odoo import _, api, fields, models
from odoo.exceptions import ValidationError


class SaleOrder(models.Model):
    """Add provisional field in Sales Budget / Sales Order."""

    _inherit = ["sale.order"]

    provisional = fields.Boolean(
        "Provisorio",
    )
    provisional2 = fields.Boolean(
        "Provisorio",
        compute="_compute_prov",
    )
    state_readonly = fields.Boolean("state", compute="_compute_state", default=False)

    @api.model
    def get_view(self, view_id=None, view_type="form", **options):
        """Hides the temporary field for the group_user group."""
        res = super().get_view(view_id, view_type, **options)
        group_user = self.env.user.has_group(
            "nybble_internal_voucher.group_voucher_user"
        )
        group_manager = self.env.user.has_group(
            "nybble_internal_voucher.group_voucher_admin"
        )
        if not group_manager and (group_user):
            if view_type in ["form"]:
                root = ET.fromstring(res["arch"])
                for el in root.iter("field"):
                    if el.attrib.get("name") in ["provisional", "provisional2"]:
                        el.attrib.update(
                            {
                                "modifiers": '{"invisible": [["id", "!=", 0]]}',
                            }
                        )
                res.update({"arch": ET.tostring(root, encoding="utf8", method="xml")})
        return res

    @api.depends("provisional")
    def _compute_prov(self):
        for rec in self:
            rec.provisional2 = rec.provisional

    @api.depends("state")
    def _compute_state(self):
        if self.state == "sale":
            self.state_readonly = True
        elif self.state == "done":
            self.state_readonly = True
        else:
            self.state_readonly = False

    def _prepare_invoice(self):
        """
        We extend to contemplate the creation of provisional invoices.
        If the sales order is provisional, a provisional invoice is
        created by setting the invoice provisional field and
        establishing a provisional sales journal.
        """
        res = super(SaleOrder, self)._prepare_invoice()
        if self.provisional:
            journal1 = self.env["account.journal"].search(
                [("provisional", "=", True), ("type", "=", "sale")]
            )
            journal = journal1[0]
            res.update({"provisional": True})
        else:
            journal2 = self.env["account.journal"].search(
                [("provisional", "=", False), ("type", "=", "sale")]
            )
            journal = journal2[0]
        if not journal:
            raise ValidationError(
                _("Provissional journal for sales not found." " Please create one.")
            )
        res.update({"journal_id": journal.id})
        return res

    def action_manual_delivery_wizard(self):
        """Open the manual delivery wizard for the selected sale order"""
        self.ensure_one()
        action = self.env.ref("sale_manual_delivery.action_wizard_manual_delivery")
        [action] = action.read()
        action["context"] = {"default_carrier_id": self.carrier_id.id}
        if self.provisional:
            action["context"] = {"provisional": self.provisional}
        return action

    def action_confirm(self):
        """
        We extend to create stock.picking with
        the "tentative = True" field if applicable
        to a provisional sales order.
        """
        res = super(SaleOrder, self).action_confirm()

        # We iterate over each sales order
        for order in self:
            if order.provisional:
                order.action_unlock()
                # Get the sales order company
                company = order.company_id
                # Get global settings related to the current company
                config_settings = (
                    self.env["res.config.settings"]
                    .sudo()
                    .search([("company_id", "=", company.id)], limit=1)
                )
                # Check if the configuration was found
                if config_settings:
                    # Get the internal sales tax from the configuration
                    internal_voucher_sale_tax = (
                        config_settings.internal_voucher_sale_tax
                    )
                    # Assign tax only if configured
                    if internal_voucher_sale_tax:
                        for line in order.order_line:
                            line.tax_id = [(6, 0, [internal_voucher_sale_tax.id])]
                else:
                    tax = self.env["account.tax"].search(
                        [
                            ("type_tax_use", "=", "sale"),
                            ("provisional", "=", True),
                            ("company_id", "=", company.id),
                        ],
                        limit=1,
                    )
                    for line in order.order_line:
                        line.tax_id = [(6, 0, [tax.id])]
                # Update the provisional field of the associated pickings
                order.picking_ids.write({"provisional": True})
        return res


class SaleOrderLine(models.Model):
    """
    Sale Order line functionality is added to determine which
    tax apply in case of internal receipt.
    """

    _inherit = "sale.order.line"

    def _prepare_invoice_line(self, **optional_values):
        """
        Extends function and adds in the case of a marked Order
        As provisional, the VAT tax line not taxed.
        """
        res = super(SaleOrderLine, self)._prepare_invoice_line(**optional_values)
        if self.order_id.provisional:
            # I get the tax set in the module configuration
            # and I validate that the parameter is set
            company = self.order_id.company_id
            # Get global settings related to the current company
            config_settings = (
                self.env["res.config.settings"]
                .sudo()
                .search([("company_id", "=", company.id)], limit=1)
            )
            # Check if the configuration was found
            if config_settings:
                # Get the internal sales tax from the configuration
                tax_id = config_settings.internal_voucher_sale_tax
                if not tax_id:
                    raise ValidationError(
                        _("Internal voucher sale tax not found. Please set one.")
                    )
                res.update({"tax_ids": [(6, 0, [tax_id.id])]})
            else:
                tax = self.env["account.tax"].search(
                    [
                        ("type_tax_use", "=", "sale"),
                        ("provisional", "=", True),
                        ("company_id", "=", company.id),
                    ],
                )
                res.update({"tax_ids": [(6, 0, [tax.id])]})
        return res
