"""this module is used to add classification field in product template and product product model"""
from dateutil.relativedelta import relativedelta

from odoo import api, fields, models, _
from odoo.exceptions import ValidationError


class ProductTemplate(models.Model):
    """Inherit res.config.settings to add a field for users to notify when a cost is determined."""

    _inherit = "product.template"

    classification_type = fields.Selection(
        [
            ("active", "Active"),
            ("new", "New"),
            ("no_sales", "No Sales"),
            ("dormant", "Dormant"),
        ],
        string="Classification",
        help="Classification of the product or product variant based on turnover",
    )

    show_classification = fields.Boolean(compute="_compute_classification_visibility")

    @api.depends("detailed_type", "attribute_line_ids")
    def _compute_classification_visibility(self):
        """Compute the visibility of the classification field"""
        for product in self:
            product.show_classification = (
                product.detailed_type == "product" and not product.has_variants()
            )

    def has_variants(self):
        """Check if the product template has real variants"""
        self.ensure_one()
        if not self.attribute_line_ids:
            return False
        lines_with_multiple_values = sum(
            1 for line in self.attribute_line_ids if len(line.value_ids) > 1
        )
        return lines_with_multiple_values > 0
    
    @api.model_create_multi
    def create(self, vals_list):
        '''
        company = self.env["res.users"].browse(self._uid).company_id
        if not company.is_main_company:
            raise ValidationError(_("Only the main company can create products."))
        '''
        templates = super(ProductTemplate, self).create(vals_list)
        companies = self.env["res.company"].search([('is_main_company', '!=', True)])
        for company_id in companies:
            company = self.with_company(company_id)
            for template in templates:
                f_product = company.env["product.template"].sudo().browse(template.id)
                for tax in template.sudo().taxes_id:
                    f_tax = company.env["account.tax"].search([('name', '=', tax.name), ('company_id', '=', company_id.id), ('type_tax_use', '=', 'sale')])
                    f_product.taxes_id = [(4, f_tax.id)]
                    f_tax = company.env["account.tax"].search([('name', '=', tax.name), ('company_id', '=', company_id.id), ('type_tax_use', '=', 'purchase')])
                    f_product.supplier_taxes_id = [(4, f_tax.id)]
        return templates

    def write(self, vals):
        """Override write method to update the classification visibility"""
        '''
        company = self.env["res.users"].browse(self._uid).company_id
        confirming_purchase_order = self.env.context.get('confirming_purchase_order', False)
        #picking_ids = self.env.context.get('picking_ids', False)
        #picking_type_id = self.env.context.get('default_picking_type_id', False)
        is_picking = self.env.context.get('is_picking', False)
        #import wdb; wdb.set_trace()
        if not company.is_main_company and not confirming_purchase_order and not is_picking:
            raise ValidationError(_("Only the main company can modify products."))
        '''
        res = super(ProductTemplate, self).write(vals)
        if "detailed_type" in vals:
            self._compute_classification_visibility()
        return res

    def unlink(self):
        """Override unlink method to update the classification visibility"""
        '''
        company = self.env["res.users"].browse(self._uid).company_id
        if not company.is_main_company:
            raise ValidationError(_("Only the main company can delete products."))
        '''
        res = super(ProductTemplate, self).unlink()
        self._compute_classification_visibility()
        return res


class ProductProduct(models.Model):
    """Inherit res.config.settings to add a field for users to notify when a cost is determined."""

    _inherit = "product.product"

    classification_type = fields.Selection(
        related="product_tmpl_id.classification_type", readonly=False, store=True
    )
    show_classification = fields.Boolean(
        compute="_compute_classification_visibility", default=True
    )

    @api.depends()
    def _compute_classification_visibility(self):
        """Compute the visibility of the classification field"""
        for product in self:
            product.show_classification = True

    @api.model_create_multi
    def create(self, vals_list):
        '''
        company = self.env["res.users"].browse(self._uid).company_id
        if not company.is_main_company:
            raise ValidationError(_("Only the main company can create product variants."))
        '''
        products = super(ProductProduct, self).create(vals_list)
        companies = self.env["res.company"].search([('is_main_company', '!=', True)])
        for company_id in companies:
            company = self.with_company(company_id)
            for product in products:
                f_product = company.env["product.product"].sudo().browse(product.id)
                for tax in product.sudo().taxes_id:
                    f_tax = company.env["account.tax"].search([('name', '=', tax.name), ('company_id', '=', company_id.id), ('type_tax_use', '=', 'sale')])
                    f_product.taxes_id = [(4, f_tax.id)]
                    f_tax = company.env["account.tax"].search([('name', '=', tax.name), ('company_id', '=', company_id.id), ('type_tax_use', '=', 'purchase')])
                    f_product.supplier_taxes_id = [(4, f_tax.id)]
        return products

    def write(self, vals):
        """Override write method to update the classification visibility"""
        '''
        company = self.env["res.users"].browse(self._uid).company_id
        confirming_purchase_order = self.env.context.get('confirming_purchase_order', False)
        #picking_ids = self.env.context.get('picking_ids', False)
        #picking_type_id = self.env.context.get('default_picking_type_id', False)
        is_picking = self.env.context.get('is_picking', False)
        #import wdb; wdb.set_trace()
        if not company.is_main_company and not confirming_purchase_order and not is_picking:
            raise ValidationError(_("Only the main company can modify product variants."))
        '''
        res = super(ProductProduct, self).write(vals)
        if "detailed_type" in vals:
            self._compute_classification_visibility()
        if "classification_type" in vals:
            template = self.product_tmpl_id
            if all(
                v.classification_type == vals["classification_type"]
                for v in template.product_variant_ids
            ):
                template.classification_type = vals["classification_type"]
        return res

    def unlink(self):
        """Override unlink method to update the classification visibility"""
        '''
        company = self.env["res.users"].browse(self._uid).company_id
        if not company.is_main_company:
            raise ValidationError(_("Only the main company can delete product variants."))
        '''
        res = super(ProductProduct, self).unlink()
        self._compute_classification_visibility()
        return res

    @api.model
    def _run_stock_classification(
        self, new=True, active=True, no_sales=True, dormant=True
    ):
        """Run the stock classification process"""
        products = self.search([("detailed_type", "=", "product")])
        for product in products:
            old_classification = product.classification_type
            new_classification = old_classification

            if new and self._is_new_product(product, self._get_new_period()):
                new_classification = "new"
            elif active and self._is_active_product(product, self._get_active_period()):
                new_classification = "active"
            else:
                no_sales_period = self._get_zero_sales_period()
                dormant_period = self._get_dormant_period()
                longer_period = max(no_sales_period, dormant_period)
                shorter_period = min(no_sales_period, dormant_period)

                if self._is_inactive_product(product, longer_period):
                    new_classification = (
                        "dormant" if dormant_period > no_sales_period else "no_sales"
                    )
                elif self._is_inactive_product(product, shorter_period):
                    new_classification = (
                        "no_sales" if dormant_period > no_sales_period else "dormant"
                    )

            if new_classification != old_classification:
                self._update_classification(product, new_classification)

    def _is_new_product(self, product, new_period):
        """Check if a product is new based on the new_period and has no sales in that period"""
        today = fields.Date.today()
        period_start = today - relativedelta(months=new_period)

        if period_start <= product.create_date.date() <= today:
            return not self._has_sales_since(product, product.create_date.date())

        return False

    def _is_active_product(self, product, active_period):
        """Check if a product has been active (invoiced sales) for the last active_period months"""
        start_date = fields.Date.today() - relativedelta(months=active_period)
        return self._has_sales_since(product, start_date)

    def _is_inactive_product(self, product, inactive_period):
        """Check if a product has been inactive for the given inactive_period"""
        last_sale_date = self._get_last_sale_date(product)
        if not last_sale_date:
            return (
                fields.Date.today() - product.create_date.date()
            ).days > inactive_period * 30
        return (fields.Date.today() - last_sale_date).days > inactive_period * 30

    def _has_sales_since(self, product, start_date):
        """Check if a product has any sales from a sale order since the given start_date"""
        invoices = self.env["account.move"].search(
            [
                ("move_type", "=", "out_invoice"),
                ("state", "=", "posted"),
                ("invoice_date", ">=", start_date),
                ("invoice_line_ids.product_id", "=", product.id),
                (
                    "invoice_line_ids.sale_line_ids",
                    "!=",
                    False,
                ),  # Ensure the invoice line is linked to a sale order line
            ],
            limit=1,
        )
        return bool(invoices)

    def _get_last_sale_date(self, product):
        """Get the date of the last invoiced sale from a sale order for a product"""
        last_invoice = self.env["account.move"].search(
            [
                ("move_type", "=", "out_invoice"),
                ("state", "=", "posted"),
                ("invoice_line_ids.product_id", "=", product.id),
                (
                    "invoice_line_ids.sale_line_ids",
                    "!=",
                    False,
                ),  # Ensure the invoice line is linked to a sale order line
            ],
            order="invoice_date desc",
            limit=1,
        )
        return last_invoice.invoice_date if last_invoice else None

    def _update_classification(self, product, classification):
        """Update the classification of a product"""
        company = self.env["res.company"].search(
            [("is_main_company", "=", True)], limit=1
        )
        if not company:
            raise ValidationError(_("No main company found."))
        product.write({"classification_type": classification})
        self.env["stock.classification.rotation"].create(
            {
                "product_template_id": product.product_tmpl_id.id,
                "product_id": product.id,
                "classification_type": classification,
                "company_id": company.id,
            }
        )

    def _get_new_period(self):
        """Get the period for a product to be considered new"""
        return int(
            self.env["ir.config_parameter"]
            .sudo()
            .get_param("stock.new_sales_period", default=0)
        )

    def _get_active_period(self):
        """Get the period for a product to be considered active"""
        return int(
            self.env["ir.config_parameter"]
            .sudo()
            .get_param("stock.active_sales_period", default=0)
        )

    def _get_zero_sales_period(self):
        """Get the period for a product to be considered to have zero sales"""
        return int(
            self.env["ir.config_parameter"]
            .sudo()
            .get_param("stock.zero_sales_period", default=0)
        )

    def _get_dormant_period(self):
        """Get the period for a product to be considered dormant"""
        return int(
            self.env["ir.config_parameter"]
            .sudo()
            .get_param("stock.dormant_sales_period", default=0)
        )


class Company(models.Model):
    """Inherit res.config.settings to add a field for users to notify when a cost is determined."""

    _inherit = "res.company"

    is_main_company = fields.Boolean(string="Is Main Company", default=False)

    _sql_constraints = [
        (
            "unique_main_company",
            "unique(is_main_company)",
            "Only one main company is allowed.",
        )
    ]
