"""Inherit res.config.settings to add a field for users to notify when a cost is determined."""
from odoo import _, api, fields, models
from odoo.exceptions import ValidationError


class ResConfigSettings(models.TransientModel):
    """Inherit res.config.settings to add a field for users to notify when a cost is determined."""

    _inherit = "res.config.settings"

    active_sales_period = fields.Integer(string="Sales Period (Active)", default=0)
    new_sales_period = fields.Integer(string="No Sales Period (New)", default=0)
    zero_sales_period = fields.Integer(string="No Sales Period (Zero Sales)", default=0)
    dormant_sales_period = fields.Integer(string="No Sales Period (Dormant)", default=0)

    @api.constrains("new_sales_period", "zero_sales_period", "dormant_sales_period")
    def _check_unique_sales_periods(self):
        """Check that the periods for New, Zero Sales, and Dormant classifications are different."""
        for record in self:
            periods = [
                record.new_sales_period,
                record.zero_sales_period,
                record.dormant_sales_period,
            ]
            unique_periods = set(periods)
            if len(unique_periods) < 3:
                raise ValidationError(
                    _(
                        "The periods for New, Zero Sales, and Dormant classifications must be different."
                    )
                )

    def set_values(self):
        """Override set_values to save the cost_determination_users in the ir.config_parameter."""
        super(ResConfigSettings, self).set_values()
        self.env["ir.config_parameter"].set_param(
            "stock.active_sales_period", self.active_sales_period
        )
        self.env["ir.config_parameter"].set_param(
            "stock.new_sales_period", self.new_sales_period
        )
        self.env["ir.config_parameter"].set_param(
            "stock.zero_sales_period", self.zero_sales_period
        )
        self.env["ir.config_parameter"].set_param(
            "stock.dormant_sales_period", self.dormant_sales_period
        )

    @api.model
    def get_values(self):
        """Override get_values to get the cost_determination_users from the ir.config_parameter."""
        res = super(ResConfigSettings, self).get_values()
        res.update(
            active_sales_period=int(
                self.env["ir.config_parameter"].get_param(
                    "stock.active_sales_period", default=0
                )
            ),
            new_sales_period=int(
                self.env["ir.config_parameter"].get_param(
                    "stock.new_sales_period", default=0
                )
            ),
            zero_sales_period=int(
                self.env["ir.config_parameter"].get_param(
                    "stock.zero_sales_period", default=0
                )
            ),
            dormant_sales_period=int(
                self.env["ir.config_parameter"].get_param(
                    "stock.dormant_sales_period", default=0
                )
            ),
        )
        return res
