"""Test cases for the stock rotation module"""
from dateutil.relativedelta import relativedelta

from odoo.fields import Date
from odoo.tests.common import TransactionCase


class TestProductProductClassification(TransactionCase):
    """Test cases for the product classification"""

    def setUp(self):
        """Setup for the test cases"""
        super(TestProductProductClassification, self).setUp()
        # Create a partner to use in the tests
        self.partner = self.env["res.partner"].create(
            {
                "name": "Test Partner",
            }
        )
        # Create a vendor (supplier) for the product
        self.vendor = self.env["res.partner"].create(
            {
                "name": "Test Vendor",
                "supplier_rank": 1,
            }
        )
        # Initial setup for the product and context
        self.product = self.env["product.product"].create(
            {
                "name": "Test Product",
                "type": "product",
                "seller_ids": [
                    (0, 0, {"name": self.vendor.id})
                ],  # Assign the vendor to the product
            }
        )
        self.today = Date.today()

    def test_classification_visibility(self):
        """Test the visibility of the classification field"""
        self.assertTrue(self.product.show_classification)

    def test_classification_type_selection(self):
        """Test the selection options for classification_type field"""
        expected_types = [
            ("active", "Active"),
            ("new", "New"),
            ("no_sales", "No Sales"),
            ("dormant", "Dormant"),
        ]
        self.assertEqual(
            self.product._fields["classification_type"].selection, expected_types
        )

    def test_update_classification_on_write(self):
        """Test the update of classification visibility when writing to the product"""
        self.product.write({"detailed_type": "product"})
        self.assertTrue(self.product.show_classification)

    def test_run_stock_classification(self):
        """Test running the stock classification process"""
        self.product._run_stock_classification()
        # Add specific assertions here to validate the classification process

    def test_is_new_product(self):
        """Test if the product is classified as new"""
        # Set the product's creation date within the new_period to ensure it's classified as new
        new_period = 1  # Set the new_period to 1 month for this test
        self.env["ir.config_parameter"].sudo().set_param(
            "stock.new_sales_period", str(new_period)
        )
        self.product.create_date = self.today - relativedelta(
            months=new_period - 0.5
        )  # Adjust creation date within the period

        self.assertTrue(self.product._is_new_product(self.product, new_period))

    def test_is_active_product(self):
        """Test if the product is classified as active"""
        # Create a sale order to simulate activity
        sale_order = self.env["sale.order"].create(
            {
                "partner_id": self.partner.id,
                "order_line": [
                    (0, 0, {"product_id": self.product.id, "product_uom_qty": 1})
                ],
            }
        )
        sale_order.action_confirm()
        self.assertTrue(self.product._is_active_product(self.product, 1))

    def test_is_no_sales_product(self):
        """Test if the product is classified as no_sales"""
        # Set the product's creation date to be beyond the zero_sales_period to ensure it's classified as no_sales
        zero_sales_period = 4  # Set zero_sales_period to 4 months for this test
        self.env["ir.config_parameter"].sudo().set_param(
            "stock.zero_sales_period", str(zero_sales_period)
        )
        self.product.create_date = self.today - relativedelta(months=5)

        # Ensure there are no sales within the zero_sales_period
        self.assertTrue(
            self.product._is_no_sales_product(self.product, zero_sales_period)
        )

    def test_is_dormant_product(self):
        """Test if the product is classified as dormant"""
        self.product.create_date = self.today - relativedelta(months=10)
        self.assertTrue(self.product._is_dormant_product(self.product, 9))

    def test_get_last_sale(self):
        """Test getting the last sale order for a product"""
        # Create a sale order for this test
        sale_order = self.env["sale.order"].create(
            {
                "partner_id": self.partner.id,
                "order_line": [
                    (0, 0, {"product_id": self.product.id, "product_uom_qty": 1})
                ],
            }
        )
        sale_order.action_confirm()
        last_sale = self.product._get_last_sale(self.product)
        self.assertEqual(last_sale.id, sale_order.id)

    def test_update_classification(self):
        """Test updating the classification of a product"""
        self.product._update_classification(self.product, "active")
        self.assertEqual(self.product.classification_type, "active")
