"""Model to manage minimum margin for sale orders."""
from odoo import _, api, fields, models
from odoo.exceptions import UserError


class SaleOrder(models.Model):
    """Class to manage minimum margin for sale orders."""

    _inherit = "sale.order"

    has_below_minimum_margin = fields.Boolean(
        string="Has Products Below Minimum Margin",
        compute="_compute_has_below_minimum_margin",
        store=True,
    )

    @api.depends(
        "order_line.margin_percent", "order_line.product_id", "team_id", "pricelist_id"
    )
    def _compute_has_below_minimum_margin(self):
        """Compute if any product in the order has margin below minimum configured."""
        for order in self:
            order.has_below_minimum_margin = False
            if order.team_id and order.pricelist_id:
                for line in order.order_line:
                    if line.product_id and line.product_id.categ_id:
                        margin_rule = self.env["sale.order.minimum.margin"].search(
                            [
                                ("team_id", "=", order.team_id.id),
                                ("pricelist_id", "=", order.pricelist_id.id),
                                (
                                    "product_category_ids",
                                    "in",
                                    line.product_id.categ_id.id,
                                ),
                                ("company_id", "=", order.company_id.id),
                            ],
                            limit=1,
                        )

                        if (
                            margin_rule
                            and line.margin_percent < margin_rule.minimum_margin
                        ):
                            order.has_below_minimum_margin = True
                            break

    def action_confirm(self):
        """Override to check minimum margin before confirmation."""
        self.ensure_one()
        if self.has_below_minimum_margin:
            if not self.env.user.has_group(
                "nybble_margin_sale.group_authorize_below_margin"
            ):
                raise UserError(
                    _(
                        "This transaction can only be confirmed by users authorized to "
                        "confirm sales below the minimum margin. Please contact the responsible area"
                    )
                )
        return super(SaleOrder, self).action_confirm()


class SaleOrderLine(models.Model):
    """Class to manage minimum margin for sale order lines."""

    _inherit = "sale.order.line"

    @api.onchange("product_id", "discount", "price_unit", "product_uom_qty")
    def _onchange_verify_margin(self):
        """Verify margin when product details change."""
        if self.product_id and self.order_id.team_id and self.order_id.pricelist_id:
            margin_rule = self.env["sale.order.minimum.margin"].search(
                [
                    ("team_id", "=", self.order_id.team_id.id),
                    ("pricelist_id", "=", self.order_id.pricelist_id.id),
                    ("product_category_ids", "in", self.product_id.categ_id.id),
                    ("company_id", "=", self.order_id.company_id.id),
                ],
                limit=1,
            )

            if margin_rule and self.margin_percent < margin_rule.minimum_margin:
                return {
                    "warning": {
                        "title": _("Warning!"),
                        "message": _(
                            "The margin for this product (%s%%) is below the minimum allowed margin (%s%%) "
                            "for this sales team and pricelist combination."
                        )
                        % (
                            round(self.margin_percent, 2),
                            round(margin_rule.minimum_margin, 2),
                        ),
                    }
                }
