"""Module for Sale Order Minimum Margin model."""
from odoo import _, api, fields, models
from odoo.exceptions import ValidationError


class SaleOrderMinimumMargin(models.Model):
    """Model for Sale Order Minimum Margin."""

    _name = "sale.order.minimum.margin"
    _description = "Minimum Sales Margin"
    _order = "team_id"

    team_id = fields.Many2one(
        "crm.team",
        string="Sales Team",
        required=True,
        ondelete="cascade",
        no_create=True,
        no_open=True,
    )

    pricelist_id = fields.Many2one(
        "product.pricelist",
        string="Pricelist",
        required=True,
        ondelete="cascade",
        no_create=True,
        no_open=True,
    )

    product_category_ids = fields.Many2many(
        "product.category",
        string="Product Categories",
        required=True,
        ondelete="cascade",
        no_create=True,
        no_open=True,
    )

    minimum_margin = fields.Float(
        string="Minimum Margin",
        required=True,
        widget="percentage",
    )

    company_id = fields.Many2one(
        "res.company",
        string="Company",
        required=True,
        default=lambda self: self.env.company,
        store=True,
    )

    @api.constrains("minimum_margin")
    def _check_minimum_margin(self):
        """Check that the minimum margin is greater than 0."""
        for record in self:
            if record.minimum_margin <= 0:
                raise ValidationError(_("Minimum margin must be greater than 0."))

    @api.constrains("team_id", "pricelist_id", "product_category_ids", "company_id")
    def _check_unique_configuration(self):
        """Check that the configuration is unique."""
        for record in self:
            domain = [
                ("team_id", "=", record.team_id.id),
                ("pricelist_id", "=", record.pricelist_id.id),
                ("company_id", "=", record.company_id.id),
                ("id", "!=", record.id),
            ]

            # Check if categories match
            existing_records = self.search(domain)
            for existing in existing_records:
                if set(record.product_category_ids.ids) == set(
                    existing.product_category_ids.ids
                ):
                    raise ValidationError(
                        _(
                            "A configuration already exists for this combination of Sales Team, "
                            "Pricelist, Product Categories and Company."
                        )
                    )
