"""Test cases for the minimum margin functionality in sale orders."""
from odoo.exceptions import UserError
from odoo.tests import common


class TestSaleOrderMinimumMargin(common.TransactionCase):
    """Test cases for the minimum margin functionality in sale orders."""

    def setUp(self):
        """Set up test data."""
        super(TestSaleOrderMinimumMargin, self).setUp()

        # Create test user without margin authorization
        self.regular_user = self.env["res.users"].create(
            {
                "name": "Regular User",
                "login": "regular_user",
                "email": "regular@test.com",
                "groups_id": [
                    (6, 0, [self.env.ref("sales_team.group_sale_salesman").id])
                ],
            }
        )

        # Create test user with margin authorization
        self.authorized_user = self.env["res.users"].create(
            {
                "name": "Authorized User",
                "login": "authorized_user",
                "email": "authorized@test.com",
                "groups_id": [
                    (
                        6,
                        0,
                        [
                            self.env.ref("sales_team.group_sale_salesman").id,
                            self.env.ref(
                                "nybble_margin_sale.group_authorize_below_margin"
                            ).id,
                        ],
                    )
                ],
            }
        )

        # Create test product category
        self.test_category = self.env["product.category"].create(
            {"name": "Test Category"}
        )

        # Create test product
        self.test_product = self.env["product.product"].create(
            {
                "name": "Test Product",
                "type": "product",
                "categ_id": self.test_category.id,
                "standard_price": 100.0,  # Cost
                "list_price": 150.0,  # Sales price
            }
        )

        # Create sales team
        self.sales_team = self.env["crm.team"].create({"name": "Test Sales Team"})

        # Create pricelist
        self.pricelist = self.env["product.pricelist"].create(
            {"name": "Test Pricelist"}
        )

        # Create minimum margin rule
        self.margin_rule = self.env["sale.order.minimum.margin"].create(
            {
                "team_id": self.sales_team.id,
                "pricelist_id": self.pricelist.id,
                "product_category_ids": [(6, 0, [self.test_category.id])],
                "minimum_margin": 30.0,
                "company_id": self.env.company.id,
            }
        )

    def test_01_compute_has_below_minimum_margin(self):
        """Test computation of below minimum margin flag."""
        # Create sale order with margin above minimum
        sale_order = self.env["sale.order"].create(
            {
                "partner_id": self.env.ref("base.res_partner_1").id,
                "team_id": self.sales_team.id,
                "pricelist_id": self.pricelist.id,
            }
        )

        # Add order line with good margin (50%)
        self.env["sale.order.line"].create(
            {
                "order_id": sale_order.id,
                "product_id": self.test_product.id,
                "product_uom_qty": 1,
                "price_unit": 150.0,  # 50% margin
            }
        )

        self.assertFalse(
            sale_order.has_below_minimum_margin,
            "Sale order should not be marked as below minimum margin",
        )

        # Modify price to create margin below minimum
        sale_order.order_line.price_unit = 120.0  # 20% margin

        sale_order._compute_has_below_minimum_margin()
        self.assertTrue(
            sale_order.has_below_minimum_margin,
            "Sale order should be marked as below minimum margin",
        )

    def test_02_order_confirmation_authorization(self):
        """Test order confirmation with different user authorizations."""
        # Create sale order with margin below minimum
        sale_order = self.env["sale.order"].create(
            {
                "partner_id": self.env.ref("base.res_partner_1").id,
                "team_id": self.sales_team.id,
                "pricelist_id": self.pricelist.id,
            }
        )

        # Add order line with low margin (20%)
        self.env["sale.order.line"].create(
            {
                "order_id": sale_order.id,
                "product_id": self.test_product.id,
                "product_uom_qty": 1,
                "price_unit": 120.0,
            }
        )

        # Test with unauthorized user
        sale_order = sale_order.with_user(self.regular_user)
        with self.assertRaises(
            UserError, msg="Regular user should not be able to confirm order"
        ):
            sale_order.action_confirm()

        # Test with authorized user
        sale_order = sale_order.with_user(self.authorized_user)
        try:
            sale_order.action_confirm()
        except UserError:
            self.fail("Authorized user should be able to confirm order")

    def test_03_onchange_verify_margin(self):
        """Test margin verification on product changes."""
        # Create sale order
        sale_order = self.env["sale.order"].create(
            {
                "partner_id": self.env.ref("base.res_partner_1").id,
                "team_id": self.sales_team.id,
                "pricelist_id": self.pricelist.id,
            }
        )

        # Create order line
        order_line = self.env["sale.order.line"].create(
            {
                "order_id": sale_order.id,
                "product_id": self.test_product.id,
                "product_uom_qty": 1,
                "price_unit": 120.0,  # 20% margin
            }
        )

        # Test onchange warning
        result = order_line._onchange_verify_margin()
        self.assertTrue(
            result.get("warning"), "Should return warning for margin below minimum"
        )
        self.assertIn(
            "margin",
            result["warning"]["message"].lower(),
            "Warning message should mention margin",
        )

        # Test no warning with good margin
        order_line.price_unit = 150.0  # 50% margin
        result = order_line._onchange_verify_margin()
        self.assertFalse(result, "Should not return warning for good margin")
