"""Test cases for Sale Order Minimum Margin."""
from odoo.exceptions import ValidationError
from odoo.tests.common import TransactionCase


class TestSaleOrderMinimumMargin(TransactionCase):
    """Test cases for Sale Order Minimum Margin."""

    def setUp(self):
        """Set up test data."""
        super().setUp()

        # Create test company
        self.company = self.env["res.company"].create(
            {
                "name": "Test Company",
                "currency_id": self.env.ref("base.USD").id,
            }
        )

        # Create another company for multi-company tests
        self.company2 = self.env["res.company"].create(
            {
                "name": "Test Company 2",
                "currency_id": self.env.ref("base.USD").id,
            }
        )

        # Create test sales team
        self.sales_team = self.env["crm.team"].create(
            {
                "name": "Test Sales Team",
                "company_id": self.company.id,
            }
        )

        # Create test pricelist
        self.pricelist = self.env["product.pricelist"].create(
            {
                "name": "Test Pricelist",
                "company_id": self.company.id,
            }
        )

        # Create test product categories
        self.category1 = self.env["product.category"].create(
            {
                "name": "Test Category 1",
            }
        )
        self.category2 = self.env["product.category"].create(
            {
                "name": "Test Category 2",
            }
        )

        # Model to test
        self.MarginModel = self.env["sale.order.minimum.margin"]

        # Create base test record
        self.test_margin = self.MarginModel.create(
            {
                "team_id": self.sales_team.id,
                "pricelist_id": self.pricelist.id,
                "product_category_ids": [(6, 0, [self.category1.id])],
                "minimum_margin": 10.0,
                "company_id": self.company.id,
            }
        )

    def test_01_create_margin_configuration(self):
        """Test creation of margin configuration."""
        margin = self.MarginModel.create(
            {
                "team_id": self.sales_team.id,
                "pricelist_id": self.pricelist.id,
                "product_category_ids": [(6, 0, [self.category2.id])],
                "minimum_margin": 15.0,
                "company_id": self.company.id,
            }
        )

        self.assertEqual(margin.minimum_margin, 15.0)
        self.assertEqual(margin.company_id, self.company)

    def test_02_negative_margin_validation(self):
        """Test validation for negative margin."""
        with self.assertRaises(ValidationError):
            self.MarginModel.create(
                {
                    "team_id": self.sales_team.id,
                    "pricelist_id": self.pricelist.id,
                    "product_category_ids": [(6, 0, [self.category1.id])],
                    "minimum_margin": -5.0,
                    "company_id": self.company.id,
                }
            )

    def test_03_zero_margin_validation(self):
        """Test validation for zero margin."""
        with self.assertRaises(ValidationError):
            self.MarginModel.create(
                {
                    "team_id": self.sales_team.id,
                    "pricelist_id": self.pricelist.id,
                    "product_category_ids": [(6, 0, [self.category1.id])],
                    "minimum_margin": 0.0,
                    "company_id": self.company.id,
                }
            )

    def test_04_duplicate_configuration_validation(self):
        """Test validation for duplicate configuration."""
        with self.assertRaises(ValidationError):
            self.MarginModel.create(
                {
                    "team_id": self.sales_team.id,
                    "pricelist_id": self.pricelist.id,
                    "product_category_ids": [(6, 0, [self.category1.id])],
                    "minimum_margin": 20.0,
                    "company_id": self.company.id,
                }
            )

    def test_05_multi_company_access(self):
        """Test multi-company record visibility."""
        # Create record for company 2
        margin_company2 = self.MarginModel.create(
            {
                "team_id": self.sales_team.id,
                "pricelist_id": self.pricelist.id,
                "product_category_ids": [(6, 0, [self.category1.id])],
                "minimum_margin": 25.0,
                "company_id": self.company2.id,
            }
        )

        # Switch to company 2 user context
        user_company2 = self.env["res.users"].create(
            {
                "name": "User Company 2",
                "login": "user_company2",
                "company_id": self.company2.id,
                "company_ids": [(6, 0, [self.company2.id])],
            }
        )

        # Search records as company 2 user
        margins = self.MarginModel.with_user(user_company2).search([])
        self.assertIn(margin_company2, margins)
        self.assertNotIn(self.test_margin, margins)

    def test_06_multiple_categories(self):
        """Test configuration with multiple product categories."""
        margin = self.MarginModel.create(
            {
                "team_id": self.sales_team.id,
                "pricelist_id": self.pricelist.id,
                "product_category_ids": [
                    (6, 0, [self.category1.id, self.category2.id])
                ],
                "minimum_margin": 30.0,
                "company_id": self.company.id,
            }
        )

        self.assertEqual(len(margin.product_category_ids), 2)
        self.assertIn(self.category1, margin.product_category_ids)
        self.assertIn(self.category2, margin.product_category_ids)

    def test_07_update_margin(self):
        """Test updating existing margin configuration."""
        self.test_margin.write({"minimum_margin": 35.0})

        self.assertEqual(self.test_margin.minimum_margin, 35.0)

    def test_08_required_fields(self):
        """Test required fields validation."""
        with self.assertRaises(ValueError):
            self.MarginModel.create(
                {
                    "team_id": self.sales_team.id,  # Missing required fields
                }
            )
