"""Inherit product.pricelist model to add new fields"""
import math

from odoo import fields, models


class ProductPricelistInherit(models.Model):
    """Inherit product.pricelist model to add new fields"""

    _inherit = "product.pricelist"

    is_franchise_list = fields.Boolean(
        string="Ready for franchise",
        default=False,
    )
    is_main_list = fields.Boolean(
        string="Is a main price list",
        default=False,
        store=True,
        help="If checked, this pricelist will be used to update the pricelists from the franchises."
    )
    is_updated_by_main = fields.Boolean(
        string="Update with main price list",
        default=False,
        store=True,
        help="If checked, this pricelist will be updated automatically when the CC pricelists is changed."
    )
    valid_for = fields.Many2one(
        "res.company",
        string="Franchise",
        domain="[('is_main_company', '=', False)]",
        help="Company for which this rate will be valid",
    )
    company_id = fields.Many2one(
        "res.company",
        string="Company",
        default=lambda self: self.env.company,
    )
    is_main_company = fields.Boolean(
        string="Main Company",
        related="company_id.is_main_company",
        store=True,
        readonly=True,
    )

    def update_supplier_info(self):
        """Update the product_supplierinfo table with the price list variants."""
        # Fetch the price list data
        price_list_data = self._get_price_list_data()

        # Define the batch size
        batch_size = 5000

        # Calculate the number of batches
        total_variants = len(price_list_data)
        num_batches = math.ceil(total_variants / batch_size)

        # Process the price list in batches
        for batch_idx in range(num_batches):
            start = batch_idx * batch_size
            end = min((batch_idx + 1) * batch_size, total_variants)
            batch_data = price_list_data[start:end]

            # Prepare the data for bulk upsert
            supplier_info_data = self._prepare_supplier_info_data(batch_data)

            # Perform the bulk upsert
            self._bulk_upsert_supplier_info(supplier_info_data)

            print(f"Processed batch {batch_idx + 1} of {num_batches}")

    def _get_price_list_data(self):
        """Fetch the price list data from the database."""
        query = """
            SELECT
                product_id,
                product_tmpl_id,
                currency_id,
                company_id,
                price_discount,
                price_surcharge,
                price_round,
                price_min_margin,
                date_start,
                date_end,
                min_quantity
            FROM product_pricelist_item
            WHERE pricelist_id = %s
        """
        self.env.cr.execute(query, (self.id,))
        return self.env.cr.dictfetchall()

    def _prepare_supplier_info_data(self, price_list_data):
        """Prepare the data for bulk upsert on the product_supplierinfo table."""
        supplier_info_data = []
        for row in price_list_data:
            product = self.env["product.product"].browse(row["product_id"])
            product_tmpl_id = product.product_tmpl_id.id or row["product_tmpl_id"]
            # Calculate the price based on the price list rules
            price = self._calculate_price(row)

            supplier_info_data.append(
                {
                    "product_id": row["product_id"],
                    "product_tmpl_id": product_tmpl_id or row["product_tmpl_id"],
                    "company_id": self.valid_for.id,
                    "currency_id": self.currency_id.id,
                    "vendor_price": price,
                    "price": price,
                    "ars_price": price,
                    "delay": 1,
                    "qty_min": row["min_quantity"],
                    "date_start": row["date_start"],
                    "date_end": row["date_end"],
                }
            )

        return supplier_info_data

    def _bulk_upsert_supplier_info(self, supplier_info_data):
        """Perform a bulk upsert operation on the product_supplierinfo table."""
        # Prepare the SQL queries
        insert_query = """
            INSERT INTO product_supplierinfo (
                product_id, product_tmpl_id, company_id, partner_id, currency_id,
                vendor_price, price, ars_price,
                create_uid, create_date, write_uid, write_date, delay, min_qty, date_start, date_end, inverse_company_rate
            )
            VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)
        """

        update_query = """
            UPDATE product_supplierinfo
            SET product_tmpl_id = %s, vendor_price = %s, price = %s, ars_price = %s,
                write_uid = %s, write_date = %s, delay = 1, min_qty = %s, date_start = %s, date_end = %s, inverse_company_rate = %s
            WHERE product_id = %s::integer AND company_id = %s
        """

        current_user_id = self.env.uid
        current_time = fields.Datetime.now()

        # Iterate over the supplier info data
        for data in supplier_info_data:
            if data["product_tmpl_id"]:
                # Check if the record exists
                partner_id = 1
                inverse_company_rate = 1
                self.env.cr.execute(
                    "SELECT 1 FROM product_supplierinfo WHERE product_id = %s::integer AND company_id = %s LIMIT 1",
                    (data["product_id"], data["company_id"]),
                )
                if self.env.cr.fetchone():
                    # Update the existing record
                    self.env.cr.execute(
                        update_query,
                        (
                            data["product_tmpl_id"],
                            data["vendor_price"],
                            data["price"],
                            data["ars_price"],
                            partner_id,
                            current_time,
                            data["qty_min"],
                            data["date_start"],
                            data["date_end"],
                            inverse_company_rate,
                            str(data["product_id"]),
                            data["company_id"],
                        ),
                    )
                else:
                    # Insert a new record
                    self.env.cr.execute(
                        insert_query,
                        (
                            str(data["product_id"]),
                            data["product_tmpl_id"],
                            data["company_id"],
                            partner_id,
                            data["currency_id"],
                            data["vendor_price"],
                            data["price"],
                            data["ars_price"],
                            current_user_id,
                            current_time,
                            current_user_id,
                            current_time,
                            data["delay"],
                            data["qty_min"],
                            data["date_start"],
                            data["date_end"],
                            inverse_company_rate,
                        ),
                    )

        # Commit the changes
        self.env.cr.commit()

    def _calculate_price(self, price_list_row):
        """Calculate the price based on the price list rules."""
        cost = (
            self.env["product.product"]
            .browse(price_list_row["product_id"])
            .standard_price
        )
        discount = price_list_row["price_discount"]
        extra_fee = price_list_row["price_surcharge"]
        if discount is not None:
            if extra_fee is None:
                extra_fee = 0
            if discount > 0:
                return cost * (1 - abs(discount) / 100) + extra_fee
            else:
                return cost * (1 + abs(discount) / 100) + extra_fee
        else:
            return cost

    def action_update_franchise_prices(self):
        """Update the price list for the franchise"""
        if self.valid_for is None or not self.valid_for:
            return {
                "type": "ir.actions.client",
                "tag": "display_notification",
                "params": {
                    "title": "Error",
                    "message": "No se puede actualizar la lista de precios, debe seleccionar Sucursal.",
                    "sticky": False,
                },
            }
        self.ensure_one()
        self.update_supplier_info()
        return {
            "type": "ir.actions.client",
            "tag": "display_notification",
            "params": {
                "title": "Éxito",
                "message": "Los precios de la franquicia se han actualizado.",
                "sticky": False,
            },
        }
