"""This module contains the definition of the ProductSupplierinfo model."""
import logging
from datetime import date, timedelta

from odoo import _, api, fields, models
from odoo.exceptions import UserError, ValidationError

_logger = logging.getLogger(__name__)


class ProductSupplierinfo(models.Model):
    """This class extends the product.supplierinfo model to add additional fields and methods."""

    _inherit = "product.supplierinfo"

    vendor_price = fields.Float(string="Price vendor")
    dis_1 = fields.Float(string="Dto 1", default=0)
    dis_2 = fields.Float(string="Dto 2", default=0)
    dis_3 = fields.Float(string="Dto 3", default=0)
    dis_inc_SIA = fields.Float(string="Inc/Dto SIA", default=0)
    dis_franchise_SIA = fields.Float(string="SIA Discount from this franchise", default=0)
    price_list = fields.Float(string="Price List")
    product_tmpl_id = fields.Many2one(
        "product.template", string="Producto", required=True, nullable=False
    )
    brand_name = fields.Char(
        related="product_tmpl_id.tmpl_brand_id.name",
        string="Brand",
        store=True,
        readonly=True,
    )
    family_name = fields.Char(
        related="product_tmpl_id.family_id.name",
        string="Category",
        store=True,
        readonly=True,
    )
    subfamily_name = fields.Char(
        related="product_tmpl_id.subfamily_id.name",
        string="SubItem",
        store=True,
        readonly=True,
    )
    price = fields.Float(string="Precio", compute="_compute_price", store=True)
    inverse_company_rate = fields.Float(
        string="Exchange rate",
        compute="_compute_inverse_company_rate",
        store=True,
        readonly=True,
    )
    ars_price = fields.Float(
        string="Price ($)", compute="_compute_ars_price", store=True, readonly=True
    )
    determine_cost = fields.Boolean(
        string="Determine cost",
        default=False,
        help="If true, this record will determine the cost of the product in the product form",
    )

    @api.onchange("determine_cost")
    def _onchange_determine_cost(self):
        """Set the determine_cost field to False for other records with the same product and date range."""
        if self.determine_cost:
            # Define the domain to search for overlapping records with the same product or variant
            if isinstance(self.id, int):
                # Define the domain to search for overlapping records with the same product or variant
                self._set_other_records_false(self)
            else:
                raise ValidationError(_("Cannot create/modify/delete from this view."))

    def _set_other_records_false(self, record):
        """Set the determine_cost field to False for other records with the same product and date range."""
        domain = [
            ("id", "!=", record.id),
            ("company_id", "=", record.company_id.id),
            ("determine_cost", "=", True),
            ("product_tmpl_id", "=", record.product_tmpl_id.id),
        ]
        other_records = self.search(domain)
        other_records.write({"determine_cost": False})

    def _check_unique_determine_cost(self):
        """Check if there is another record that determines the cost for the same product in the same date range."""
        for record in self:
            if record.determine_cost:
                domain = [
                    ("id", "!=", record.id),
                    ("product_id", "=", record.product_id.id),
                    ("date_start", "<=", record.date_end),
                    ("date_end", ">=", record.date_start),
                    ("determine_cost", "=", True),
                ]
                if not record.product_id:
                    domain[1] = ("product_tmpl_id", "=", record.product_tmpl_id.id)
                # Skip the check if the record is being created from the wizard
                if self.env.context.get("creating_from_wizard"):
                    continue
                if self.search_count(domain) > 0:
                    raise ValidationError(
                        _(
                            "here is already a record that determines the cost for this product in the same validity period.."
                        )
                    )

    @api.depends("currency_id", "currency_id.rate_ids", "date_start", "date_end")
    def _compute_inverse_company_rate(self):
        """
        Compute the inverse company rate based on the currency, start date, and end date.

        This method is triggered whenever the currency, start date, or end date is changed.
        It calculates the inverse company rate for the record based on the following logic:
        - If the currency is 'ARS', the inverse company rate is set to 1.0.
        - Otherwise, it searches for the most recent currency rate within the specified date range.
            If a rate is found, the inverse company rate is set to the rate's inverse company rate.
            If no rate is found, the inverse company rate is set to 0.0 and an error is raised.

        :raises UserError: If no valid currency rate is found within the specified date range.
        """
        for record in self:
            if record.currency_id.name == "ARS":
                record.inverse_company_rate = 1.0
            else:
                domain = [
                    ("currency_id", "=", record.currency_id.id),
                ]
                if record.date_start:
                    domain.append(("name", ">=", record.date_start))
                if record.date_end:
                    domain.append(("name", "<=", record.date_end))

                rate = self.env["res.currency.rate"].search(
                    domain, order="name desc", limit=1
                )

                if rate:
                    record.inverse_company_rate = rate.inverse_company_rate
                else:
                    record.inverse_company_rate = 0.0
                    # Verify if the module is being installed or updated
                    if not self.env.registry._init:
                        if record.date_start and record.date_end:
                            raise UserError(
                                _(
                                    "There is no valid exchange rate for the currency %s within your date range"
                                )
                                % record.currency_id.name
                            )
                        elif record.date_start:
                            raise UserError(
                                _(
                                    "Check that the Exchange Rate %s for this record is valid"
                                )
                                % record.currency_id.name
                            )

    @api.depends("price", "inverse_company_rate")
    def _compute_ars_price(self):
        """
        Compute the ARS price based on the price and inverse company rate.

        This method calculates the ARS price by multiplying the price of the record
        with the inverse company rate. The result is stored in the 'ars_price' field
        of the record.

        :param self: The current record(s) on which the computation is performed.
        """
        for record in self:
            purchase_unit = record.product_tmpl_id.uom_po_id
            sale_unit = record.product_tmpl_id.uom_id
            price = record.price
            if purchase_unit != sale_unit:
                price = price / purchase_unit.factor_inv * sale_unit.factor_inv

            record.ars_price = price * record.inverse_company_rate
        for record in self.filtered(lambda r: r.determine_cost == True):
            record._update_product_cost()

    @api.constrains("dis_1", "dis_2", "dis_3")
    def _check_discounts(self):
        """Check if the discounts are negative."""
        for record in self:
            if any(
                discount < 0 for discount in [record.dis_1, record.dis_2, record.dis_3]
            ):
                raise ValidationError(_("Discounts cannot be negative."))

    @api.onchange("vendor_price", "dis_1", "dis_2", "dis_3", "dis_inc_SIA", "dis_franchise_SIA")
    def _onchange_discounts(self):
        """Recalculate the price when the vendor price or discounts change."""
        self._compute_price()

    @api.depends("vendor_price", "dis_1", "dis_2", "dis_3", "dis_inc_SIA", "dis_franchise_SIA")
    def _compute_price(self):
        """Compute the price based on the vendor price and discounts."""
        is_confirming_purchase = self.env.context.get(
            "confirming_purchase_order", False
        )
        # Only perform the validation if we're not confirming a purchase order
        if not is_confirming_purchase:
            dis_inc_SIA = 0
            for record in self.filtered(lambda r: r.vendor_price > 0):
                if record.id:
                    main_company = self.env["res.company"].search(
                        [("is_main_company", "=", True)], limit=1
                    )
                    if record.partner_id == main_company.partner_id:
                        record.dis_franchise_SIA = self._calculate_vendor_discount(self.company_id, self.product_id)
                        main_company_record = self.env["product.supplierinfo"].sudo().search(
                            [
                                ("product_tmpl_id", "=", record.product_tmpl_id.id),
                                ("company_id", "=", main_company.id),
                                ("determine_cost", "=", True),
                            ]
                        )
                        if main_company_record:
                            record.vendor_price = main_company_record.ars_price * (1 - record.dis_franchise_SIA / 100.00)
                    price = record.vendor_price
                    discounts = [record.dis_1, record.dis_2, record.dis_3]
                    for discount in discounts:
                        if discount > 0:
                            discount = discount * 100
                            price = price * (1 - discount / 100)
                    if record.dis_inc_SIA != 0:
                        dis_inc_SIA = record.dis_inc_SIA * 100
                        price = price * (1 + dis_inc_SIA / 100.0)
                    if price <= 0:
                        raise ValidationError(
                            _(
                                "The final price after applying discounts cannot be zero or negative."
                            )
                        )
                    record.price = round(price, 2)
                else:
                    record.price = record.vendor_price

    @api.constrains("vendor_price")
    def _check_vendor_price(self):
        """Check if the vendor price is positive."""
        if not self.env.context.get("params", {}).get("model") == "purchase.order":
            for record in self:
                if record.vendor_price <= 0:
                    raise ValidationError(
                        _("The list price cannot be zero or negative.")
                    )

    @api.model
    def create(self, vals):
        """Override the create method to compute the price."""
        try:
            record = super(ProductSupplierinfo, self).create(vals)
            record._compute_price()
            record._check_unique_determine_cost()
            if self.env.context.get("creating_from_wizard") != True:
                if record.determine_cost:
                    self._set_other_records_false(record)
                    self._create_or_update_historical_record(record)
                    record._update_product_cost()
            return record
        except Exception as e:
            raise

    def write(self, vals):
        """Override the write method to compute the price."""
        if not self.env.context.get("cron_pricelist"):
            if not self.env.context.get("recalculating_price"):
                try:
                    result = super(ProductSupplierinfo, self).write(vals)
                    self.with_context(recalculating_price=True)._compute_price()
                    if "determine_cost" in vals and vals["determine_cost"]:
                        for record in self:
                            self._set_other_records_false(record)
                            record._update_product_cost()
                            if record.company_id.is_main_company:
                                    self.update_child_pricelists(record)
                    if self.env.context.get("creating_from_wizard") != True:
                        pre_write_determine_cost = {
                            record.id: record.determine_cost for record in self
                        }
                        for record in self:
                            if (
                                pre_write_determine_cost.get(record.id)
                                or record.determine_cost
                            ):
                                self._create_or_update_historical_record(record)
                                record._update_product_cost()
                                if record.company_id.is_main_company:
                                    self.update_child_pricelists(record)
                    return result
                except Exception as e:
                    raise
            else:
                return super(ProductSupplierinfo, self).write(vals)

    def update_child_pricelists(self, record):
        child_companies = self.env["res.company"].search([('id', '!=', record.company_id.id)])
        for company_id in child_companies:
            company = self.with_company(company_id)
            child_pricelists = company.env["product.supplierinfo"].search(
                [
                    ("product_tmpl_id", "=", record.product_tmpl_id.id),
                    ("company_id", "=", company_id.id),
                    ("partner_id", "=", record.company_id.partner_id.id),
                ]
            )
            for rec in child_pricelists: 
                rec._update_child_pricelists(record)
            if not child_pricelists:
                determine_cost = False
                d_cost_pl = company.env["product.supplierinfo"].search(
                    [
                        ("product_tmpl_id", "=", record.product_tmpl_id.id),
                        ("company_id", "=", company_id.id),
                        ("determine_cost", "=", True)
                    ]
                )
                if not d_cost_pl:
                    determine_cost = True
                new_pl = child_pricelists._create_child_pricelists(record, company_id, determine_cost)

    def _update_child_pricelists(self, parent):
        """Update the child pricelists when the parent pricelist is updated."""
        for record in self.with_context(visible_product_tmpl_id=False):
            record.update({
                "date_start": parent.date_start,
                "date_end": parent.date_end,
                "dis_1": 0,
                "dis_2": 0,
                "dis_3": 0,
                "dis_inc_SIA": 0,
                "price_list": parent.price_list,
                "vendor_price": parent.vendor_price,
                "determine_cost": True,
            })

    def _create_child_pricelists(self, parent, company_id, determine_cost=False):
        """Create the child pricelists when the parent pricelist is updated."""
        vals = {
            "product_tmpl_id": parent.product_tmpl_id.id,
            "product_id": parent.product_id.id,
            "date_start": parent.date_start,
            "date_end": parent.date_end,
            "ars_price": parent.ars_price,
            "price": parent.price,
            "vendor_price": parent.vendor_price,
            "dis_1": 0,
            "dis_2": 0,
            "dis_3": 0,
            "dis_inc_SIA": 0,
            "price_list": parent.price_list,
            "partner_id": parent.company_id.partner_id.id,
            "company_id": company_id.id,
            "determine_cost": True,
            #"determine_cost": determine_cost,
        }
        
        res = super(ProductSupplierinfo, self.with_context(visible_product_tmpl_id=False)).create(vals)
        
        return res
    
    def _calculate_vendor_discount(self, company_id, product_id):
        """Calculate the vendor price based on the product and company."""
        child_pricelists = self.env["product.pricelist"].search(
            [
                ("is_franchise_list", "=", True),
                ("valid_for", "=", company_id.id),
            ]
        )
        product_discount = self.env["product.pricelist.item"].search(
            [
                ("pricelist_id", "=", child_pricelists.id),
                ("product_id", "=", product_id.id),
            ]
        )
        all_products_discount = self.env["product.pricelist.item"].search(
            [
                ("pricelist_id", "=", child_pricelists.id),
                ("applied_on", "=", "3_global"),
            ],
            order="date_start desc",
            limit=1
        )
        if not all_products_discount and not product_discount:
            return 0
        if not all_products_discount:
            vendor_discount = product_discount.price_discount
        elif not product_discount:
            vendor_discount = all_products_discount.price_discount
        elif not all_products_discount.date_start:
            vendor_discount = product_discount.price_discount
        elif not product_discount.date_start:
            vendor_discount = all_products_discount.price_discount
        elif all_products_discount.date_start > product_discount.date_start:
            vendor_discount = all_products_discount.price_discount
        else:
            vendor_discount = product_discount.price_discount
        return vendor_discount
    
    def _recompute_price(self, parent, discount):
        vals = {}
        vals['vendor_price'] = parent.vendor_price * (1 + discount / 100.00)
        vals['price'] = vals['vendor_price'] * (1 - parent.dis_1) * (1 - parent.dis_2) * (1 - parent.dis_3) * (1 + parent.dis_inc_SIA)
        vals['ars_price'] = vals['price'] * parent.inverse_company_rate
        return vals
    
    def unlink(self):
        """Override the unlink method to prevent deletion from certain views."""
        if self.env.context.get("visible_product_tmpl_id") != False:
            raise ValidationError(_("Cannot delete from this view."))
        messages = self.env["mail.message"].search(
            [("model", "=", self._name), ("res_id", "in", self.ids)]
        )
        for message in messages:
            message.unlink()
        return super(ProductSupplierinfo, self).unlink()

    def action_mod_price_list(self):
        """Perform an action to modify the price list."""
        tomorrow = fields.Date.today() + timedelta(days=1)
        return {
            "name": _("Modificar Precio de Lista"),
            "type": "ir.actions.act_window",
            "res_model": "purchase.update.price",
            "view_mode": "form",
            "view_id": self.env.ref("nybble_purchase.view_mod_price_list_popup").id,
            "target": "new",
            "context": {
                "default_new_start_date": tomorrow,
            },
        }

    def _create_or_update_historical_record(self, record):
        """Create or update a historical record for the product supplier information"""
        HistoricalModel = self.env["product.supplierinfo.historical"]
        historical_record = HistoricalModel.search(
            [("supplierinfo_id", "=", record.id)], limit=1
        )
        values = {
            "product_tmpl_id": record.product_tmpl_id.id,
            "product_id": record.product_id.id,
            "partner_id": record.partner_id.id,
            "company_id": record.company_id.id,
            "currency_id": record.currency_id.id,
            "date_start": record.date_start,
            "date_end": record.date_end,
            "vendor_price": record.vendor_price,
            "price": record.price,
            "inverse_company_rate": record.inverse_company_rate,
            "ars_price": record.ars_price,
            "supplierinfo_id": record.id,
        }
        if self.env.context.get("creating_from_wizard") != True:
            if historical_record:
                historical_record.write(values)
            else:
                HistoricalModel.create(values)

    def notify_users(self, msg, subject):
        """Send a notification to the users defined in the configuration parameters."""
        try:
            user_ids = (
                self.env["ir.config_parameter"]
                .sudo()
                .get_param("nybble_purchase.cost_determination_users")
            )
            users = self.env["res.users"].browse(
                [int(id) for id in user_ids.split(",")] if user_ids else []
            )
            partners = users.mapped("partner_id")
            if partners:
                # Crear el mensaje
                message = self.env["mail.message"].create(
                    {
                        "message_type": "notification",
                        "body": msg,
                        "subject": subject,
                        "model": self._name,
                        "res_id": self.id,
                        "author_id": self.env.user.partner_id.id,
                        "partner_ids": [(6, 0, partners.ids)],
                    }
                )
                # Crear notificaciones para cada partner
                notifications = []
                for partner in partners:
                    notifications.append(
                        {
                            "mail_message_id": message.id,
                            "res_partner_id": partner.id,
                            "notification_type": "inbox",
                        }
                    )
                if notifications:
                    self.env["mail.notification"].create(notifications)
                # Notificar a los partners
                self.env["bus.bus"]._sendone(
                    self.env.user.partner_id,
                    "mail.message/inbox",
                    {
                        "type": "mail.message/inbox",
                        "payload": {
                            "message_id": message.id,
                            "model": self._name,
                            "res_id": self.id,
                        },
                    },
                )
                _logger.info(
                    f"Cost Determination: Notification sent to {partners.mapped('name')}"
                )
            else:
                _logger.warning("Cost Determination: No users found to notify")
        except Exception as e:
            _logger.warning(
                f"Cost Determination: Could not send notification to user(s): {e}"
            )
        return True

    @api.model
    def _cron_check_expired_cost_determination(self):
        """Check for expired cost determination records and notify the users."""
        today = date.today()
        expired_records = self.search(
            [("determine_cost", "=", True), ("date_end", "<", today)]
        )
        if expired_records:
            product_names = expired_records.mapped("product_tmpl_id.name")
            message = _(
                "Las siguientes variantes de producto/productos determinan costo y su período de validez ha finalizado."
                "Por favor modifique desde Tarifas de Proveedores:\n\n{}"
            ).format("\n".join(product_names))
            subject = _("Tarifas con determina costo vencidas")
            expired_records[0].notify_users(message, subject)

    @api.model
    def _cron_update_product_costs(self):
        """Update the cost of the products based on the supplier information."""
        supplier_infos = self.search([("determine_cost", "=", True)])
        for supplier_info in supplier_infos:
            supplier_info._update_product_cost()

    def _update_related_kit_cost(self, products):
        boms = (
            self.env["mrp.bom.line"]
            .search([("product_id", "in", products.ids)])
            .mapped("bom_id")
            .filtered(lambda bom: bom.type == "phantom")
        )

        for bom in boms:
            bom.product_tmpl_id.action_bom_cost()

    def _update_product_cost(self):
        """Update the cost of the product based on the supplier information."""
        for record in self:
            if record.product_tmpl_id:
                if record.product_id:
                    # This is a variant-specific record
                    record.product_id.sudo().write({"standard_price": record.ars_price})
                    self._update_related_kit_cost(record.product_id.sudo())
                else:
                    # This is a template-level record
                    if record.has_variants():
                        # Update only variants that don't have their own supplier info
                        variants_without_info = (
                            record.product_tmpl_id.product_variant_ids.filtered(
                                lambda v: not v.seller_ids.filtered(
                                    lambda s: s.determine_cost
                                )
                            )
                        )
                        variants_without_info.sudo().write(
                            {"standard_price": record.ars_price}
                        )
                        self._update_related_kit_cost(variants_without_info.sudo())
                    else:
                        record.product_tmpl_id.sudo().write(
                            {"standard_price": record.ars_price}
                        )
                        product = (
                            self.env["product.product"]
                            .sudo()
                            .search(
                                [("product_tmpl_id", "=", record.product_tmpl_id.id)]
                            )
                        )
                        self._update_related_kit_cost(product)

    def has_variants(self):
        """Check if the product template has real variants"""
        self.ensure_one()
        if not self.product_tmpl_id.attribute_line_ids:
            return False
        lines_with_multiple_values = sum(
            1
            for line in self.product_tmpl_id.attribute_line_ids
            if len(line.value_ids) > 1
        )
        return lines_with_multiple_values > 0


class ProductSupplierinfoHistorical(models.Model):
    """This class defines the historical records of the ProductSupplierinfo model."""

    _name = "product.supplierinfo.historical"
    _description = "Historical supplier rates"

    product_tmpl_id = fields.Many2one(
        "product.template", string="Product Template", readonly=True
    )
    product_id = fields.Many2one("product.product", string="Product", readonly=True)
    partner_id = fields.Many2one("res.partner", string="Purchase", readonly=True)
    company_id = fields.Many2one("res.company", string="Company", readonly=True)
    currency_id = fields.Many2one("res.currency", string="Monney", readonly=True)
    date_start = fields.Date(string="Start date", readonly=True)
    date_end = fields.Date(string="End date", readonly=True)
    vendor_price = fields.Float(string="Price list", readonly=True)
    price = fields.Float(string="Price", readonly=True)
    inverse_company_rate = fields.Float(string="Tasa de cambio", readonly=True)
    ars_price = fields.Float(string="Price ($)", readonly=True)
    supplierinfo_id = fields.Many2one(
        "product.supplierinfo", string="Supplier fee", readonly=True
    )


class PurchaseOrder(models.Model):
    """This class extends the purchase.order model to add additional fields and methods."""

    _inherit = "purchase.order"

    def button_confirm(self):
        """Override the button_confirm method to add the supplier to the product."""
        self = self.with_context(confirming_purchase_order=True)
        return super(PurchaseOrder, self).button_confirm()
