"""Tests for the purchase update price wizard."""
from datetime import timedelta

from odoo import fields
from odoo.exceptions import UserError
from odoo.tests.common import TransactionCase


class TestPurchaseUpdatePrice(TransactionCase):
    """Tests for the purchase update price wizard."""

    def setUp(self):
        """Set up the test environment."""
        super().setUp()
        self.wizard = self.env["purchase.update.price"].create(
            {
                "mod_per": 10.0,
                "new_start_date": fields.Date.today(),
                "finish_date": fields.Date.today() + timedelta(days=1),
            }
        )
        self.product = self.env["product.template"].create({"name": "Test Product"})
        self.partner = self.env["res.partner"].create({"name": "Test Partner"})

    def test_default_get(self):
        """Test the default_get method."""
        defaults = self.wizard.default_get([])
        self.assertEqual(defaults["new_start_date"], fields.Date.today())
        self.assertEqual(
            defaults["finish_date"], fields.Date.today() + timedelta(days=1)
        )

    def test_check_dates(self):
        """Test the _check_dates method."""
        with self.assertRaises(UserError):
            self.wizard.write(
                {
                    "new_start_date": fields.Date.today(),
                    "finish_date": fields.Date.today() - timedelta(days=1),
                }
            )
            self.wizard._check_dates()

        with self.assertRaises(UserError):
            self.wizard.write(
                {
                    "new_start_date": fields.Date.today(),
                    "finish_date": fields.Date.today(),
                }
            )
            self.wizard._check_dates()

        self.wizard.write(
            {
                "new_start_date": fields.Date.today(),
                "finish_date": fields.Date.today() + timedelta(days=1),
            }
        )
        self.wizard._check_dates()

    def test_check_overlap(self):
        """Test the _check_overlap method."""
        period1 = self.env["product.supplierinfo"].create(
            {
                "product_tmpl_id": self.product.id,
                "partner_id": self.partner.id,
                "date_start": fields.Date.today(),
                "date_end": fields.Date.today() + timedelta(days=5),
            }
        )

        period2 = self.env["product.supplierinfo"].create(
            {
                "product_tmpl_id": self.product.id,
                "partner_id": self.partner.id,
                "date_start": fields.Date.today() + timedelta(days=6),
                "date_end": fields.Date.today() + timedelta(days=10),
            }
        )
        self.assertFalse(self.wizard._check_overlap(period1, period2))

        period3 = self.env["product.supplierinfo"].create(
            {
                "product_tmpl_id": self.product.id,
                "partner_id": self.partner.id,
                "date_start": fields.Date.today() + timedelta(days=3),
                "date_end": fields.Date.today() + timedelta(days=7),
            }
        )
        self.assertTrue(self.wizard._check_overlap(period1, period3))

        period4 = self.env["product.supplierinfo"].create(
            {
                "product_tmpl_id": self.product.id,
                "partner_id": self.partner.id,
                "date_start": fields.Date.today() + timedelta(days=5),
                "date_end": fields.Date.today() + timedelta(days=10),
            }
        )
        self.assertTrue(self.wizard._check_overlap(period1, period4))

    def test_validate_overlapping_periods(self):
        """Test the _validate_overlapping_periods method."""
        existing_info = self.env["product.supplierinfo"].create(
            {
                "product_tmpl_id": self.product.id,
                "partner_id": self.partner.id,
                "date_start": fields.Date.today(),
                "date_end": fields.Date.today() + timedelta(days=5),
            }
        )

        new_info = self.env["product.supplierinfo"].create(
            {
                "product_tmpl_id": self.product.id,
                "partner_id": self.partner.id,
                "date_start": fields.Date.today() + timedelta(days=3),
                "date_end": fields.Date.today() + timedelta(days=7),
            }
        )
        with self.assertRaises(UserError):
            self.wizard._validate_overlapping_periods(new_info)

        new_info.write(
            {
                "date_start": fields.Date.today() + timedelta(days=6),
                "date_end": fields.Date.today() + timedelta(days=10),
            }
        )
        self.wizard._validate_overlapping_periods(new_info)

    def test_action_apply_mod_price_list(self):
        """Test the action_apply_mod_price_list method."""
        supplierinfo = self.env["product.supplierinfo"].create(
            {
                "product_tmpl_id": self.product.id,
                "partner_id": self.partner.id,
                "vendor_price": 100.0,
            }
        )

        self.wizard.with_context(
            active_ids=[supplierinfo.id]
        ).action_apply_mod_price_list()

        new_supplierinfo = self.env["product.supplierinfo"].search(
            [], order="id desc", limit=1
        )
        self.assertEqual(new_supplierinfo.date_start, fields.Date.today())
        self.assertEqual(
            new_supplierinfo.date_end, fields.Date.today() + timedelta(days=1)
        )

        # Round the actual price to avoid floating-point precision issues
        self.assertEqual(round(new_supplierinfo.vendor_price, 2), 110.0)
        self.assertEqual(supplierinfo.date_end, fields.Date.today() - timedelta(days=1))

    def test_reorder_supplier_info(self):
        """Test the _reorder_supplier_info method."""
        self.env["product.supplierinfo"].create(
            [
                {
                    "product_tmpl_id": self.product.id,
                    "partner_id": self.partner.id,
                    "date_start": fields.Date.today(),
                    "sequence": 10,
                },
                {
                    "product_tmpl_id": self.product.id,
                    "partner_id": self.partner.id,
                    "date_start": fields.Date.today() + timedelta(days=1),
                    "sequence": 5,
                },
            ]
        )
        self.wizard._reorder_supplier_info()

        sorted_records = self.env["product.supplierinfo"].search([], order="sequence")
        self.assertEqual(sorted_records[0].sequence, 0)
        self.assertEqual(sorted_records[1].sequence, 1)
