"""
This module contains the definition of the PurchaseUpdatePrice wizard.

The PurchaseUpdatePrice wizard is used to update the prices of selected supplier information records.
"""
from datetime import timedelta

from odoo import _, api, fields, models
from odoo.exceptions import UserError


class PurchaseUpdatePrice(models.TransientModel):
    """
    Purchase Update Price Wizard.

    This wizard is used to update the price of selected supplier information records.
    It allows the user to specify an increment or discount percentage and a new start date.
    When the user clicks on the "Apply" button, the wizard applies the modified price list
    to the selected supplier information records.
    """

    _name = "purchase.update.price"
    _description = "Update Price Wizard"

    mod_per = fields.Float(string="Increment(+) / Discount(-) in %", required=True)
    new_start_date = fields.Date(
        string="Start date",
        required=True,
        default=fields.Date.today,
    )
    finish_date = fields.Date(
        string="End date",
        required=True,
        default=lambda self: fields.Date.today() + timedelta(days=1),
        help="Supplier rate end date.",
    )

    @api.model
    def default_get(self, fields_list):
        """Get the default values for the wizard."""
        defaults = super(PurchaseUpdatePrice, self).default_get(fields_list)
        defaults["new_start_date"] = fields.Date.today()
        defaults["finish_date"] = fields.Date.today() + timedelta(days=1)
        return defaults

    @api.constrains("finish_date", "new_start_date")
    def _check_dates(self):
        """Check that the finish date is greater than the start date."""
        for record in self:
            if record.finish_date <= record.new_start_date:
                raise UserError(_("The end date must be greater than the start date."))

    def _validate_overlapping_periods(self, supplierinfo):
        """Validate that the new period does not overlap with existing periods."""
        domain = [
            ("product_tmpl_id", "=", supplierinfo.product_tmpl_id.id),
            ("product_id", "=", supplierinfo.product_id.id),
            ("partner_id", "=", supplierinfo.partner_id.id),
            ("min_qty", "=", supplierinfo.min_qty),
            ("id", "!=", supplierinfo.id),
        ]
        existing_records = self.env["product.supplierinfo"].search(domain)
        for record in existing_records:
            if self._check_overlap(supplierinfo, record):
                raise UserError(
                    _(
                        "No se puede crear el registro debido a que se superpone con un período existente."
                    )
                )

    def _check_overlap(self, period1, period2):
        """Check if two periods overlap."""
        if (
            not period1.date_start
            or not period1.date_end
            or not period2.date_start
            or not period2.date_end
        ):
            # If any of the dates are not set, assume they don't overlap
            return False
        # Check if period1 starts during period2
        if (
            period1.date_start <= period2.date_end
            and period1.date_start >= period2.date_start
        ):
            return True
        if (
            period1.date_end >= period2.date_start
            and period1.date_end <= period2.date_end
        ):
            return True
        # Check if period2 completely contains period1
        if (
            period1.date_start <= period2.date_start
            and period1.date_end >= period2.date_end
        ):
            return True
        return False

    def action_apply_mod_price_list(self):
        """Apply the modified price list to the selected supplier information records."""
        context = dict(self._context or {})
        active_ids = context.get("active_ids", [])
        supplierinfo_obj = self.env["product.supplierinfo"]
        if not active_ids:
            return {"type": "ir.actions.act_window_close"}
        for record in supplierinfo_obj.browse(active_ids):
            new_record = record.with_context(creating_from_wizard=True).copy()
            new_record.date_start = self.new_start_date
            new_record.date_end = self.finish_date
            new_record.determine_cost = False
            new_price = record.vendor_price * (1 + (self.mod_per / 100))
            new_record.vendor_price = new_price
            # Validate period overlap
            self._validate_overlapping_periods(new_record)
        # Sort the records
        self._reorder_supplier_info()
        return {"type": "ir.actions.act_window_close"}

    def _reorder_supplier_info(self):
        """Reorder the supplier information records."""
        query = """
            SELECT id, product_tmpl_id, product_id, partner_id, date_start
            FROM product_supplierinfo
            ORDER BY product_tmpl_id, product_id, partner_id,
                    COALESCE(date_start, '1900-01-01') DESC
        """
        self.env.cr.execute(query)
        records = self.env.cr.fetchall()

        case_statements = []
        ids_to_update = []
        for index, record in enumerate(records):
            if record[1] != index:  # Compare current sequence with new sequence
                case_statements.append(f"WHEN id = {record[0]} THEN {index}")
                ids_to_update.append(record[0])

            # Execute update in batches of 500
            if len(case_statements) == 500:
                case_query = f"""
                    UPDATE product_supplierinfo
                    SET sequence = CASE {' '.join(case_statements)} END
                    WHERE id IN ({', '.join(str(id) for id in ids_to_update)})
                """
                self.env.cr.execute(case_query)
                case_statements = []
                ids_to_update = []

        # Execute remaining updates
        if case_statements:
            case_query = f"""
                UPDATE product_supplierinfo
                SET sequence = CASE {' '.join(case_statements)} END
                WHERE id IN ({', '.join(str(id) for id in ids_to_update)})
            """
            self.env.cr.execute(case_query)
