"""Account Journal Report"""
import base64
import io
from datetime import datetime

import xlsxwriter

from odoo import _, api, fields, models
from odoo.exceptions import ValidationError


class JournalReport(models.Model):
    """Account Journal Report"""

    _name = "journal.report"
    _description = "Journal Report"

    @api.model
    def generate_report(self, initial_move_id):
        """Generate the journal report."""
        if not initial_move_id:
            raise ValidationError(_("No initial journal entry specified."))

        moves = self.env["account.move"].search(
            [
                ("state", "=", "posted"),
                ("id", ">=", initial_move_id),
                ("journal_id.provisional", "=", False),
            ],
            order="date, id",
        )

        if not moves:
            raise ValidationError(_("No journal entries found."))

        report_data = self._prepare_report_data(moves)

        return self.env.ref(
            "nybble_reports_dinamyc.action_report_journal_pdf"
        ).report_action(self, data={"report_data": report_data})

    @api.model
    def _prepare_report_data(self, moves):
        """Prepare the data for the journal report."""
        report_data = []
        for move in moves:
            move_data = {
                "entry_number": move.x_entry_number,
                "name": move.name,
                "date": move.date,
                "journal": move.journal_id.name,
                "lines": [
                    {
                        "account": line.account_id.name,
                        "label": line.name,
                        "debit": line.debit,
                        "credit": line.credit,
                        "taxes": ", ".join(line.tax_ids.mapped("name")),
                    }
                    for line in move.line_ids
                ],
                "taxes": [
                    {
                        "name": tax.name,
                        "base_amount": sum(
                            move.line_ids.filtered(lambda l: tax in l.tax_ids).mapped(
                                "balance"
                            )
                        ),
                        "tax_amount": sum(
                            move.line_ids.filtered(
                                lambda l: l.tax_line_id == tax
                            ).mapped("balance")
                        ),
                    }
                    for tax in move.line_ids.mapped("tax_ids")
                ],
            }
            report_data.append(move_data)
        return report_data

    @api.model
    def export_excel_report(self, initial_move_id):
        """Export the journal report to Excel."""
        if not initial_move_id:
            raise ValidationError(_("No initial journal entry specified."))

        moves = self.env["account.move"].search(
            [
                ("state", "=", "posted"),
                ("id", ">=", initial_move_id),
                ("journal_id.provisional", "=", False),
            ],
            order="date, id",
        )

        if not moves:
            raise ValidationError(_("No journal entries found."))

        report_data = self._prepare_report_data(moves)

        # Create Excel file
        output = io.BytesIO()
        workbook = xlsxwriter.Workbook(output, {"in_memory": True})
        worksheet = workbook.add_worksheet("Journal Report")

        # Add headers
        headers = [
            "Entry Number",
            "Name",
            "Date",
            "Journal",
            "Account",
            "Label",
            "Debit",
            "Credit",
            "Taxes",
        ]
        for col, header in enumerate(headers):
            worksheet.write(0, col, header)

        # Add data
        row = 1
        for move in report_data:
            for line in move["lines"]:
                worksheet.write(row, 0, move["entry_number"])
                worksheet.write(row, 1, move["name"])
                worksheet.write(row, 2, str(move["date"]))
                worksheet.write(row, 3, move["journal"])
                worksheet.write(row, 4, line["account"])
                worksheet.write(row, 5, line["label"])
                worksheet.write(row, 6, line["debit"])
                worksheet.write(row, 7, line["credit"])
                worksheet.write(row, 8, line["taxes"])
                row += 1

        workbook.close()
        output.seek(0)

        # Encode the Excel file for download
        excel_file = base64.b64encode(output.read())
        filename = f'journal_report_{datetime.now().strftime("%Y%m%d_%H%M%S")}.xlsx'

        attachment = self.env["ir.attachment"].create(
            {
                "name": filename,
                "datas": excel_file,
                "type": "binary",
            }
        )

        return {
            "type": "ir.actions.act_url",
            "url": f"/web/content/{attachment.id}?download=true",
            "target": "self",
        }


class JournalReportWizard(models.TransientModel):
    """Journal Report Wizard"""

    _name = "journal.report.wizard"
    _description = "Journal Report Wizard"

    date_start = fields.Date()
    date_end = fields.Date()
    initial_move_domain = fields.Many2many(
        "account.move", compute="_compute_initial_move_domain"
    )
    initial_move_id = fields.Many2one(
        "account.move",
        string="Initial Journal Entry",
        required=True,
    )

    @api.depends("date_start", "date_end")
    def _compute_initial_move_domain(self):
        for rec in self:
            domain = [("state", "=", "posted"), ("journal_id.provisional", "=", False)]
            if rec.date_start:
                domain.append(("date", ">=", rec.date_start))
            if rec.date_end:
                domain.append(("date", "<=", rec.date_end))
            self.initial_move_domain = self.env["account.move"].search(domain)

    def enumerate_entries(self):
        """Enumerate journal entries starting from the initial entry, resetting previous enumeration"""
        if not self.initial_move_id:
            raise ValidationError(_("Please select an initial journal entry."))

        # Get entries to enumerate
        moves = self.env["account.move"].search(
            [
                ("journal_id.provisional", "=", False),
                ("state", "=", "posted"),
                ("date", ">=", self.initial_move_id.date),
                "|",
                ("id", ">=", self.initial_move_id.id),
                ("date", ">", self.initial_move_id.date),
            ],
            order="date, id",
        )

        # Reset enumeration for all entries
        self.env["account.move"].search([]).write({"x_entry_number": False})

        # Enumerate selected entries starting from 1
        sequence = 1
        for move in moves:
            move.write({"x_entry_number": sequence})
            sequence += 1

        # Generate report data
        report_data = self.env["journal.report"]._prepare_report_data(moves)

        return {
            "type": "ir.actions.client",
            "name": "Journal Report",
            "tag": "journal_report",
            "params": {
                "initial_move_id": self.initial_move_id.id,
                "report_data": report_data,
            },
        }


class AccountMove(models.Model):
    """Inherit account.move to add new fields"""

    _inherit = "account.move"

    x_entry_number = fields.Integer(string="Entry Number", readonly=True, copy=False)
