"""This module contains the custom implementation of the `account.move` model."""
from odoo import models
import logging

_logger = logging.getLogger(__name__)

class AccountMove(models.Model):
    """
    Added to the invoice validation and cancellation flow the creation and
    deletion of AccountSalesAnalysis lines
    """

    _inherit = ["account.move"]

    def _create_sale_analysis_lines(self):
        """The AccountSalesAnalysis lines are created for each SALES invoice line."""
        sale_analysis_lines_vals = []
        for move in self:
            if move.is_sale_document(include_receipts=True):
                sale_analysis_lines_vals.extend(
                    move.invoice_line_ids.get_sale_analysis_lines()
                )
        if sale_analysis_lines_vals:
            self.env["account.sales.analysis"].create(sale_analysis_lines_vals)

    def _post(self, soft=True):
        """For the validated invoices, we proceed to the creation of the lines for the AccountSalesAnalysis"""
        posted = super(AccountMove, self)._post(soft)
        posted._create_sale_analysis_lines()
        return posted

    def button_draft(self):
        """If an invoice changes to Draft state, the lines for the AccountSalesAnalysis are deleted."""
        super(AccountMove, self).button_draft()
        for move in self:
            move.invoice_line_ids.clean_sale_analysis_lines()

    def button_cancel(self):
        """If an invoice changes to Canceled state, the lines for the AccountSalesAnalysis are deleted."""
        super(AccountMove, self).button_cancel()
        for move in self:
            move.invoice_line_ids.clean_sale_analysis_lines()

    def update_sale_analysis_lines(self):
        """If an invoice changes to Canceled state, the lines for the AccountSalesAnalysis are deleted."""
        sms = self.env['account.move'].search([('state', '=', 'posted')])
        for move in sms:
            if move.is_sale_document(include_receipts=True):
                    _logger.info('account_move_id executed: %s' %(move.id))
                    move._create_sale_analysis_lines()

    def delete_duplicated_sale_analysis_lines(self):
        analysis_records = self.env["account.sales.analysis"].search([])

        duplicates = {}
        for record in analysis_records:
            if record.move_line_id.id:
                if record.move_line_id.id in duplicates:
                    duplicates[record.move_line_id.id].append(record)
                else:
                    duplicates[record.move_line_id.id] = [record]

        for move_line_id, records in duplicates.items():
            if len(records) > 1:
                records_to_delete = records[1:]
                for record in records_to_delete:
                    record.unlink()
