"""This module contains the creation of new class `account.sales.analysis` model."""
from odoo import api, fields, models


class AccountSalesAnalysis(models.Model):
    """Aggregates detailed information for the generation of a Sales Analysis Report"""

    _name = "account.sales.analysis"
    _description = "Sales Analysis Report"

    brand_id = fields.Many2one("product.brand2")
    company_id = fields.Many2one("res.company", required=True)
    cost_currency = fields.Many2one("res.currency")
    date = fields.Date(required=True)
    default_code = fields.Char()
    document_type = fields.Char()
    family_id = fields.Many2one("product.family")
    historical_cost = fields.Float(required=True)
    inverse_company_rate = fields.Float()
    journal_id = fields.Many2one("account.journal")
    move_line_id = fields.Many2one(
        "account.move.line"
    )  # Guardamos relacion al modelo origen
    name = fields.Char()
    partner_id = fields.Many2one("res.partner", required=True)
    partner_city = fields.Char()
    price_unit = fields.Float(required=True)
    product_id = fields.Many2one("product.product", required=True)
    product_qty = fields.Float(required=True)
    sale_order_line_id = fields.Many2one(
        "sale.order.line"
    )  # Guardamos relacion al modelo origen
    team_id = fields.Many2one("crm.team", required=True)
    total_historical_cost = fields.Float(
        compute="_compute_total_historical_cost", store=True
    )
    total_replacement_cost = fields.Float(
        compute="_compute_total_replacement_cost", store=True
    )
    total_sale = fields.Float(compute="_compute_total_sale", store=True)
    user_id = fields.Many2one("res.users", required=True)
    utility_historical_amount = fields.Float(
        compute="_compute_utility_historical_amount"
    )
    utility_historical_cost_percentage = fields.Float(
        compute="_compute_utility_historical_cost_percentage"
    )
    utility_historical_sale_percentage = fields.Float(
        compute="_compute_utility_historical_sale_percentage"
    )
    utility_reposition_amount = fields.Float(
        compute="_compute_utility_reposition_amount"
    )
    utility_reposition_cost_percentage = fields.Float(
        compute="_compute_utility_reposition_cost_percentage"
    )
    utility_reposition_sale_percentage = fields.Float(
        compute="_compute_utility_reposition_sale_percentage"
    )
    replacement_cost = fields.Float(store=True, related="product_id.standard_price")
    subfamily_id = fields.Many2one("product.subfamily")

    @api.depends("total_sale", "total_historical_cost")
    def _compute_utility_historical_amount(self):
        for rec in self:
            rec.utility_historical_amount = rec.total_sale - rec.total_historical_cost

    @api.depends("total_historical_cost", "utility_historical_amount")
    def _compute_utility_historical_cost_percentage(self):
        for rec in self:
            rec.utility_historical_cost_percentage = 0.0
            if rec.total_historical_cost != 0.0:
                rec.utility_historical_cost_percentage = (
                    rec.utility_historical_amount / rec.total_historical_cost
                ) / 100

    @api.depends("utility_historical_amount", "total_sale")
    def _compute_utility_historical_sale_percentage(self):
        for rec in self:
            rec.utility_historical_sale_percentage = 0.0
            if rec.total_sale != 0.0:
                rec.utility_historical_sale_percentage = (
                    rec.utility_historical_amount / rec.total_sale
                ) / 100

    @api.depends("total_sale", "total_replacement_cost")
    def _compute_utility_reposition_amount(self):
        for rec in self:
            rec.utility_reposition_amount = rec.total_sale - rec.total_replacement_cost

    @api.depends("total_replacement_cost", "utility_reposition_amount")
    def _compute_utility_reposition_cost_percentage(self):
        for rec in self:
            rec.utility_reposition_cost_percentage = 0.0
            if rec.total_replacement_cost != 0.0:
                rec.utility_reposition_cost_percentage = (
                    rec.utility_reposition_amount / rec.total_replacement_cost
                ) * 100

    @api.depends("utility_reposition_amount", "total_sale")
    def _compute_utility_reposition_sale_percentage(self):
        for rec in self:
            rec.utility_reposition_sale_percentage = 0.0
            if rec.total_sale != 0.0:
                rec.utility_reposition_sale_percentage = (
                    rec.utility_reposition_amount / rec.total_sale
                ) * 100

    @api.depends("product_qty", "historical_cost")
    def _compute_total_historical_cost(self):
        for rec in self:
            rec.total_historical_cost = rec.product_qty * rec.historical_cost

    @api.depends("product_qty", "replacement_cost")
    def _compute_total_replacement_cost(self):
        for rec in self:
            rec.total_replacement_cost = rec.product_qty * rec.replacement_cost

    @api.depends("product_qty", "price_unit")
    def _compute_total_sale(self):
        for rec in self:
            rec.total_sale = rec.product_qty * rec.price_unit
