"""test_sale_order.py - Tests for the sale.order model."""
from odoo.tests.common import TransactionCase


class TestSaleOrder(TransactionCase):
    """Test cases for the sale.order model."""

    def setUp(self):
        """Create a customer, product, sale order, and lost reason."""
        super().setUp()
        # Create a customer
        self.customer = self.env["res.partner"].create(
            {
                "name": "Test Customer",
                "email": "test@example.com",
            }
        )
        # Create a product
        self.product = self.env["product.product"].create(
            {
                "name": "Test Product",
                "type": "consu",
                "list_price": 100.0,
            }
        )
        # Create a sale order
        self.sale_order = self.env["sale.order"].create(
            {
                "partner_id": self.customer.id,
                "order_line": [
                    (
                        0,
                        0,
                        {
                            "product_id": self.product.id,
                            "product_uom_qty": 1,
                            "price_unit": 100.0,
                        },
                    )
                ],
            }
        )
        # Create a lost reason
        self.lost_reason = self.env["sale.lost.reason"].create(
            {
                "name": "Test Lost Reason",
            }
        )

    def test_action_draft(self):
        """Test the action_draft method."""
        # Confirm the sale order
        self.sale_order.action_confirm()
        # Mark the sale order as lost
        wizard = self.env["sale.order.lost.reason"].create(
            {
                "order_id": self.sale_order.id,
                "lost_reason": self.lost_reason.id,
                "sale_lost_reason_obs": "Test observation",
            }
        )
        wizard.action_mark_as_lost()
        # Assert that the sale order is marked as lost
        self.assertTrue(self.sale_order.is_lost)
        self.assertEqual(self.sale_order.lost_reason, self.lost_reason)
        # Assert that a lost sale record is created
        lost_sales = self.env["sale.order.lost"].search(
            [("name", "=", self.sale_order.name)]
        )
        self.assertTrue(lost_sales)
        self.assertEqual(len(lost_sales), 1)
        # Call the action_draft method
        self.sale_order.action_draft()
        # Assert that the sale order is no longer marked as lost
        self.assertFalse(self.sale_order.is_lost)
        self.assertFalse(self.sale_order.lost_reason)
        # Assert that the related lost sale record is deleted
        lost_sales = self.env["sale.order.lost"].search(
            [("name", "=", self.sale_order.name)]
        )
        self.assertFalse(lost_sales)
        # Assert that the sale order state is 'draft'
        self.assertEqual(self.sale_order.state, "draft")

    def test_action_draft_not_lost(self):
        """Test the action_draft method on a sale order that is not lost."""
        # Test action_draft on a sale order that is not lost
        self.sale_order.action_confirm()
        self.sale_order.action_draft()
        # Assert that no error is raised and the state is 'draft'
        self.assertEqual(self.sale_order.state, "draft")

    def test_name_get(self):
        """Test the name_get method."""
        # Create a sale order line with a product having a default code
        product = self.env["product.product"].create(
            {
                "name": "Test Product",
                "default_code": "TEST",
            }
        )
        sale_order_line = self.env["sale.order.line"].create(
            {
                "product_id": product.id,
            }
        )
        # Call the name_get method
        name = sale_order_line.name_get()[0][1]
        # Assert that the name includes the default code
        self.assertEqual(name, "[TEST] Test Product")

    def test_action_mark_as_lost(self):
        """Test the action_mark_as_lost method."""
        # Create a sale order
        sale_order = self.env["sale.order"].create(
            {
                "partner_id": self.customer.id,
                "order_line": [
                    (
                        0,
                        0,
                        {
                            "product_id": self.product.id,
                            "product_uom_qty": 1,
                            "price_unit": 100.0,
                        },
                    )
                ],
            }
        )
        # Call the action_mark_as_lost method
        action = sale_order.action_mark_as_lost()
        # Assert that the action is of the correct type and model
        self.assertEqual(action["type"], "ir.actions.act_window")
        self.assertEqual(action["res_model"], "sale.order.lost.reason")
        # Assert that the default_order_id context is set correctly
        self.assertEqual(action["context"], {"default_order_id": sale_order.id})
